/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


%macro _bad_inputs_checking(_indash);

 * create output reporting dataset;
 data _error_checks;
  length _checktype $100 
         _error_flag 3
         _message $400;
  label _checktype = 'type of error to check'
        _error_flag = 'whether error occurred: 1=error 0=no error'
        _message = 'error message';
  stop;
 run;

 proc contents data=&_indash out=_contents_dashboard noprint;
 run;

 %global _maj_error;  
 %let _maj_error = 0; 
 *=========
 BLOCK A
 ===========;
 %macro _dashboard_col_check(_colname,_vtype);
  data _contents_&_colname;
   set _contents_dashboard;
   if name = "&_colname";   
  run;

  %_mtanyobs;
 
  %let _col_&_colname = &hasobs;
 
  %if &&&_col_&_colname = 1 %then %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set _contents_&_colname;
    if name = "&_colname" then do;
     if type = &_vtype then do;
      _checktype = "dashboard: column &_colname is properly formatted";
      _error_flag = 0;
      _message = "";
      output _errors;
     end;
     else if type ne &_vtype then do;
      _checktype = "dashboard: column &_colname is properly formatted";
      _error_flag = 1;
      _message = "ERROR: column &_colname is not formatted properly";
      output _errors;    
      call symput("_maj_error",1);
     end;
    end;
    keep _checktype _error_flag _message;
   run;
  %end;
  %else %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set _contents_dashboard;
    if _n_ = 1;
    if _n_ = 1 then do;    
     _checktype = "dashboard: column &_colname is properly formatted";
     _error_flag = 1;
     _message = "ERROR: column &_colname not detected";
     output _errors;    
     call symput("_maj_error",1);
    end;      
    keep _checktype _error_flag _message;
   run;
  %end;

  proc append data=_errors base=_error_checks force; 
  run;
 
  proc datasets library = work nolist;
   delete _errors _contents_&_colname;
  quit;  
 
 %mend _dashboard_col_check;
 
 %_dashboard_col_check(_type,2);
 %_dashboard_col_check(_variable,2);
 %_dashboard_col_check(_sas_data_name,2);
 %_dashboard_col_check(_sas_date_var,2);  
 %_dashboard_col_check(_sas_start_var,2);
 %_dashboard_col_check(_sas_end_var,2);  
 %_dashboard_col_check(_varfmt,2);
 %_dashboard_col_check(_varclass,2);
 %_dashboard_col_check(_value,2);
 %_dashboard_col_check(_behavior,1);
 %_dashboard_col_check(_no_acute_change,1);
 %_dashboard_col_check(_first_exp_rule,1);
 %_dashboard_col_check(_impute,1);
 

 * test dashboard dataset - necessary rows present;
 
 %macro _onerow(_dbdata,_targettype,_rownum);
  
  data &_targettype._row;
   set &_dbdata;
   if lowcase(_type) = "&_targettype" then output &_targettype._row;
  run;
  
  %_mtanyobs;
  
  %let _row_&_targettype._miss = &hasobs;
  
  %if &&&_row_&_targettype._miss = 0 %then %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set &_dbdata (keep = _type);
    if _n_ = 1;
    if _n_ = 1 then do;
     _checktype = "dashboard: &_targettype row present";
     _error_flag = 1;
     _message = "ERROR: no &_targettype row detected";
     output _errors;
     call symput("_maj_error",1);
    end;
    drop _type;
   run;
  %end;
  
  %else %if &&&_row_&_targettype._miss = 1 %then %do;
   data &_targettype._row;
    set &_targettype._row nobs=_obscounter;
    _obs_count = _obscounter;
   run;
  
   data _null_;
    set &_targettype._row;
    if _n_=1;
    call symput("_nobs_&_targettype._row",_obs_count);
   run;
   
   %if &&&_nobs_&_targettype._row = 1 %then %do;
    data _errors;
     length _checktype $100 
            _error_flag 3
            _message $400;
     set &_targettype._row (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: &_targettype row present";
      _error_flag = 0;
      _message = "";
      output _errors;
     end;
     drop _type;
    run;
   %end;
   
   %else %if &&&_nobs_&_targettype._row > 1  %then %do;
    data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
     set &_targettype._row (keep = _type);
     if _n_ = 1;
     %if &_rownum = 1 %then %do;
      if _n_ = 1 then do;
       _checktype = "dashboard: &_targettype row present";
       _error_flag = 1;
       _message = "ERROR: multiple &_targettype rows detected";
       output _errors;
       call symput("_maj_error",1);
      end;
     %end;
     %else %if &_rownum = 2 %then %do;
      if _n_ = 1 then do;
       _checktype = "dashboard: &_targettype row present";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
     %end;
     drop _type;
    run;
   %end;
  %end;
  
  proc append data=_errors base=_error_checks force; 
  run;
  
  proc datasets library = work nolist;
   delete &_targettype._row _errors;
  quit;
  
 %mend _onerow;
 
 %_onerow(&_indash,outdata,1);
 %_onerow(&_indash,exposure,1);
 %_onerow(&_indash,cohort,1);
 %_onerow(&_indash,studystart,1);
 %_onerow(&_indash,studyend,1);
 %_onerow(&_indash,eoftype,1);
 %_onerow(&_indash,studyid,1);
 %_onerow(&_indash,outcome,1);
 %_onerow(&_indash,unitdays,1);
 %_onerow(&_indash,harvardformat,1);
 %_onerow(&_indash,covariate,2);


*  check that _sas_data_name is populated and references an existing dataset;
 
 %macro _data_name_check(_dbdata,_targettype);
  
  data &_targettype._row;
   set &_dbdata;
   if lowcase(_type) = "&_targettype" then output &_targettype._row;
  run;
  
  %_mtanyobs;
  
  %let _row_&_targettype._miss = &hasobs;
  
  %if &&&_row_&_targettype._miss = 0 %then %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set &_dbdata (keep = _type);
    if _n_ = 1;
    if _n_ = 1 then do;
     _checktype = "dashboard: &_targettype row specifies a dataset";
     _error_flag = 1;
     _message = "ERROR: no &_targettype row detected";
     output _errors;
     call symput("_maj_error",1);
    end;
    drop _type;
   run;
  %end;
  
  %else %if &&&_row_&_targettype._miss = 1 %then %do;
   data &_targettype._row;
    set &_targettype._row nobs=_obscounter;
    _obs_count = _obscounter;
   run;
  
   data _null_;
    set &_targettype._row;
    if _n_=1;
    call symput("_nobs_&_targettype._row",_obs_count);
   run;
   
   %if &&&_nobs_&_targettype._row = 1 %then %do;
    data _null_;
     set &_targettype._row;
     if _n_=1;
     call symput("_data_&_targettype._row",lengthn(trim(left(_sas_data_name))));
    run;
  
    %put &&&_data_&_targettype._row;
    
    %if &&&_data_&_targettype._row = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_targettype._row (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: &_targettype row specifies a dataset";
       _error_flag = 1;
       _message = "ERROR: no dataset specified in _sas_data_name of &_targettype row";
       output _errors;
       call symput("_maj_error",1);
      end;
      drop _type;
     run;
    %end;
  
    %else %do;   
     data _null_;
      set &_targettype._row;
      if _n_=1;
      call symput("_file_&_targettype._row",trim(left(_sas_data_name)));
     run;
     
     %let _check_dset_&_targettype = %sysfunc(open(&&&_file_&_targettype._row,IS));
     %if &&&_check_dset_&_targettype = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._row (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype row specifies a dataset";
        _error_flag = 1;
        _message = "ERROR: dataset specified in _sas_data_name cannot be opened";
        output _errors;
        call symput("_maj_error",1);
       end;
       drop _type;
      run;
     %end;
     %else %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._row (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype row specifies a dataset";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
     %end;
    %end;  
   %end;
   
   %else %if &&&_nobs_&_targettype._row > 1  %then %do;
    data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
     set &_targettype._row (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: &_targettype row specifies a dataset";
      _error_flag = 1;
      _message = "ERROR: multiple &_targettype rows detected";
      output _errors;
      call symput("_maj_error",1);
     end;
     drop _type;
    run;
   %end;
  %end;
  
  proc append data=_errors base=_error_checks force; 
  run;
  
  proc datasets library = work nolist;
   delete &_targettype._row _errors;
  quit;
  
 %mend _data_name_check;
 
 %_data_name_check(&_indash,cohort);

 %_data_name_check(&_indash,studystart);
 

 %macro _dataset_vs_cohort(_dbdata,_targettype);
 
  data &_targettype._compare;
   set &_dbdata;
   if lowcase(_type) = "&_targettype" then output &_targettype._compare;
   else if lowcase(_type) = "cohort" then output &_targettype._compare;
  run;
  
  data _null_;
   set &_targettype._compare;
   if lowcase(_type) = "cohort" then do;
    call symput("_input_cohort_dset",trim(left(_sas_data_name)));
   end;
   else if lowcase(_type) = "&_targettype" then do;
    call symput("_input_&_targettype._dset",trim(left(_sas_data_name)));
   end;    
  run;
  
  %if &_input_cohort_dset = &&&_input_&_targettype._dset %then %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set &_targettype._compare (keep = _type);
    if _n_ = 1;
    if _n_ = 1 then do;
     _checktype = "dashboard: &_targettype _sas_data_name matches _sas_data_name in cohort row";
     _error_flag = 0;
     _message = "";
     output _errors;
    end;
    drop _type;
   run;
  %end;
  %else %do;
   data _errors;
    length _checktype $100 
           _error_flag 3
           _message $400;
    set &_targettype._compare (keep = _type);
    if _n_ = 1;
    if _n_ = 1 then do;
     _checktype = "dashboard: &_targettype _sas_data_name matches _sas_data_name in cohort row";
     _error_flag = 1;
     _message = "ERROR: _sas_data_name values for &_targettype and cohort rows are not equal";
     output _errors;
     call symput("_maj_error",1);
    end;
    drop _type;
   run;  
  %end;
 
  proc append data=_errors base=_error_checks force; 
  run;
  
  proc datasets library = work nolist;
   delete &_targettype._compare _errors;
  quit;
 
 %mend _dataset_vs_cohort;
 
 %_dataset_vs_cohort(&_indash,studystart);

 %_data_name_check(&_indash,studyend);
  
 %_dataset_vs_cohort(&_indash,studyend);

 %_data_name_check(&_indash,eoftype);

 %_dataset_vs_cohort(&_indash,eoftype);

 %_data_name_check(&_indash,studyid);

 %_dataset_vs_cohort(&_indash,studyid); 

 %_data_name_check(&_indash,outcome);
   
 %_dataset_vs_cohort(&_indash,outcome); 

   
 %put &_maj_error;

 %if &_maj_error = 1 %then %do;
  
  proc datasets library = work nolist;
   delete _errors;
  quit;   
   
 %end;
 %else %do;

*=========
 BLOCK B
==========;
 * add on 1/2016, the macro to check if _variable column contains strings longer than 20;
%macro _variable_length(_vname);

  data _contents_&_vname;
   set _contents_dashboard;
   if name = "&_vname";   
  run;

  data _null_;
      set _contents_&_vname;
      if length > 20 then do;
       call symput("_lcheck",0);
      end;
      else do;
       call symput("_lcheck",1);
      end;
     run;

 %if &_lcheck = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set _contents_&_vname (keep = type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable column contains character strings longer than 20 characters";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop type;
      run;
     %end;

    %if &_lcheck = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set _contents_&_vname (keep = type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_vname column contains character strings longer than 20 characters";
        _error_flag = 1;
        _message = "Error: &_vname column  of dashboard contains character strings longer than 20 characters";
        output _errors;
        call symput("_maj_error",2); 
       end;
       drop type;
      run;
    %end;
   

  proc append data=_errors base=_error_checks force; 
  run;
 
  proc datasets library = work nolist;
   delete _errors _contents_&_vname;
  quit;  
 
 %mend _variable_length;

 %_variable_length(_variable);

 %macro _datevar_vs_variable(_dbdata,_targettype);
  
   data &_targettype._compare;
    set &_dbdata;
    if lowcase(_type) = "&_targettype" then output &_targettype._compare;
   run;
   
   data _null_;
    set &_targettype._compare;
    call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
    call symput("_&_targettype._date",lowcase(trim(left(_sas_date_var))));
    call symput("_&_targettype._var",lowcase(trim(left(_variable))));
   run;
   
   %if &&&_&_targettype._var = &&&_&_targettype._date %then %do;
    
     *ods listing close;
    *ods output variables=_variables_&_targettype;
    proc contents data=&&&_&_targettype._sourcedata out=_variables_&_targettype noprint;
    run;
    *ods listing;

  data _variables_&_targettype;
  set _variables_&_targettype;
  length type_c $4.;
  if type=1 then type_c ='Num';
  if type=2 then type_c='Char';
  drop type;
   rename type_c=type;
  run;

    data _variables_&_targettype;
     set _variables_&_targettype;
     if lowcase(Name) = "&&&_&_targettype._date";
    run;
  
    %_mtanyobs;
   
    %let _var_&_targettype._date = &hasobs;
    
    %if &&&_var_&_targettype._date = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable equals _sas_date_var for row &_targettype";
       _error_flag = 1;
       _message = "ERROR: variable specified by _sas_date_var cannot be found in the cohort dataset";
       output _errors;
       call symput("_maj_error",2);
      end;
      drop _type;
     run;
    %end;
    %else %do;
     data _variables_&_targettype;
      set _variables_&_targettype;
      if upcase(type) in ('NUM') then do;
       call symput ("_error_date_&_targettype",0);
      end;
      else do;
       call symput ("_error_date_&_targettype",1);
      end;
     run;
     %if &&&_error_date_&_targettype = 0 %then %do;
      data __&_targettype._check_date;
       set &&&_&_targettype._sourcedata;
       _remtest = mod(&&&_&_targettype._var,1);
       if _remtest ne 0 then do;
        call symput ("_error_values_&_targettype",1);
       end;
       else do;
        call symput ("_error_values_&_targettype",0);
       end;
      run;
      %if &&&_error_values_&_targettype = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._compare (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _variable equals _sas_date_var for row &_targettype";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;
      %end;
      %if &&&_error_values_&_targettype = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._compare (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _variable equals _sas_date_var for row &_targettype";
         _error_flag = 1;
         _message = "ERROR: variable _sas_date_var in _sas_data_name contains non-integer or missing values";
         output _errors;
         call symput("_maj_error",2);
        end;
        drop _type;
       run; 
      %end;     
     %end;   
     %else %if &&&_error_date_&_targettype = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._compare (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable equals _sas_date_var for row &_targettype";
        _error_flag = 1;
        _message = "ERROR: variable specified by _sas_date_var is not numeric";
        output _errors;
        call symput("_maj_error",2);
       end;
       drop _type;
      run; 
     %end;
    %end;
   %end;
   %else %do;
    data _errors;
     length _checktype $100 
            _error_flag 3
            _message $400;
     set &_targettype._compare (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: _variable equals _sas_date_var for row &_targettype";
      _error_flag = 1;
      _message = "ERROR: values of _variable and _sas_date_var for &_targettype row are not equal";
      output _errors;
      call symput("_maj_error",2);     
     end;
     drop _type;
    run;  
   %end;
  
   proc append data=_errors base=_error_checks force; 
   run;
   
   proc datasets library = work nolist;
    delete &_targettype._compare _errors;
   quit;
  
  %mend _datevar_vs_variable;
  
  %_datevar_vs_variable(&_indash,studystart);
   
  %_datevar_vs_variable(&_indash,studyend);

  %macro _variable_blank(_dbdata,_targettype);
  
   data &_targettype._compare;
    set &_dbdata;
    if lowcase(_type) = "&_targettype" then output &_targettype._compare;
   run;
  
   data _null_;
    set &_targettype._compare;
    if _variable = '' then do;
     call symput("_&_targettype._varcheck",0);
    end;
    else do;
     call symput("_&_targettype._varcheck",1);
     call symput("_&_targettype._var",lowcase(trim(left(_variable))));
     call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
    end;
   run;
   
   %if &&&_&_targettype._varcheck = 1 %then %do;
    *ods listing close;
    *ods output variables=_variables_&_targettype;
    proc contents data=&&&_&_targettype._sourcedata out=_variables_&_targettype noprint;
    run;
    *ods listing;
  
  data _variables_&_targettype;
  set _variables_&_targettype;
  length type_c $4.;
  if type=1 then type_c ='Num';
  if type=2 then type_c='Char';
  drop type;
   rename type_c=type;
  run;



    data _variables_&_targettype;
     set _variables_&_targettype;
     if lowcase(Name) = "&&&_&_targettype._var";
    run;
  
    %_mtanyobs;
  
    %let _var_&_targettype._var = &hasobs;
    
    %if &&&_var_&_targettype._var = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
       _error_flag = 1;
       _message = "ERROR: _variable cannot be found in the dataset specified in _sas_data_name";
       output _errors;
       call symput("_maj_error",2); 
      end;
      drop _type;
     run;
    %end;
    %else %do;
     data _null_;
      set _variables_&_targettype;
      if length > 20 then do;
       call symput("_&_targettype._lcheck",0);
      end;
      else do;
       call symput("_&_targettype._lcheck",1);
      end;
     run;
     %if &&&_&_targettype._lcheck = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
     %end;
     %else %if &&&_&_targettype._lcheck = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
        _error_flag = 1;
        _message = "ERROR: _variable contains a value with length exceeding 20 characters";
        output _errors;
        call symput("_maj_error",2); 
       end;
       drop _type;
      run;
     %end;
    %end;
   %end;
   %else %if &&&_&_targettype._varcheck = 0 %then %do;
    data _errors;
     length _checktype $100 
            _error_flag 3
            _message $400;
     set &_dbdata (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
      _error_flag = 1;
      _message = "ERROR: no value specified in _variable of row &_targettype";
      output _errors;
      call symput("_maj_error",2); 
     end;
     drop _type;
    run;  
   %end;
    
   proc append data=_errors base=_error_checks force; 
   run;
   
   proc datasets library = work nolist;
    delete &_targettype._compare _errors;
   quit;
  
  %mend _variable_blank;
  
  %_variable_blank(&_indash,eoftype); 

  %_variable_blank(&_indash,studyid); 



  %macro _expos_id_check(_dbdata,_targettype);
  
   data &_targettype._row;
    set &_dbdata;
    if lowcase(_type) = "&_targettype" then output &_targettype._row;
   run;
   
   %_mtanyobs;
   
   %let _row_&_targettype._miss = &hasobs;
   
   %if &&&_row_&_targettype._miss = 0 %then %do;
    data _errors;
     length _checktype $100 
            _error_flag 3
            _message $400;
     set &_dbdata (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
      _error_flag = 1;
      _message = "ERROR: no &_targettype row detected";
      output _errors;
      call symput("_maj_error",2);
     end;
     drop _type;
    run;
   %end;
   
   %else %if &&&_row_&_targettype._miss = 1 %then %do;
    data &_targettype._row;
     set &_targettype._row nobs=_obscounter;
     _obs_count = _obscounter;
    run;
   
    data _null_;
     set &_targettype._row;
     if _n_=1;
     call symput("_nobs_&_targettype._row",_obs_count);
    run;
    
    %if &&&_nobs_&_targettype._row = 1 %then %do;
     data _null_;
      set &_targettype._row;
      if _n_=1;
      call symput("_data_&_targettype._row",lengthn(trim(left(_sas_data_name))));
     run;
   
     %put &&&_data_&_targettype._row;
     
     %if &&&_data_&_targettype._row = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._row (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
        _error_flag = 1;
        _message = "ERROR: no dataset specified in _sas_data_name of &_targettype row";
        output _errors;
        call symput("_maj_error",2);
       end;
       drop _type;
      run;
     %end;
   
     %else %do;   
      data _null_;
       set &_targettype._row;
       if _n_=1;
       call symput("_file_&_targettype._row",trim(left(_sas_data_name)));
      run;
      
      %let _check_dset_&_targettype = %sysfunc(open(&&&_file_&_targettype._row,IS));
      %if &&&_check_dset_&_targettype = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._row (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
         _error_flag = 1;
         _message = "ERROR: dataset specified in _sas_data_name cannot be opened";
         output _errors;
         call symput("_maj_error",2);
        end;
        drop _type;
       run;
      %end;
      %else %do;
      
       data _idtarget_row;
        set &_dbdata;
        if lowcase(_type) = "studyid" then output _idtarget_row;
       run;     
  
       data _null_;
        set _idtarget_row;
        if _n_=1;
        call symput("_id_var_target",lowcase(trim(left(_variable))));
       run;
       
      
       data &_targettype._ids_miss;
        set &&&_file_&_targettype._row;
        if &_id_var_target = '';
       run;
    
       %_mtanyobs;
   
       %let _missing_ids_&_targettype = &hasobs;     
      
       %if &&&_missing_ids_&_targettype = 1 %then %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_targettype._row (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
          _error_flag = 1;
          _message = "ERROR: dataset _sas_data_name has missing values in ID column specified by _studyid";
          output _errors;
          call symput("_maj_error",2);
         end;
         drop _type;
        run;      
       %end;
       %else %do;
        proc sort data=&&&_file_&_targettype._row out=_exposure_id_checking (keep = &_id_var_target) nodupkey;
         by &_id_var_target;
        run;
        
        data _cohort_target_row;
         set &_dbdata;
         if lowcase(_type) = "cohort" then output _cohort_target_row;
        run;     
  
        data _null_;
         set _cohort_target_row;
         if _n_=1;
         call symput("_cohort_target",trim(left(_sas_data_name)));
        run;
        
        data _cohort_id_testing;
         set &_cohort_target;
         keep &_id_var_target;
        run;
        
        proc sort data=_cohort_id_testing (keep = &_id_var_target) nodupkey;
         by &_id_var_target;
        run;
        
        data _exp_cohort_overlap _exp_ids_only _cohort_ids_only;
         merge _cohort_id_testing(in=a) _exposure_id_checking(in=b);
         by &_id_var_target;
         if a and b then output _exp_cohort_overlap;
         else if a and not b then output _cohort_ids_only;
         else if b and not a then output _exp_ids_only;
        run;
        
        data _exp_ids_only;
         set _exp_ids_only;
        run;
  
        %_mtanyobs;
    
        %let _exp_ids_only = &hasobs;          
        
        %if &_exp_ids_only = 0 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_targettype._row (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;
        %end;
        %else %if &_exp_ids_only = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_targettype._row (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
           _error_flag = 1;
           _message = "ERROR: exposure dataset _sas_data_name contains records for IDs not present in cohort dataset";
           output _errors;
           call symput("_maj_error",2);
          end;
          drop _type;
         run;       
        %end;
       %end;
      %end;
     %end;  
    %end;
    
    %else %if &&&_nobs_&_targettype._row > 1  %then %do;
     data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
      set &_targettype._row (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: exposure row specifies a dataset with patient IDs in cohort";
       _error_flag = 1;
       _message = "ERROR: multiple &_targettype rows detected";
       output _errors;
       call symput("_maj_error",2);
      end;
      drop _type;
     run;
    %end;
   %end;
   
   proc append data=_errors base=_error_checks force; 
   run;
   
   proc datasets library = work nolist;
    delete &_targettype._row _errors _exp_cohort_overlap _exp_ids_only _cohort_ids_only 
    _cohort_id_testing _exposure_id_checking _cohort_target_row;
   quit;
  
  %mend _expos_id_check;
  
  %_expos_id_check(&_indash,exposure); 


  %macro _expos_date_check(_dbdata,_targettype,_dtype);
  
   data &_targettype._row;
    set &_dbdata;
    if lowcase(_type) = "&_targettype" then output &_targettype._row;
   run;
   
   %_mtanyobs;
   
   %let _row_&_targettype._miss = &hasobs;
   
   %if &&&_row_&_targettype._miss = 0 %then %do;
    data _errors;
     length _checktype $100 
            _error_flag 3
            _message $400;
     set &_dbdata (keep = _type);
     if _n_ = 1;
     if _n_ = 1 then do;
      _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
      _error_flag = 1;
      _message = "ERROR: no &_targettype row detected";
      output _errors;
      call symput("_maj_error",2);
     end;
     drop _type;
    run;
   %end;
   
   %else %if &&&_row_&_targettype._miss = 1 %then %do;
    data &_targettype._row;
     set &_targettype._row nobs=_obscounter;
     _obs_count = _obscounter;
    run;
   
    data _null_;
     set &_targettype._row;
     if _n_=1;
     call symput("_nobs_&_targettype._row",_obs_count);
    run;
    
    %if &&&_nobs_&_targettype._row = 1 %then %do;
     data _null_;
      set &_targettype._row;
      if _n_=1;
      call symput("_&_dtype.date_length",lengthn(trim(left(_sas_&_dtype._var))));
      call symput("_&_dtype.date_exposure",lowcase(trim(left(_sas_&_dtype._var))));
     run;
   
     %put &&&_&_dtype.date_length;
     
     %if &&&_&_dtype.date_length = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._row (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
        _error_flag = 1;
        _message = "ERROR: no _sas_&_dtype._var variable specified in &_targettype row";
        output _errors;
        call symput("_maj_error",2);
       end;
       drop _type;
      run;
     %end;
   
     %else %do;   
      data _null_;
       set &_targettype._row;
       if _n_=1;
       call symput("_file_&_targettype._row",trim(left(_sas_data_name)));
      run;
      
      %let _check_dset_&_targettype = %sysfunc(open(&&&_file_&_targettype._row,IS));
      %if &&&_check_dset_&_targettype = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._row (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
         _error_flag = 1;
         _message = "ERROR: dataset specified in _sas_data_name cannot be opened";
         output _errors;
         call symput("_maj_error",2);
        end;
        drop _type;
       run;
      %end;
      
      %else %do;
      
       proc contents data=&&&_file_&_targettype._row out =_variables_&_dtype._date noprint;
       run;
      
  
      data _variables_&_dtype._date;
      set _variables_&_dtype._date;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
      run;

       data _variables_&_dtype._date;
        set _variables_&_dtype._date;
        if lowcase(Name) = "&&&_&_dtype.date_exposure";
       run;
       
       %_mtanyobs;
       
       %let _var_&_dtype._date = &hasobs;      
  
       %if &&&_var_&_dtype._date = 0 %then %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_dbdata (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
          _error_flag = 1;
          _message = "ERROR: _sas_&_dtype._var cannot be found in the exposure dataset specified by _sas_data_name";
          output _errors;
          call symput("_maj_error",2);
         end;
         drop _type;
        run;
       %end;
       %else %do;  
        data _variables_&_dtype._date;
         set _variables_&_dtype._date;
         if upcase(type) in ('NUM') then do;
          call symput ("_error_date_&_dtype",0);
         end;
         else do;
          call symput ("_error_date_&_dtype",1);
         end;
        run;
        %if &&&_error_date_&_dtype = 0 %then %do;
         data _&_dtype._check_date;
          set &&&_file_&_targettype._row;
          _remtest = mod(&&&_&_dtype.date_exposure,1);
          if _remtest ne 0 then do;
           call symput ("_error_values_&_dtype",1);
          end;
          else do;
           call symput ("_error_values_&_dtype",0);
          end;
         run;
         %if &&&_error_values_&_dtype = 0 %then %do;    
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_dbdata (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
            _error_flag = 0;
            _message = "";
            output _errors;
           end;
           drop _type;
          run;
         %end;
         %else %if &&&_error_values_&_dtype = 1 %then %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_dbdata (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
            _error_flag = 1;
            _message = "ERROR: variable specified by _sas_&_dtype._var contains non-integer or missing values";
            output _errors;
            call symput("_maj_error",2);
           end;
           drop _type;
          run; 
         %end;
        %end;
        %else %if &&&_error_date_&_dtype = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_dbdata (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
           _error_flag = 1;
           _message = "ERROR: variable specified by _sas_&_dtype._var is not numeric";
           output _errors;
           call symput("_maj_error",2);
          end;
          drop _type;
         run; 
        %end;     
       %end;   
      %end;
     %end;  
    %end;
    
    %else %if &&&_nobs_&_targettype._row > 1  %then %do;
     data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
      set &_targettype._row (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: exposure row specifies valid _sas_&_dtype._var variable";
       _error_flag = 1;
       _message = "ERROR: multiple &_targettype rows detected";
       output _errors;
       call symput("_maj_error",2);
      end;
      drop _type;
     run;
    %end;
   %end;
   
   proc append data=_errors base=_error_checks force; 
   run;
   
   proc datasets library = work nolist;
    delete &_targettype._row _errors _&_dtype._check_date _variables_&_dtype._date;
   quit;
  
  %mend _expos_date_check;
  
  %_expos_date_check(&_indash,exposure,start); 
  %_expos_date_check(&_indash,exposure,end); 
  
  %put &_maj_error;

  %if &_maj_error = 2 %then %do;
  
   proc datasets library = work nolist;
    delete _errors;
   quit;   
   
  %end;
  %else %do;
   
*=========
 BLOCK C
==========;
   
   %macro _varfmt_confirm(_dbdata,_targettype);
    data &_targettype._varfmt;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._varfmt;
    run; 
   
    data &_targettype._varfmt;
     set &_targettype._varfmt;
     if _varfmt in ('C','N') then do;
      call symput("_&_targettype._vfmtcheck",1);
      call symput("_&_targettype._var",lowcase(trim(left(_variable))));
      call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
      if _varfmt in ('C') then do;
       call symput("_&_targettype._vfmt_type",2);
      end;
      else if _varfmt in ('N') then do;
       call symput("_&_targettype._vfmt_type",1);    
      end;
     end;
     else do;
      call symput("_&_targettype._vfmtcheck",0);   
     end;
    run;
   
   
    %if &&&_&_targettype._vfmtcheck = 1 %then %do;
     *ods listing close;
     *ods output variables=_varfmt_&_targettype;
     proc contents data=&&&_&_targettype._sourcedata out=_varfmt_&_targettype noprint;
     run;
     *ods listing;
     
	  data _varfmt_&_targettype;
      set _varfmt_&_targettype;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
      run;


     data _varfmt_&_targettype;
      set _varfmt_&_targettype;
      if lowcase(Name) = "&&&_&_targettype._var";
     run;
     
     %if &&&_&_targettype._vfmt_type = 1 %then %do;
      data _varfmt_&_targettype;
       set _varfmt_&_targettype;
       if type = "Num" then do;
        call symput("_&_targettype._vfmtclear",1);        
       end;
       else do;
        call symput("_&_targettype._vfmtclear",0);       
       end;
      run;
      %if &&&_&_targettype._vfmtclear = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;      
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format";
         _error_flag = 1;
         _message = "ERROR: _varfmt in row &_targettype does not match format of variable given in _variable";
         output _errors;
        end;
        drop _type;
       run;  
      %end;
     %end;   
     %else %if &&&_&_targettype._vfmt_type = 2 %then %do;
      data _varfmt_&_targettype;
       set _varfmt_&_targettype;
       if type = "Char" then do;
        call symput("_&_targettype._vfmtclear",1);        
       end;
       else do;
        call symput("_&_targettype._vfmtclear",0);       
       end;
      run;
      %if &&&_&_targettype._vfmtclear = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;      
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format";
         _error_flag = 1;
         _message = "ERROR: _varfmt in row &_targettype does not match format of variable given in _variable";
         output _errors;
        end;
        drop _type;
       run;  
      %end;
     %end;   
    %end;
   
    %else %if &&&_&_targettype._vfmtcheck= 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format";
       _error_flag = 1;
       _message = "ERROR: _varfmt in row &_targettype is neither C nor N";
       output _errors;
      end;
      drop _type;
     run;  
    %end;  
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._varfmt _errors;
    quit;
   
   %mend _varfmt_confirm;
   
   %_varfmt_confirm(&_indash,eoftype);  
   
   
   
   %macro _value_confirm(_dbdata,_targettype);
   
    data &_targettype._value;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._value;
    run; 
   
    data _null_;
     set &_targettype._value;
     if _value = '' then do;
      call symput("_&_targettype._checkvalue",0);    
     end;
     else do;
      call symput("_&_targettype._checkvalue",1);    
      call symput("_&_targettype._var",lowcase(trim(left(_variable))));
      call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
      call symput("_&_targettype._value",trim(left(_value)));
     end;
    run; 
    
    %if &&&_&_targettype._checkvalue = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row found in cohort dataset";
       _error_flag = 1;
       _message = "ERROR: no value specified for _value";
       output _errors;
      end;
      drop _type;
     run;
    %end;
   
    %else %if &&&_&_targettype._checkvalue = 1 %then %do;
     *ods listing close;
     *ods output onewayfreqs=freqs_&_targettype;
      proc freq data=&&&_&_targettype._sourcedata noprint;
       tables &&&_&_targettype._var/out=freqs_&_targettype;
      run;
     *ods listing;

     
     data freqs_&_targettype;
      set freqs_&_targettype;
      if &&&_&_targettype._var = "&&&_&_targettype._value";
     run;
     
     %_mtanyobs;
     
     %let _value_&_targettype._var = &hasobs;
     
     %if &&&_value_&_targettype._var = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _value of &_targettype row found in cohort dataset";
        _error_flag = 1;
        _message = "ERROR: _value specified in &_targettype row not found in the EOF column of cohort dataset";
        output _errors;
       end;
       drop _type;
      run;
     %end;
     %else %do;
      *ods listing close;
      *ods output onewayfreqs=miss_&_targettype;
       proc freq data=&&&_&_targettype._sourcedata noprint;
        tables &&&_&_targettype._var /missing out=miss_&_targettype;
       run;
      *ods listing;
     
      data miss_&_targettype;
       set miss_&_targettype;
       if &&&_&_targettype._var = '';
      run; 
   
      %_mtanyobs;
      
      %let _miss_&_targettype._var = &hasobs;
   
      %if &&&_miss_&_targettype._var = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _value of &_targettype row found in cohort dataset";
         _error_flag = 1;
         _message = "ERROR: _variable specified in &_targettype row has missing values in cohort dataset";
         output _errors;
        end;
        drop _type;
       run;
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _value of &_targettype row found in cohort dataset";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;
      %end;
     %end;
    %end;
   
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._value freqs_&_targettype _errors;
    quit;
   
   
   %mend _value_confirm; 
   
   %_value_confirm(&_indash,eoftype); 



   %macro _output_loc_check(_dbdata,_targettype);
    
    data &_targettype._row;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._row;
    run;
    
    %_mtanyobs;
    
    %let _row_&_targettype._miss = &hasobs;
    
    %if &&&_row_&_targettype._miss = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: &_targettype row specifies a valid output location";
       _error_flag = 1;
       _message = "ERROR: no &_targettype row detected";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    
    %else %if &&&_row_&_targettype._miss = 1 %then %do;
     data &_targettype._row;
      set &_targettype._row nobs=_obscounter;
      _obs_count = _obscounter;
     run;
    
     data _null_;
      set &_targettype._row;
      if _n_=1;
      call symput("_nobs_&_targettype._row",_obs_count);
     run;
     
     %if &&&_nobs_&_targettype._row = 1 %then %do;
      data _null_;
       set &_targettype._row;
       if _n_=1;
       call symput("_data_&_targettype._row",lengthn(trim(left(_sas_data_name))));
      run;
    
      %put &&&_data_&_targettype._row;
      
      %if &&&_data_&_targettype._row = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._row (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: &_targettype row specifies a valid output location";
         _error_flag = 1;
         _message = "ERROR: no dataset specified in _sas_data_name of &_targettype row";
         output _errors;
        end;
        drop _type;
       run;
      %end;
    
      %else %do;   
       data _null_;
        set &_targettype._row;
        if _n_=1;
        call symput("_file_&_targettype._row",trim(left(scan(_sas_data_name,1))));
       run;
       
       %let _check_loc_&_targettype = %sysfunc(libref(&&&_file_&_targettype._row));
       
       %if &&&_check_loc_&_targettype = 0 %then %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_targettype._row (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: &_targettype row specifies a valid output location";
          _error_flag = 0;
          _message = "";
          output _errors;
         end;
         drop _type;
        run;
       %end;
       %else %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_targettype._row (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: &_targettype row specifies a valid output location";
          _error_flag = 1;
          _message = "ERROR: libname specified in _sas_data_name is not assigned or invalid";
          output _errors;
         end;
         drop _type;
        run;
       %end;
      %end;  
     %end;
   
     %else %if &&&_nobs_&_targettype._row > 1  %then %do;
      data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
       set &_targettype._row (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype row specifies a valid output location";
        _error_flag = 1;
        _message = "ERROR: multiple &_targettype rows detected";
        output _errors;
       end;
       drop _type;
      run;
     %end;
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._row _errors;
    quit;
    
   %mend _output_loc_check;
   
   %_output_loc_check(&_indash,outdata);
   
   
   
   %macro _var_nodups(_dbdata,_targettype);
    
    data &_targettype._dupe;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._dupe;
    run; 
   
    data _null_;
     set &_targettype._dupe;  
     call symput("_&_targettype._var",lowcase(trim(left(_variable))));
     call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
     call symput("_&_targettype._value",trim(left(_value)));
    run;  
   
    proc sort data=&&&_&_targettype._sourcedata out=_sort_&_targettype;
     by &&&_&_targettype._var;
    run;
   
    data _single_&_targettype _multi_&_targettype;
     set _sort_&_targettype;
     by &&&_&_targettype._var;
     if first.&&&_&_targettype._var and last.&&&_&_targettype._var then output _single_&_targettype;
     else output _multi_&_targettype;
    run;
   
    data _multi_&_targettype;
     set _multi_&_targettype;
    run;
   
    %_mtanyobs;
     
    %let _dset_dupes_&_targettype = &hasobs;
    
    %if &&&_dset_dupes_&_targettype = 0 %then %do;
   
     *ods listing close;
     *ods output onewayfreqs=miss_&_targettype;
      proc freq data=&&&_&_targettype._sourcedata noprint;
       tables &&&_&_targettype._var /missing out=miss_&_targettype;
      run;
     *ods listing;
     
     data miss_&_targettype;
      set miss_&_targettype;
      if &&&_&_targettype._var = '';
     run; 
   
     %_mtanyobs;
      
     %let _miss_&_targettype._var = &hasobs;
   
     %if &&&_miss_&_targettype._var = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: no duplicate or missing values of the ID variable in cohort dataset";
        _error_flag = 1;
        _message = "ERROR: missing values for variable specified by &_targettype row found in cohort dataset";
        output _errors;
       end;
       drop _type;
      run;
     %end;
     %else %do;  
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: no duplicate or missing values of the ID variable in cohort dataset";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
     %end;  
    %end; 
    %else %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: no duplicate or missing values of the ID variable in cohort dataset";
       _error_flag = 1;
       _message = "ERROR: duplicate values of variable specified by &_targettype row found in cohort dataset";
       output _errors;
      end;
      drop _type;
     run; 
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._dupe _sort_&_targettype _single_&_targettype _multi_&_targettype _errors;
    quit;
   %mend _var_nodups; 
   
   %_var_nodups(&_indash,studyid); 
   
   %_varfmt_confirm(&_indash,studyid);  
   
   %macro _unitdays_check(_dbdata,_targettype);
    
    data &_targettype._unitdays;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._unitdays;
    run; 
   
    data _null_;
     set &_targettype._unitdays;  
     _int_value = floorz(_value);
     if _value <= 0 then do;
      call symput("_&_targettype._integer",0);   
     end;
     else if _int_value = _value then do;
      call symput("_&_targettype._integer",1);      
     end;
     else if _int_value ne _value then do;
      call symput("_&_targettype._integer",2);      
     end;
    run;  
   
    %if &&&_&_targettype._integer = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is an integer";
       _error_flag = 1;
       _message = "ERROR: _value of _unitdays row is not a positive number";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    %else %if &&&_&_targettype._integer = 1 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is an integer";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    %else %if &&&_&_targettype._integer = 2 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is an integer";
       _error_flag = 1;
       _message = "ERROR: _value of _unitdays row is not an integer";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._unitdays _errors;
    quit;
   %mend _unitdays_check; 
   
   %_unitdays_check(&_indash,unitdays); 
   
   
   %macro _hformat_check(_dbdata,_targettype);
    
    data &_targettype._hformat;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._hformat;
    run; 
   
    data _null_;
     set &_targettype._hformat;  
     _int_value = floorz(_value);
     if _value = 0 then do;
      call symput("_&_targettype._hformat",0);   
     end;
     else if _value = 1 then do;
      call symput("_&_targettype._hformat",1);      
     end;
     else do;
      call symput("_&_targettype._hformat",2);      
     end;
    run;  
   
    %if &&&_&_targettype._hformat = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is 0 or 1";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    %else %if &&&_&_targettype._hformat = 1 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is 0 or 1";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    %else %if &&&_&_targettype._hformat = 2 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value of &_targettype row is 0 or 1";
       _error_flag = 1;
       _message = "ERROR: _value of &_targettype row is neither 0 nor 1";
       output _errors;
      end;
      drop _type;
     run;   
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._hformat _errors;
    quit;
   %mend _hformat_check; 
   
   %_hformat_check(&_indash,harvardformat); 
   
   
   
   %macro _variable_outcome(_dbdata,_targettype);
   
    data &_targettype._compare;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._compare;
    run;
   
    data _null_;
     set &_targettype._compare;
     if _variable = '' then do;
      call symput("_&_targettype._varcheck",0);
     end;
     else do;
      call symput("_&_targettype._varcheck",1);
     end;
    run;
    
    %if &&&_&_targettype._varcheck = 1 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype is a character string";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    %else %if &&&_&_targettype._varcheck = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype is a character string";
       _error_flag = 1;
       _message = "ERROR: _variable in row &_targettype is missing";
       output _errors;
       call symput("_maj_error",2); 
      end;
      drop _type;
     run;  
    %end;
     
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._compare _errors;
    quit;
   
   %mend _variable_outcome;   
    

   
   %_variable_outcome(&_indash,outcome);
   
   
   
   %macro _datevar_outcome(_dbdata,_targettype);
   
    data &_targettype._compare;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._compare;
    run;
    
    data _null_;
     set &_targettype._compare;
     call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
     call symput("_&_targettype._date",lowcase(trim(left(_sas_date_var))));
    run;
     
    *ods listing close;
    *ods output variables=_variables_&_targettype;
    proc contents data=&&&_&_targettype._sourcedata out=_variables_&_targettype noprint;
    run;
    *ods listing;

	  data _variables_&_targettype;
      set _variables_&_targettype;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
      run;

   
    data _variables_&_targettype;
     set _variables_&_targettype;
     if lowcase(Name) = "&&&_&_targettype._date";
    run;
   
    %_mtanyobs;
    
    %let _var_&_targettype._date = &hasobs;
    
    %if &&&_var_&_targettype._date = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _sas_date_var for row &_targettype names a date variable in the cohort dataset";
       _error_flag = 1;
       _message = "ERROR: the variable specified by _sas_date_var cannot be found in the cohort dataset";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    %else %do;
     data _variables_&_targettype;
      set _variables_&_targettype;
      if upcase(type) in ('NUM') then do;
       call symput ("_error_date_&_targettype",0);
      end;
      else do;
       call symput ("_error_date_&_targettype",1);
      end;
     run;
     %if &&&_error_date_&_targettype = 0 %then %do;
      data __&_targettype._check_date;
       set &&&_&_targettype._sourcedata;
       _remtest = mod(&&&_&_targettype._date,1);
       if _remtest ne 0 then do;
        call symput ("_error_values_&_targettype",1);
       end;
       else do;
        call symput ("_error_values_&_targettype",0);
       end;
      run;
      %if &&&_error_values_&_targettype = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._compare (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _sas_date_var for row &_targettype names a date variable in the cohort dataset";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;
      %end;
      %if &&&_error_values_&_targettype = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_targettype._compare (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _sas_date_var for row &_targettype names a date variable in the cohort dataset";
         _error_flag = 1;
         _message = "ERROR: variable specified by _sas_date_var contains non-integer or missing values";
         output _errors;
        end;
        drop _type;
       run; 
      %end;     
     %end;   
     %else %if &&&_error_date_&_targettype = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_targettype._compare (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _sas_date_var for row &_targettype names a date variable in the cohort dataset";
        _error_flag = 1;
        _message = "ERROR: variable specified by _sas_date_var is not numeric";
        output _errors;
       end;
       drop _type;
      run; 
     %end;
    %end;
   
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._compare _errors;
    quit;
   
   %mend _datevar_outcome;
   
   %_datevar_outcome(&_indash,outcome);
   
   
   %macro _behavior_outcome(_dbdata,_targettype);
   
    data &_targettype._compare;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._compare;
    run;
   
    data _null_;
     set &_targettype._compare;
     call symput("_&_targettype._behavior",lowcase(trim(left(_behavior))));
    run;
   
    %if &&&_&_targettype._behavior = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _behavior in row &_targettype is 0";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    %else %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _behavior in row &_targettype is 0";
       _error_flag = 1;
       _message = "ERROR: the value of _behavior for outcome row must be 0";
       output _errors;
      end;
      drop _type;
     run;
    %end;
   
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._compare _errors;
    quit;
    
   %mend _behavior_outcome;
   
   %_behavior_outcome(&_indash,outcome);
   
   

   
   
   %macro _expos_var_blank(_dbdata,_targettype);
   
    data &_targettype._compare;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._compare;
    run;
   
    data _null_;
     set &_targettype._compare;
     if _variable = '' then do;
      call symput("_&_targettype._varcheck",0);
     end;
     else do;
      call symput("_&_targettype._varcheck",1);
      call symput("_&_targettype._var",lowcase(trim(left(_variable))));
      call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
     end;
    run;
    %if &&&_&_targettype._varcheck = 1 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype specifies a character string";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    %else %if &&&_&_targettype._varcheck = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype specifies a character string";
       _error_flag = 1;
       _message = "ERROR: _variable in row &_targettype is missing";
       output _errors;
      end;
      drop _type;
     run;  
    %end;
     
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._compare _errors;
    quit;
   
   %mend _expos_var_blank;
   
   %_expos_var_blank(&_indash,exposure); 
   
   %macro _first_exp_check(_dbdata);
    data _null_;
     set &_dbdata;
     if lowcase(_type) = "exposure";
     if _first_exp_rule = 0 then do;
      call symput("_first_exp_error",0);
     end;
     else if _first_exp_rule = 1 then do;
      call symput("_first_exp_error",0);
     end;
     else do;
      call symput("_first_exp_error",1);
     end;
    run;
   
    %if &_first_exp_error = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: value _first_exp_rule either 0 or 1 in exposure row";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;    
    %end;
    %else %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: value _first_exp_rule either 0 or 1 in exposure row";
       _error_flag = 1;
       _message = "ERROR: value other than 0 or 1 detected for _first_exp_rule of exposure row";
       output _errors;
      end;
      drop _type;
     run;    
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete _start_date_comp _end_date_comp _errors;
    quit;
   
   %mend _first_exp_check;
   
   %_first_exp_check(&_indash); 
   

   
   
   %macro _expos_date_overlap(_dbdata);
   
    data _start_date_comp _end_date_comp;
     set _error_checks;
     if _checktype = "dashboard: exposure row specifies valid _sas_start_var variable" then output _start_date_comp;
     else if _checktype = "dashboard: exposure row specifies valid _sas_end_var variable" then output _end_date_comp;   
    run;
   
    data _null_;
     set _start_date_comp;
     if _error_flag = 1 then do;
      call symput ("_error_start_date_prev",1);    
     end;
     else if _error_flag = 0 then do;
      call symput ("_error_start_date_prev",0);   
     end;
    run;
   
    data _null_;
     set _end_date_comp;
     if _error_flag = 1 then do;
      call symput ("_error_end_date_prev",1);    
     end;
     else if _error_flag = 0 then do;
      call symput ("_error_end_date_prev",0);   
     end;
    run;
   
    %if _error_start_date_prev = 1 %then %do;
     data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "exposure: no abnormal start and end date values";
       _error_flag = 1;
       _message = "ERROR: dashboard parameter _sas_start_var in exposure row contains an error";
       output _errors;
      end;
      drop _type;
     run;
    %end;  
    %else %if _error_end_date_prev = 1 %then %do;
     data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "exposure: no abnormal start and end date values";
       _error_flag = 1;
       _message = "ERROR: dashboard parameter _sas_end_var in exposure row contains an error";
       output _errors;
      end;
      drop _type;
     run;
    %end;
    %else %do;
     data _null_;
      set &_dbdata;
      if lowcase(_type) = "exposure";
      call symput("_file_exposure_target",trim(left(_sas_data_name)));
      call symput("_var_exposure_start",trim(left(_sas_start_var)));
      call symput("_var_exposure_end",trim(left(_sas_end_var)));
     run;
    
     data _exposure_dates_overrun;
      set &_file_exposure_target;
      if &_var_exposure_end < &_var_exposure_start;
     run;
   
     %_mtanyobs;
     
     %let _overrun_exposure_dates = &hasobs;      
     
     %if &_overrun_exposure_dates = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "exposure: no abnormal start and end date values";
        _error_flag = 1;
        _message = "ERROR: records found in exposure dataset where _sas_end_var < _sas_start_var";
        output _errors;
       end;
       drop _type;
      run;
     %end;
     %else %do;
      data _null_;
       set &_dbdata;
       if lowcase(_type) = "studyid";
       call symput("_var_studyid",lowcase(trim(left(_variable))));
      run;
      
      proc sort data=&_file_exposure_target out=_exposure_date_order;
       by &_var_studyid &_var_exposure_start &_var_exposure_end;
      run;
      
      data _exposure_date_order;
       set _exposure_date_order;
       by &_var_studyid;
       _prev_end_date = lag1(&_var_exposure_end);
       if first.&_var_studyid then _prev_end_date = .;
      run;
   
      data _exposure_date_order;
       set _exposure_date_order;
       by &_var_studyid;
       if &_var_exposure_start < _prev_end_date;
      run;    
   
      %_mtanyobs;
     
      %let _overlap_exposure_dates = &hasobs;  
      %if &_overlap_exposure_dates = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "exposure: no abnormal start and end date values";
         _error_flag = 1;
         _message = "ERROR: records found in exposure dataset where _sas_start_var < _sas_end_var of the previous exposure record for patient";
         output _errors;
        end;
        drop _type;
       run;
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "exposure: no abnormal start and end date values";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;    
      %end;
     %end;
    %end;
      
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete _start_date_comp _end_date_comp _exposure_dates_overrun _exposure_date_order _errors;
    quit;
   
   %mend _expos_date_overlap;
   
   %_expos_date_overlap(&_indash); 
   
%macro _expos_value_check(_dbdata);

   data _null_;
     set &_dbdata;
     if lowcase(_type) = "exposure";
     if 0 lt _value le 1 then do;
      call symput("_exposure_value_error",0);
     end;
     else if missing(_value) = 1 then do;
      call symput("_exposure_value_error",0);
     end;
     else do;
      call symput("_exposure_value_error",1);
     end;
    run;
   
    %if &_exposure_value_error = 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value is either missing or between 0-1 in exposure row";
       _error_flag = 0;
       _message = "";
       output _errors;
      end;
      drop _type;
     run;    
    %end;
    %else %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _value is either missing or between 0-1 in exposure row";
       _error_flag = 1;
       _message = "ERROR: _value of exposure is strictly not above 0 and below/equal 1";
       output _errors;
      end;
      drop _type;
     run;    
    %end;
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete _start_date_comp _end_date_comp _errors;
    quit;
   
   %mend _expos_value_check;
   
  %_expos_value_check (&_indash); 


  * covariates processing begins here;
   
   data _null_;
    set &_indash;
    if lowcase(_type) = 'studyid' then do;
     call symput("_cov_studyid",lowcase(trim(left(_variable))));
    end;
    if lowcase(_type) = 'cohort' then do;
     call symput("_cov_cohort_data",trim(left(_sas_data_name)));
    end;
    if lowcase(_type) = 'studystart' then do;
     call symput("_cov_cohort_sdate",lowcase(trim(left(_sas_date_var))));
    end;
   run;
   
*=========
 BLOCK D
==========;
  *NEW Macro added 1/2016 See macro D0.1 in excel macro lists;
 
 %macro _variable_expos(_dbdata ,_targettype);
  
   data &_targettype._compare;
    set &_dbdata;
    if lowcase(_type) = "&_targettype" then output &_targettype._compare;
   run;
  
   data _null_;
    set &_targettype._compare;
     call symput("_&_targettype._var",lowcase(trim(left(_variable))));
     call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
	 call symput("_&_targettype._fmt",trim(left(_varfmt)));
   run;
 
    proc contents data=&&&_&_targettype._sourcedata out=_variables_&_targettype noprint;
    run;
 
    data _variables_&_targettype;
     set _variables_&_targettype;
     if lowcase(Name) = "&&&_&_targettype._var";
    run;
   
	%_mtanyobs;

 %let _var_&_targettype._var = &hasobs; 



  %if  &&&_&_targettype._fmt ne  and &&&_var_&_targettype._var = 0 %then %do;
    data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
       _error_flag = 1;
       _message = "ERROR: _variable cannot be found in the dataset specified in _sas_data_name or _varfmt should be missing";
       output _errors;
       call symput("_maj_error",2); 
      end;
      drop _type;
    run;
   %end; 
   %else %if &&&_&_targettype._fmt =  and  &&&_var_&_targettype._var = 1  %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable in row &_targettype should not specify a valid variable in the exposure data set";
        _error_flag = 1;
        _message = "ERROR:_variable should not be found in the dataset specified in _sas_data_name or _varfmt should not be  missing";
        output _errors;
        call symput("_maj_error",2); 
       end;
       drop _type;
      run;
   %end;
  %else %do;
       data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable in row &_targettype specifies a valid variable";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
  %end;
 
  proc append data=_errors base=_error_checks force; 
   run;
   
  proc datasets library = work nolist;
    delete &_targettype._compare _errors;
  quit;
  
%mend _variable_expos;

%_variable_expos(&_indash,exposure);


   data _covars;
    set &_indash;
    if lowcase(_type) = 'covariate';
   run;
                                                           
   %_nobs_msm(_covars);
   %global _cov_counter;
   %let _cov_counter = %eval(&_nobs);
   
*=====================================
do loop start here cover block D E F
=====================================;
   %do _i=1 %to &_cov_counter;
    data _covars_&_i;
     set _covars;
     if _n_=&_i;
     call symput("_cov_variable_&_i",lowcase(trim(left(_variable))));
     call symput("_cov_dataset_&_i",trim(left(_sas_data_name)));
     call symput("_cov_behavior_&_i",trim(left(_behavior)));
     call symput("_cov_acute_&_i",trim(left(_no_acute_change)));
     call symput("_cov_date_&_i",lowcase(trim(left(_sas_date_var))));
     call symput("_cov_varfmt_&_i",trim(left(_varfmt)));
     call symput("_cov_impute_&_i",trim(left(_impute)));
     call symput("_cov_varclass_&_i",trim(left(_varclass)));
    run;
   
    %macro _cov_data_name_check(_dbdata,_targettype);
     %global _cov_error_&_i;  
     %let _cov_error_&_i = 0;
    
     data _null_;
      set &_dbdata;
      if _n_=1;
      call symput("_data_length_&_i",lengthn(trim(left(_sas_data_name))));
     run;
       
     %if &&&_data_length_&_i = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a valid dataset in _sas_data_name";
        _error_flag = 1;
        _message = "ERROR: no dataset specified in _sas_data_name of &_targettype covariate row";
        output _errors;
       end;
       drop _type;
      run;
      %let _cov_error_&_i = 1;    
     %end;
     
     %else %do;  
      %let _check_dset_cov_&_i = %sysfunc(open(&&&_cov_dataset_&_i,IS));
      %if &&&_check_dset_cov_&_i = 0 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: &_targettype covariate row specifies a valid dataset in _sas_data_name";
         _error_flag = 1;
         _message = "ERROR: dataset specified in _sas_data_name of &_targettype covariate row cannot be opened";
         output _errors;
        end;
        drop _type;
       run;
      %let _cov_error_&_i = 1;  
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: &_targettype covariate row specifies a valid dataset in _sas_data_name";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;
      %end;
     %end; 
     
     proc append data=_errors base=_error_checks force; 
     run;
     
     proc datasets library = work nolist;
      delete _errors;
     quit;
     
    %mend _cov_data_name_check;
    
    %_cov_data_name_check(_covars_&_i,&&&_cov_variable_&_i);
   
    %macro _cov_pop_param_check(_dbdata,_targettype,_param);
     data _null_;
      set &_dbdata;
      if _n_=1;
      call symput("_para_length_&_i",lengthn(trim(left(&_param))));
     run;
       
     %if &&&_para_length_&_i = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a value for &_param";
        _error_flag = 1;
        _message = "ERROR: no value for &_param specified in &_targettype covariate row";
        output _errors;
       end;
       drop _type;
      run;
      %let _cov_error_&_i = 1; 
     %end;
     %else %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a value for &_param";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;   
     %end;
     
     proc append data=_errors base=_error_checks force; 
     run;
     
     proc datasets library = work nolist;
      delete _errors;
     quit;
     
    %mend _cov_pop_param_check;
    
    %_cov_pop_param_check(_covars_&_i,&&&_cov_variable_&_i,_sas_date_var);
    %_cov_pop_param_check(_covars_&_i,&&&_cov_variable_&_i,_variable);
   
    %macro _cov_behav_init_check(_dbdata,_targettype);
     data _null_;
      set &_dbdata;
      if _n_=1;
      if _behavior = 1 or _behavior = 2 or _behavior = 3 or _behavior = 4 or _behavior = 5 then do;
       call symput("_behav_verif_&_i",1);
      end;
      else do;
       call symput("_behav_verif_&_i",0);    
      end;
     run;
       
     %if &&&_behav_verif_&_i = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a valid value for _behavior";
        _error_flag = 1;
        _message = "ERROR: value of _behavior in &_targettype covariate row not 1, 2, 3, 4 or 5";
        output _errors;
       end;
       drop _type;
      run;
      %let _cov_error_&_i = 1; 
     %end;
     %else %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a valid value for _behavior";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;   
     %end;
     
     proc append data=_errors base=_error_checks force; 
     run;
     
     proc datasets library = work nolist;
      delete _errors;
     quit;
     
    %mend _cov_behav_init_check;
    
    %_cov_behav_init_check(_covars_&_i,&&&_cov_variable_&_i);
   
    %macro _cov_varfmt_init_check(_dbdata,_targettype);
     data _null_;
      set &_dbdata;
      if _n_=1;
      if _varfmt = "C" or _varfmt = "N" then do;
       call symput("_varfmt_verif_&_i",1);
      end;
      else do;
       call symput("_varfmt_verif_&_i",0);    
      end;
     run;
       
     %if &&&_varfmt_verif_&_i = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a valid value for _varfmt";
        _error_flag = 1;
        _message = "ERROR: value of _varfmt in &_targettype covariate row neither C nor N";
        output _errors;
       end;
       drop _type;
      run;
      %let _cov_error_&_i = 1; 
     %end;
     %else %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: &_targettype covariate row specifies a valid value for _varfmt";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;   
     %end;
     
     proc append data=_errors base=_error_checks force; 
     run;
     
     proc datasets library = work nolist;
      delete _errors;
     quit;
     
    %mend _cov_varfmt_init_check;
    
    %_cov_varfmt_init_check(_covars_&_i,&&&_cov_variable_&_i);
   
    %put &&&_cov_error_&_i;
   
    %if &&&_cov_error_&_i = 1 %then %do;
     
     proc datasets library = work nolist;
      delete _errors;
     quit;   
      
    %end;
    %else %do;
*=========
 BLOCK E
==========;

     %macro _cov_var_second_check(_dbdata,_targettype);
     
      data _null_;
       set &_dbdata;
       if _n_=1;
       call symput("_data_loc_&_i",trim(left(_sas_data_name)));
       call symput("_var_name_&_i",lowcase(trim(left(_variable))));
       call symput("_behav_val_&_i",_behavior);
      run;
     
      %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
       %let _cov_cohort_target_&_i = 1;
      %end;
      %else %if &&&_behav_val_&_i = 1 %then %do;
       %let _cov_cohort_target_&_i = 1;
      %end;
      %else %do;
       %let _cov_cohort_target_&_i = 0;
      %end;
      
      %if &&&_cov_cohort_target_&_i = 1 %then %do;
       *ods listing close;
       *ods output variables=_varcheck_&_i;
       proc contents data=&_cov_cohort_data out=_varcheck_&_i noprint;
       run;
       *ods listing;

	  data _varcheck_&_i;
      set _varcheck_&_i;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
      run;

      
       data _varcheck_&_i;
        set _varcheck_&_i;
        if lowcase(Name) = "&&&_var_name_&_i";
       run;
       
       %_mtanyobs;
       
       %let _varcheck_present_&_i = &hasobs;
       %if &&&_varcheck_present_&_i = 1 %then %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "covariate: _variable for &_targettype is present in source datasets";
          _error_flag = 0;
          _message = "";
          output _errors;
         end;
         drop _type;
        run;
       %end;
       %else %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "covariate: _variable for &_targettype is present in source datasets";
          _error_flag = 1;
          _message = "ERROR: _variable for &_targettype specified in _variable not detected in cohort dataset";
          output _errors;
          %let _cov_error_&_i = 2; 
         end;
         drop _type;
        run;
       %end;
      %end;
      %else %do;
       *ods listing close;
       *ods output variables=_varcheck1_&_i;
       proc contents data=&_cov_cohort_data out=_varcheck1_&_i noprint;
       run;
       *ods listing;

       data _varcheck1_&_i;
        set _varcheck1_&_i;
        if lowcase(Name) = "&&&_var_name_&_i";
       run;
       
       %_mtanyobs;
       
       %let _varcheck_cohort_&_i = &hasobs;
     
       *ods listing close;
       *ods output variables=_varcheck2_&_i;
       proc contents data=&&&_data_loc_&_i out=_varcheck2_&_i noprint;
       run;
       *ods listing;
      
       data _varcheck2_&_i;
        set _varcheck2_&_i;
        if lowcase(Name) = "&&&_var_name_&_i";
       run;
       
       %_mtanyobs;
       
       %let _varcheck_covar_&_i = &hasobs;
       
       %if &&&_varcheck_cohort_&_i = 1 %then %do;
        %if &&&_varcheck_covar_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: _variable for &_targettype is present in source datasets";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;     
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: _variable for &_targettype is present in source datasets";
           _error_flag = 1;
           _message = "ERROR: _variable for &_targettype specified in _variable not detected in covariate dataset";
           output _errors;
           %let _cov_error_&_i = 2; 
          end;
          drop _type;
         run;     
        %end;
       %end;
       %else %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "covariate: _variable for &_targettype is present in source datasets";
          _error_flag = 1;
          _message = "ERROR: _variable for &_targettype specified in _variable not detected in cohort dataset";
          output _errors;
          %let _cov_error_&_i = 2; 
         end;
         drop _type;
        run;
       %end;
      %end; 
      proc append data=_errors base=_error_checks force; 
      run;
     
      proc datasets library = work nolist;
       delete _errors;
      quit;
     %mend _cov_var_second_check;
     
     %_cov_var_second_check(_covars_&_i,&&&_cov_variable_&_i);

     %put &&&_cov_error_&_i;
   
     %if &&&_cov_error_&_i = 2 %then %do;
     
      proc datasets library = work nolist;
       delete _errors;
      quit;   
      
     %end;
     
     %else %do;
      
*=========
 BLOCK F
==========; 
      %macro _cov_data_id_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
       run;
       
       data _cov_id_missing_&_i;
        set &&&_data_loc_&_i;
        if &_cov_studyid = '';
       run;
       
       %_mtanyobs;
       
       %let _cov_id_missing_&_i = &hasobs;     
          
       %if &&&_cov_id_missing_&_i = 1 %then %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "covariate: covariate dataset for &_targettype contains only study IDs in cohort";
          _error_flag = 1;
          _message = "ERROR: the study ID column of the &_targettype covariate dataset contains missing values";
          output _errors;
         end;
         drop _type;
        run;      
       %end;
       %else %do;
        proc sort data=&&&_data_loc_&_i out=_data_&_targettype (keep = &_cov_studyid);
         by &_cov_studyid;
        run;
        
        proc sort data=&_cov_cohort_data out=_ids_&_targettype (keep = &_cov_studyid);
         by &_cov_studyid;
        run;
         
        data _not_in_cohort_&_i;
         merge _data_&_targettype(in=a) _ids_&_targettype(in=b);
         by &_cov_studyid;
         if a and not b then output _not_in_cohort_&_i;
        run;
        
        %_mtanyobs;
        
        %let _not_in_cohort_&_i = &hasobs;     
            
        %if &&&_not_in_cohort_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: covariate dataset for &_targettype contains only study IDs in cohort";
           _error_flag = 1;
           _message = "ERROR: the study ID column of the &_targettype covariate dataset contains values not in the study ID column of the cohort dataset";
           output _errors;
          end;
          drop _type;
         run;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: covariate dataset for &_targettype contains only study IDs in cohort";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;   
        %end;
       %end;  
      
       proc append data=_errors base=_error_checks force; 
       run;
       
       proc datasets library = work nolist;
        delete _errors _not_in_cohort_&_i _data_&_targettype _ids_&_targettype _cov_id_missing_&_i;
       quit;
      
      %mend _cov_data_id_check;
      
      %_cov_data_id_check(_covars_&_i,&&&_cov_variable_&_i); 
      
      %macro _cov_date_var_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
        call symput("_date_var_&_i",lowcase(trim(left(_sas_date_var))));
       run;
       
       %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
        %if &_cov_cohort_sdate = &&&_date_var_&_i %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;       
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
           _error_flag = 1;
           _message = "ERROR: _sas_date_var for &_targettype is not equal to a variable in the cohort dataset";
           output _errors;
          end;
          drop _type;
         run;        
        %end;
       %end;
       %else %do;
        *ods listing close;
        *ods output variables=_variables_&_i._date;
        proc contents data=&&&_data_loc_&_i out=_variables_&_i._date noprint;
        run;
        *ods listing;
      
       data _variables_&_i._date;
       set _variables_&_i._date;
       length type_c $4.;
       if type=1 then type_c ='Num';
       if type=2 then type_c='Char';
       drop type;
       rename type_c=type;
       run;

        data _variables_&_i._date;
         set _variables_&_i._date;
         if lowcase(Name) = "&&&_date_var_&_i";
        run;
        
        %_mtanyobs;
        
        %let _var_&_i._date = &hasobs;
        %if &&&_var_&_i._date = 1 %then %do;
        
         data _variables_&_i._date;
          set _variables_&_i._date;
          if upcase(type) in ('NUM') then do;
           call symput ("_error_date_&_i",0);
          end;
          else do;
           call symput ("_error_date_&_i",1);
          end;
         run;
         
         %if &&&_error_date_&_i = 0 %then %do;
          data _date_check_cov_&_i;
           set &&&_data_loc_&_i;
           _remtest = mod(&&&_date_var_&_i,1);
           if _remtest ne 0 then do;
            call symput ("_cov_error_date_&_i",1);
           end;
           else do;
            call symput ("_cov_error_date_&_i",0);
           end;
          run;
          %if &&&_cov_error_date_&_i = 1 %then %do;
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
             _error_flag = 1;
             _message = "ERROR: _sas_date_var for &_targettype has non-integer values";
             output _errors;
            end;
            drop _type;
           run;      
          %end;
          %else %if &&&_cov_error_date_&_i = 0 %then %do;
           proc sort data=&&&_data_loc_&_i out=_cov_data_dupcheck_&_i;
            by &_cov_studyid &&&_date_var_&_i;
           run;
           
           data _cov_data_single_&_i _cov_data_dupe_&_i;
            set _cov_data_dupcheck_&_i;
            by &_cov_studyid &&&_date_var_&_i;
            if first.&&&_date_var_&_i then output _cov_data_single_&_i;
            else output _cov_data_dupe_&_i;
           run;
           
           data _cov_data_dupe_&_i;
            set _cov_data_dupe_&_i;
           run;
            
           %_mtanyobs;
        
           %let _var_&_i._dup_data = &hasobs;     
          
           %if &&&_var_&_i._dup_data = 0 %then %do;
            data _errors;
             length _checktype $100 
                    _error_flag 3
                    _message $400;
             set &_indash (keep = _type);
             if _n_ = 1;
             if _n_ = 1 then do;
              _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
              _error_flag = 0;
              _message = "";
              output _errors;
             end;
             drop _type;
            run;
           %end;
           %else %do;
            data _errors;
             length _checktype $100 
                    _error_flag 3
                    _message $400;
             set &_indash (keep = _type);
             if _n_ = 1;
             if _n_ = 1 then do;
              _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
              _error_flag = 1;
              _message = "ERROR: the covariate dataset for &_targettype contains rows with non-unique combinations of study ID and _sas_date_var values";
              output _errors;
             end;
             drop _type;
            run;      
           %end; 
          %end;        
         %end;
         %else %if &&&_error_date_&_i = 1 %then %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
            _error_flag = 1;
            _message = "ERROR: the variable specified by _sas_date_var for &_targettype is not numeric";
            output _errors;
           end;
           drop _type;
          run;         
         %end;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: _sas_date_var for &_targettype is a legitimate variable with valid values";
           _error_flag = 1;
           _message = "ERROR: _sas_date_var for &_targettype not found in dataset specified by _sas_data_name";
           output _errors;
          end;
          drop _type;
         run;  
        %end;
       %end;
       
       proc append data=_errors base=_error_checks force; 
       run;
       
       proc datasets library = work nolist;
        delete _errors _cov_data_dupe_&_i _cov_data_dupcheck_&_i 
         _cov_data_single_&_i _date_check_cov_&_i _variables_&_i._date;
       quit;
      
      %mend _cov_date_var_check;
      
      %_cov_date_var_check(_covars_&_i,&&&_cov_variable_&_i); 
      
      %macro _cov_varfmt_second_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
        call symput("_var_name_&_i",lowcase(trim(left(_variable))));
        call symput("_varfmt_val_&_i",_varfmt);
        call symput("_behav_val_&_i",_behavior);
       run;
       
       %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
        %let _cov_cohort_target_&_i = 1;
       %end;
       %else %if &&&_behav_val_&_i = 1 %then %do;
        %let _cov_cohort_target_&_i = 1;
       %end;
       %else %do;
        %let _cov_cohort_target_&_i = 0;
       %end;
       
       %if &&&_cov_cohort_target_&_i = 1 %then %do;
        *ods listing close;
        *ods output variables=_fmtcheck_&_i;
        proc contents data=&_cov_cohort_data out=_fmtcheck_&_i noprint;
        run;
        *ods listing;

	 data _fmtcheck_&_i;
      set _fmtcheck_&_i;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
      run;

       
        data _fmtcheck_&_i;
         set _fmtcheck_&_i;
         if lowcase(Name) = "&&&_var_name_&_i";
         _fmt_target = "&&&_varfmt_val_&_i";
         if _fmt_target = 'C' then do;
          if upcase(type) = 'CHAR' then do;
           call symput("_varfmt_check_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check_&_i",0);        
          end;
         end;
         else if _fmt_target = 'N' then do;
          if upcase(type) = 'NUM' then do;
           call symput("_varfmt_check_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check_&_i",0);        
          end;     
         end;
        run;
       
        %if &&&_varfmt_check_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: format of _variable in source datasets for &_targettype matches _varfmt";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: format of _variable in source datasets for &_targettype matches _varfmt";
           _error_flag = 1;
           _message = "ERROR: the format of _variable in _sas_data_name for &_targettype does not match the format specified by _varfmt";
           output _errors;
          end;
          drop _type;
         run;
        %end;
       %end;
       %else %do;
        *ods listing close;
        *ods output variables=_fmtcheck1_&_i;
        proc contents data=&_cov_cohort_data out=_fmtcheck1_&_i noprint;
        run;
        *ods listing;

		 data _fmtcheck1_&_i;
         set _fmtcheck1_&_i;
         length type_c $4.;
         if type=1 then type_c ='Num';
         if type=2 then type_c='Char';
         drop type;
         rename type_c=type;
         run;

       
        data _fmtcheck1_&_i;
         set _fmtcheck1_&_i;
         if lowcase(Name) = "&&&_var_name_&_i";
         _fmt_target = "&&&_varfmt_val_&_i";
         if _fmt_target = 'C' then do;
          if upcase(type) = 'CHAR' then do;
           call symput("_varfmt_check1_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check1_&_i",0);        
          end;
         end;
         else if _fmt_target = 'N' then do;
          if upcase(type) = 'NUM' then do;
           call symput("_varfmt_check1_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check1_&_i",0);        
          end;     
         end;
        run;
       
        *ods listing close;
        *ods output variables=_fmtcheck2_&_i;
        proc contents data=&&&_data_loc_&_i out=_fmtcheck2_&_i noprint;
        run;
        *ods listing;

         data _fmtcheck2_&_i;
         set _fmtcheck2_&_i;
         length type_c $4.;
         if type=1 then type_c ='Num';
         if type=2 then type_c='Char';
         drop type;
         rename type_c=type;
         run;


        data _fmtcheck2_&_i;
         set _fmtcheck2_&_i;
         if lowcase(Name) = "&&&_var_name_&_i";
         _fmt_target = "&&&_varfmt_val_&_i";
         if _fmt_target = 'C' then do;
          if upcase(type) = 'CHAR' then do;
           call symput("_varfmt_check2_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check2_&_i",0);        
          end;
         end;
         else if _fmt_target = 'N' then do;
          if upcase(type) = 'NUM' then do;
           call symput("_varfmt_check2_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check2_&_i",0);        
          end;     
         end;
        run;
        
        %if &&&_varfmt_check1_&_i = 1 %then %do;
         %if &&&_varfmt_check2_&_i = 1 %then %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: format of _variable in source datasets for &_targettype matches _varfmt";
            _error_flag = 0;
            _message = "";
            output _errors;
           end;
           drop _type;
          run;     
         %end;
         %else %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: format of _variable in source datasets for &_targettype matches _varfmt";
            _error_flag = 1;
            _message = "ERROR: the format of _variable in _sas_data_name for &_targettype does not match the format specified by _varfmt";
            output _errors;
           end;
           drop _type;
          run;     
         %end;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: format of _variable in source datasets for &_targettype matches _varfmt";
           _error_flag = 1;
           _message = "ERROR: the format of _variable in cohort dataset for &_targettype does not match the format specified by _varfmt";
           output _errors;
          end;
          drop _type;
         run;
        %end;
       %end; 
       proc append data=_errors base=_error_checks force; 
       run;
       
       proc datasets library = work nolist;
        delete _errors _fmtcheck_&_i _fmtcheck1_&_i _fmtcheck2_&_i;
       quit;
      %mend _cov_varfmt_second_check;
      
      %_cov_varfmt_second_check(_covars_&_i,&&&_cov_variable_&_i);
      
      %macro _cov_no_acute_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
        call symput("_no_acute_val_&_i",_no_acute_change);
       run;
      
       %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
        %let _cov_cohort_target_&_i = 1;
       %end;
       %else %do;
        %let _cov_cohort_target_&_i = 0;
       %end;
       
       %if &&&_cov_cohort_target_&_i = 1 %then %do;
        %if &&&_no_acute_val_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: valid value of _no_acute_change for &_targettype row";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;    
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: valid value of _no_acute_change for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _no_acute_change must have a value of 1 when _sas_data_name of &_targettype is the cohort dataset";
           output _errors;
          end;
          drop _type;
         run; 
        %end;
       %end;
       %else %if &&&_cov_cohort_target_&_i = 0 %then %do;
        %if &&&_no_acute_val_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: valid value of _no_acute_change for &_targettype row";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;    
        %end;   
        %else %if &&&_no_acute_val_&_i = 0 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: valid value of _no_acute_change for &_targettype row";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;    
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: valid value of _no_acute_change for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: value of _no_acute_change of &_targettype row is neither 0 nor 1";
           output _errors;
          end;
          drop _type;
         run;    
        %end;    
       %end;
      
       proc append data=_errors base=_error_checks force; 
       run;
      
       proc datasets library = work nolist;
        delete _errors;
       quit;
      %mend _cov_no_acute_check;
      
      %_cov_no_acute_check(_covars_&_i,&&&_cov_variable_&_i);
      
      %macro _cov_behav_second_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
        call symput("_var_name_&_i",lowcase(trim(left(_variable))));
        call symput("_varfmt_val_&_i",_varfmt);     
        call symput("_behav_val_&_i",_behavior);
        call symput("_impute_val_&_i",_impute);
       run;
      
       %if &&&_behav_val_&_i = 1 %then %do;
        %if &&&_varfmt_val_&_i = N %then %do;
        
        *ods listing close;
        *ods output variables=_fmtcheck_b_&_i;
        proc contents data=&_cov_cohort_data out=_fmtcheck_b_&_i noprint;
        run;
        *ods listing;

         data _fmtcheck_b_&_i;
         set _fmtcheck_b_&_i;
         length type_c $4.;
         if type=1 then type_c ='Num';
         if type=2 then type_c='Char';
         drop type;
         rename type_c=type;
         run;

       
        data _null_;
         set _fmtcheck_b_&_i;
         if lowcase(Name) = "&&&_var_name_&_i";
         _fmt_target = "&&&_varfmt_val_&_i";
         if _fmt_target = 'N' then do;
          if upcase(type) = 'NUM' then do;
           call symput("_varfmt_check_&_i",1);     
          end;
          else do;
           call symput("_varfmt_check_&_i",0);        
          end;
         end;
         else do;
           call symput("_varfmt_check_&_i",0);                 
         end;
        run;
        
         %if &&&_varfmt_check_&_i = 1 %then %do;
          data _valid_cohort_&_i _invalid_cohort_&_i;
           set &_cov_cohort_data;
           if &&&_var_name_&_i = 0 then output _valid_cohort_&_i;
           else if &&&_var_name_&_i = 1 then output _valid_cohort_&_i;
           else output _invalid_cohort_&_i;
           keep &&&_var_name_&_i;
          run;
          
          data _invalid_cohort_&_i;
           set _invalid_cohort_&_i;
          run;
      
          %_mtanyobs;
          
          %let _invalid_cohort_values_&_i = &hasobs;
         
          %if &&&_invalid_cohort_values_&_i = 1 %then %do;       
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 1;
             _message = "ERROR: _behavior has a value of 1 but _variable in the cohort dataset contains values that are not 0 or 1 for &_targettype row";
             output _errors;
            end;
            drop _type;
           run;  
          %end;
          %else %do;
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 0;
             _message = "";
             output _errors;
            end;
            drop _type;
           run;  
          %end;
         %end;      
         %else %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
            _error_flag = 1;
            _message = "ERROR: _behavior has a value of 1 but _variable in the cohort dataset is not numeric for &_targettype row";
            output _errors;
           end;
           drop _type;
          run;             
         %end;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _behavior has a value of 1 but _varfmt has a value that is not N for &_targettype row";
           output _errors;
          end;
          drop _type;
         run;         
        %end;    
       %end;
      
       %else %if &&&_behav_val_&_i = 2 %then %do;
        %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _behavior has a value of 2 but _sas_data_name for &_targettype row is the same as the cohort dataset";
           output _errors;
          end;
          drop _type;
         run; 
        %end;
        %else %do;
         %if &&&_varfmt_val_&_i = N %then %do; 
          data _valid_cohort_&_i _invalid_cohort_&_i;
           set &_cov_cohort_data;
           if &&&_var_name_&_i = . then output _invalid_cohort_&_i;
           else output _valid_cohort_&_i;
           keep &&&_var_name_&_i;
          run;
         %end;
         %else %if &&&_varfmt_val_&_i = C %then %do; 
          data _valid_cohort_&_i _invalid_cohort_&_i;
           set &_cov_cohort_data;
           if &&&_var_name_&_i = '' then output _invalid_cohort_&_i;
           else if trim(left(&&&_var_name_&_i)) = '.' then output _invalid_cohort_&_i;
           else output _valid_cohort_&_i;
           keep &&&_var_name_&_i;
          run;     
         %end;  
              
         data _invalid_cohort_&_i;
          set _invalid_cohort_&_i;
         run;
      
         %_mtanyobs;
      
         %let _invalid_cohort_values_&_i = &hasobs;        
         %if &&&_invalid_cohort_values_&_i = 1 %then %do;  
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
            _error_flag = 1;
            _message = "ERROR: _behavior has a value of 2 but _variable in the cohort dataset contains missing values for &_targettype row";
            output _errors;
           end;
           drop _type;
          run;
         %end;
         %else %do;
          %if &&&_varfmt_val_&_i = N %then %do; 
           data _valid_cov_&_i _invalid_cov_&_i;
            set &&&_data_loc_&_i;
            if &&&_var_name_&_i = . then output _invalid_cov_&_i;
            else output _valid_cov_&_i;
            keep &&&_var_name_&_i;
           run;
          %end;
          %else %if &&&_varfmt_val_&_i = C %then %do; 
           data _valid_cov_&_i _invalid_cov_&_i;
            set &&&_data_loc_&_i;
            if &&&_var_name_&_i = '' then output _invalid_cov_&_i;
            else if trim(left(&&&_var_name_&_i)) = '.' then output _invalid_cov_&_i;
            else output _valid_cov_&_i;
            keep &&&_var_name_&_i;
           run;     
          %end;  
      
          data _invalid_cov_&_i;
           set _invalid_cov_&_i;
          run;
          
          %_mtanyobs;
          
          %let _invalid_cov_values_&_i = &hasobs; 
          %if &&&_invalid_cov_values_&_i = 1 %then %do;  
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 1;
             _message = "ERROR: _behavior has a value of 2 but _variable in the covariate dataset contains missing values for &_targettype row";
             output _errors;
            end;
            drop _type;
           run;
          %end;
          %else %do;
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 0;
             _message = "";
             output _errors;
            end;
            drop _type;
           run;       
          %end;      
         %end;
        %end;    
       %end;
       
       %else %if &&&_behav_val_&_i = 3 %then %do;
        %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
      
         %if &&&_varfmt_val_&_i = N %then %do; 
          data _invalid_cohort_&_i;
           stop;
          run;
         %end;
         
         %else %if &&&_varfmt_val_&_i = C %then %do; 
          data _valid_cohort_&_i _invalid_cohort_&_i;
           set &_cov_cohort_data;
           if trim(left(&&&_var_name_&_i)) = '.' then output _invalid_cohort_&_i;
           else output _valid_cohort_&_i;
           keep &&&_var_name_&_i;
          run;     
         %end;  
          
         data _invalid_cohort_&_i;
          set _invalid_cohort_&_i;
         run;
         
         %_mtanyobs;
         
         %let _invalid_cohort_values_&_i = &hasobs;    
         %if &&&_invalid_cohort_values_&_i = 1 %then %do;       
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
            _error_flag = 1;
            _message = "ERROR: _behavior has a value of 3 but _variable in the cohort dataset contains disallowed missing values for &_targettype row";
            output _errors;
           end;
           drop _type;
          run;  
         %end;
         %else %do;
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
            _error_flag = 0;
            _message = "";
            output _errors;
           end;
           drop _type;
          run;  
         %end;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _behavior has a value of 3 but _sas_data_name for &_targettype row is different from the cohort dataset";
           output _errors;
          end;
          drop _type;
         run; 
        %end;
       %end;
       
       %else %if &&&_behav_val_&_i = 4 or &&&_behav_val_&_i = 5 %then %do;
        %if &&&_data_loc_&_i = &_cov_cohort_data %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _behavior has a value of 4 or 5 but _sas_data_name for &_targettype row is the same as the cohort dataset";
           output _errors;
          end;
          drop _type;
         run; 
        %end;    
        %else %do;
         %if &&&_varfmt_val_&_i = N %then %do; 
          data _invalid_cohort_&_i;
           stop;
          run;
         %end;
         
         %else %if &&&_varfmt_val_&_i = C %then %do; 
          data _valid_cohort_&_i _invalid_cohort_&_i;
           set &_cov_cohort_data;
           if trim(left(&&&_var_name_&_i)) = '.' then output _invalid_cohort_&_i;
           else output _valid_cohort_&_i;
           keep &&&_var_name_&_i;
          run;     
         %end;      
         data _invalid_cohort_&_i;
          set _invalid_cohort_&_i;
         run;
      
         %_mtanyobs;
      
         %let _invalid_cohort_values_&_i = &hasobs;    
         %if &&&_invalid_cohort_values_&_i = 1 %then %do;  
          data _errors;
           length _checktype $100 
                  _error_flag 3
                  _message $400;
           set &_indash (keep = _type);
           if _n_ = 1;
           if _n_ = 1 then do;
            _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
            _error_flag = 1;
            _message = "ERROR: _behavior has a value of 4 or 5 but _variable in the cohort dataset contains values of . for &_targettype row";
            output _errors;
           end;
           drop _type;
          run;
         %end;      
         %else %do;
          %if &&&_varfmt_val_&_i = N %then %do; 
           data _valid_cov_&_i _invalid_cov_&_i;
            set &&&_data_loc_&_i;
            if &&&_var_name_&_i = . then output _invalid_cov_&_i;
            else output _valid_cov_&_i;
            keep &&&_var_name_&_i;
           run;
          %end;
          %else %if &&&_varfmt_val_&_i = C %then %do; 
           data _valid_cov_&_i _invalid_cov_&_i;
            set &&&_data_loc_&_i;
            if &&&_var_name_&_i = '' then output _invalid_cov_&_i;
            else if trim(left(&&&_var_name_&_i)) = '.' then output _invalid_cov_&_i;
            else output _valid_cov_&_i;
            keep &&&_var_name_&_i;
           run;     
          %end;  
      
          data _invalid_cov_&_i;
           set _invalid_cov_&_i;
          run;
          
          %_mtanyobs;
          
          %let _invalid_cov_values_&_i = &hasobs; 
          %if &&&_invalid_cov_values_&_i = 1 %then %do;  
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 1;
             _message = "ERROR: _behavior has a value of 4 but _variable in the covariate dataset contains disallowed missing values for &_targettype row";
             output _errors;
            end;
            drop _type;
           run;
          %end;
          %else %do;
           data _errors;
            length _checktype $100 
                   _error_flag 3
                   _message $400;
            set &_indash (keep = _type);
            if _n_ = 1;
            if _n_ = 1 then do;
             _checktype = "covariate: source datasets properly formatted given _behavior for &_targettype row";
             _error_flag = 0;
             _message = "";
             output _errors;
            end;
            drop _type;
           run;       
          %end;            
         %end;
        %end;
       %end;
      
       proc append data=_errors base=_error_checks force; 
       run;
      
       proc datasets library = work nolist;
        delete _errors _fmtcheck_b_&_i _valid_cohort_&_i _invalid_cohort_&_i _valid_cov_&_i _invalid_cov_&_i;
       quit;   
      
      %mend _cov_behav_second_check;
      
      %_cov_behav_second_check(_covars_&_i,&&&_cov_variable_&_i);
      
      %macro _cov_impute_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        if _behavior = 1 or _behavior = 2 or _behavior = 5 then do;
         call symput("_behav_cat_&_i",0);
        end;
        else if _behavior = 3 or _behavior = 4 then do;
         call symput("_behav_cat_&_i",1);
        end; 
        if _impute = . then do;
         call symput("_impute_val_&_i",9);      
        end;
        else if _impute in (1,2,3,4) then do;
         call symput("_impute_val_&_i",_impute);      
        end;
        else do;
         call symput("_impute_val_&_i",0);
        end;
       run;
      
       %if &&&_behav_cat_&_i = 0 %then %do;   
        %if &&&_impute_val_&_i = 9 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: _impute consistent with _behavior for &_targettype row";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;       
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: _impute consistent with _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: no value should be specified for _impute when the value of _behavior is 1,2 or 5 ";
           output _errors;
          end;
          drop _type;
         run;      
        %end;
       %end;
       
       %else %if &&&_behav_cat_&_i = 1 %then %do;
        %if &&&_impute_val_&_i = 9 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: _impute consistent with _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: a value of 1, 2 3 or 4 must be specified for _impute when the value of _behavior is 3 or 4";
           output _errors;
          end;
          drop _type;
         run;          
        %end;
        %else %if &&&_impute_val_&_i = 0 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: _impute consistent with _behavior for &_targettype row";
           _error_flag = 1;
           _message = "ERROR: _impute is defined as a value other than 1, 2, 3, 4 or null";
           output _errors;
          end;
          drop _type;
         run;  
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard: _impute consistent with _behavior for &_targettype row";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;         
        %end;
       %end;
       
       proc append data=_errors base=_error_checks force; 
       run;
      
       proc datasets library = work nolist;
        delete _errors;
       quit;   
      
      %mend _cov_impute_check;
      
      %_cov_impute_check(_covars_&_i,&&&_cov_variable_&_i);
      

     %macro _cov_varclass_check(_dbdata,_targettype);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        if _varfmt = 'N' then do;
         if _varclass = 'cat' then do;
          call symput("_varclass_cat_&_i",1);
         end;
         else if _varclass = 'cont' then do;
          call symput("_varclass_cat_&_i",1);
         end;
         else do;
          call symput("_varclass_cat_&_i",0);
         end;
        end;
        else if _varfmt = 'C' then do;
         if _varclass = 'cat' then do;
          call symput("_varclass_cat_&_i",3);
         end;
         else do;
          call symput("_varclass_cat_&_i",2);
         end;
        end; 
      
       run;
      
       %if &&&_varclass_cat_&_i = 0 %then %do;   
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: _varclass consistent with _varfmt for &_targettype row";
          _error_flag = 1;
          _message = "ERROR: _varfmt has a value of N but _varclass does not have a value of either cat or cont";
          output _errors;
         end;
         drop _type;
        run;   
       %end;
       %else %if &&&_varclass_cat_&_i = 2 %then %do;   
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: _varclass consistent with _varfmt for &_targettype row";
          _error_flag = 1;
          _message = "ERROR: _varfmt has a value of C but _varclass does not have a value of cat";
          output _errors;
         end;
         drop _type;
        run;   
       %end;
       %else %do;
        data _errors;
         length _checktype $100 
                _error_flag 3
                _message $400;
         set &_indash (keep = _type);
         if _n_ = 1;
         if _n_ = 1 then do;
          _checktype = "dashboard: _varclass consistent with _varfmt for &_targettype row";
          _error_flag = 0;
          _message = "";
          output _errors;
         end;
         drop _type;
        run;      
       %end;
       
       proc append data=_errors base=_error_checks force; 
       run;
      
       proc datasets library = work nolist;
        delete _errors;
       quit;   
      
      %mend _cov_varclass_check;
      
      %_cov_varclass_check(_covars_&_i,&&&_cov_variable_&_i);
     

  %macro _cov_value_check(_dbdata);
      
       data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_data_loc_&_i",trim(left(_sas_data_name)));
        call symput("_varfmt_val_&_i",_varfmt);
		call symput("_impute_val_&_i",_impute);
        call symput("_value_val_&_i",_value);
       run;

       proc contents data=&_dbdata out=_value_check_&_i noprint;
       run;

	   *when _value is not blank and _impute =1 check if _value matchs _varfmt;
      %if &&&_impute_val_&_i=1  and  &&&_value_val_&_i ne  %then %do;
       
         data _value_check_&_i;
         set _value_check_&_i;
          if Type =1 then value_type='N';
 		  else value_type='C';
 
		  if value_type ="&&&_varfmt_val_&_i" then  call symput("_value_check_&_i",1);     
            else  call symput("_value_check_&_i",0);        
        run;
      %end;

      %else %do; 
        %goto exit;
	  %end;

     %if &&&_value_check_&_i = 1 %then %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard:when _inpute =1, non-missing _value matches _varfmt";
           _error_flag = 0;
           _message = "";
           output _errors;
          end;
          drop _type;
         run;
        %end;
        %else %do;
         data _errors;
          length _checktype $100 
                 _error_flag 3
                 _message $400;
          set &_indash (keep = _type);
          if _n_ = 1;
          if _n_ = 1 then do;
           _checktype = "dashboard:when _inpute =1, non-missing _value matches _varfmt";
           _error_flag = 1;
           _message = "ERROR: Non-missing _value in covarite row does not match the format specified by _varfmt";
           output _errors;
          end;
          drop _type;
         run;
        %end;
       
       
       proc append data=_errors base=_error_checks force; 
       run;
       
       proc datasets library = work nolist;
        delete _errors _value_check_&_i;
       quit;
	   %exit: 
      %mend _cov_value_check;
      
  %_cov_value_check(_covars_&_i);

  

 %macro _cov_value_length (_dbdata);

     data _null_;
        set &_dbdata;
        if _n_=1;
        call symput("_value_val_&_i",_value);
       run;

   *ods output variables=_value_check_&_i;
    proc contents data=&_dbdata out=_value_check_&_i noprint ;
    run;
   *ods listing;
 
   data _null_;
      set _value_check_&_i;
	  where Name='_value';
      if length > 20 then do;
       call symput("_length_check_&_i",0);
      end;
      else do;
       call symput("_length_check_&_i",1);
      end;
     run;


     %if &&&_length_check_&_i = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: the length of _value in covariate row is less than 20";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
     %end;
     %else %if &&&_length_check_&_i = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: the length of _value  in covariate row  is less than 20";
        _error_flag = 1;
        _message = "ERROR: _value contains a value with length exceeding 20 characters";
        output _errors;
        call symput("_maj_error",2); 
       end;
       drop _type;
      run;
     %end;
   proc append data=_errors base=_error_checks force; 
   run;
   
  proc datasets library = work nolist;
   delete _errors _value_check_&_i;
   quit;
  
  %mend _cov_value_length ;
  
  %_cov_value_length (_covars_&_i); 


  *NEW Macro added 1/2016 See macro E14.1 in excel macro list;
 %macro _varfmt_confirm_expos(_dbdata,_targettype);
    data &_targettype._varfmt;
     set &_dbdata;
     if lowcase(_type) = "&_targettype" then output &_targettype._varfmt;
    run; 
   
    data &_targettype._varfmt;
     set &_targettype._varfmt;

      call symput("_&_targettype._var",lowcase(trim(left(_variable))));
      call symput("_&_targettype._sourcedata",trim(left(_sas_data_name)));
      
    if _varfmt ='' then do;
     call symput("_&_targettype._vfmtcheck",2);
	 call symput("_&_targettype._vfmt_type",0);
	end;

     else if _varfmt in ('C','N') then do;
      call symput("_&_targettype._vfmtcheck",1);
      if _varfmt in ('C') then call symput("_&_targettype._vfmt_type",2);
      if _varfmt in ('N') then call symput("_&_targettype._vfmt_type",1);    
     end;
    else do;
     call symput("_&_targettype._vfmtcheck",0);
     call symput("_&_targettype._vfmt_type",0);
    end;
   run;

     proc contents data=&&&_&_targettype._sourcedata out=_varfmt_&_targettype noprint;
     run;
    
      data _varfmt_&_targettype;
      set _varfmt_&_targettype;
      length type_c $4.;
      if type=1 then type_c ='Num';
      if type=2 then type_c='Char';
      drop type;
      rename type_c=type;
	  if lowcase(Name) = "&&&_&_targettype._var";
      run;
    
	  %_nobs_msm(_varfmt_&_targettype);
    %put &_nobs;

    %if &_nobs = 0 %then %do;
      data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;    
    %end;

    %if &&&_&_targettype._vfmtcheck =2  %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;      
     %end;
    

    %if &&&_&_targettype._vfmtcheck = 1 %then %do;
     
    
     %if &&&_&_targettype._vfmt_type = 1 & &_nobs ne 0 %then %do;
      data _varfmt_&_targettype;
       set _varfmt_&_targettype;
       if type = "Num" then do;
        call symput("_&_targettype._vfmtclear",1);        
       end;
       else do;
        call symput("_&_targettype._vfmtclear",0);       
       end;
      run;
      %if &&&_&_targettype._vfmtclear = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;      
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 1;
         _message = "ERROR: _varfmt in row &_targettype does not match format of variable given in _variable";
         output _errors;
        end;
        drop _type;
       run;  
      %end;
     %end;   
     %else %if &&&_&_targettype._vfmt_type = 2 &  &_nobs ne 0 %then %do;
      data _varfmt_&_targettype;
       set _varfmt_&_targettype;
       if type = "Char" then do;
        call symput("_&_targettype._vfmtclear",1);        
       end;
       else do;
        call symput("_&_targettype._vfmtclear",0);       
       end;
      run;

      %if &&&_&_targettype._vfmtclear = 1 %then %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 0;
         _message = "";
         output _errors;
        end;
        drop _type;
       run;      
      %end;
      %else %do;
       data _errors;
        length _checktype $100 
               _error_flag 3
               _message $400;
        set &_dbdata (keep = _type);
        if _n_ = 1;
        if _n_ = 1 then do;
         _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
         _error_flag = 1;
         _message = "ERROR: _varfmt in row &_targettype does not match format of variable given in _variable";
         output _errors;
        end;
        drop _type;
       run;  
      %end;
     %end;   
    %end;
   
     %if &&&_&_targettype._vfmtcheck= 0 %then %do;
     data _errors;
      length _checktype $100 
             _error_flag 3
             _message $400;
      set &_dbdata (keep = _type);
      if _n_ = 1;
      if _n_ = 1 then do;
       _checktype = "dashboard: _varfmt in row &_targettype specifies a valid format or is missing";
       _error_flag = 1;
       _message = "ERROR: _varfmt in row &_targettype is neither C nor N nor missing";
       output _errors;
      end;
      drop _type;
     run;  
    %end;  

	data _null_;
    set _errors;
    call symput("_fmt_error", put(_error_flag,1.));
	run;
   %put &_fmt_error;

	%if &_fmt_error =0 & &&&_&_targettype._vfmt_type ne 0 & &_nobs ne 0 %then %do;
     data _null_;
      set _varfmt_&_targettype;
      if length > 20 then do;
       call symput("_&_targettype._lcheck",0);
      end;
      else do;
       call symput("_&_targettype._lcheck",1);
      end;
     run;

      %if &&&_&_targettype._lcheck = 1 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _variable in row &_targettype specifies a valid format or is missing";
        _error_flag = 0;
        _message = "";
        output _errors;
       end;
       drop _type;
      run;
     %end;

     %else %if &&&_&_targettype._lcheck = 0 %then %do;
      data _errors;
       length _checktype $100 
              _error_flag 3
              _message $400;
       set &_dbdata (keep = _type);
       if _n_ = 1;
       if _n_ = 1 then do;
        _checktype = "dashboard: _varfmt in row exposure specifies a valid format or is missing and if _varfmt=C then the length of the target variable in the exposure data is less than 20 characters";
        _error_flag = 1;
        _message = "ERROR: _varfmt in row exposure  matches type of variable given in _variable but the target variable in the exposure data set contains a value with length exceeding 20 characters";
        output _errors;
        call symput("_maj_error",2); 
       end;
       drop _type;
      run;
     %end;
    %end;
  
    
    proc append data=_errors base=_error_checks force; 
    run;
    
    proc datasets library = work nolist;
     delete &_targettype._varfmt _errors;
    quit;
   
   %mend _varfmt_confirm_expos;
%_varfmt_confirm_expos(&_indash,exposure); 
 
   
     %end;
    %end;
   
   %end;
  
  
  %end;
  
 %end;
 

 /*
 proc print data=_error_checks;
  title "error checks dataset";
 run;
 */
 

 data _errors_detected_print;
  set _error_checks;
  if _error_flag = 1;
 run;

 proc print data=_errors_detected_print;
  var _checktype _message;
  title "Errors detected in _MSMstructure macro run";
 run;

/* save copy of error checking dataset;


 data _error_output_dir;
  set &_indash;
  if upcase(_type) = 'OUTDATA';
  _dir = scan(_sas_data_name,1);
  call symput("_outputdir",trim(left(_dir)));
 run;

 data &_outputdir.._msmstructure_error_checks;
  set _error_checks;
 run;


 %macro _output_errors_dataset;
  data _outdata_errors;
   set _errors_detected_print;
   if _checktype = 'dashboard: outdata row present' then output _outdata_errors;
   else if _checktype = 'dashboard: outdata row specifies a valid output location' then output _outdata_errors;
  run;

  %_mtanyobs;
          
  %let _error_count_outdata = &hasobs; 
  %if &_error_count_outdata = 1 %then %do;
   %put 'MSMSTRUCTURE MACRO NOTE: ERROR IN OUTDATA ROW MEANS NO ERROR DATASET SAVED TO DISK'; 
  %end;
  %else %if &_error_count_outdata = 2 %then %do;
   %put 'MSMSTRUCTURE MACRO NOTE: ERROR IN OUTDATA ROW MEANS NO ERROR DATASET SAVED TO DISK';  
  %end;
  %else %do;
   data _error_output_dir;
    set &_indash;
    if upcase(_type) = 'OUTDATA';
    _dir = scan(_sas_data_name,1);
    call symput("_outputdir",trim(left(_dir)));
   run;
   
   data &_outputdir.._msmstructure_error_checks;
    set _error_checks;
   run; 
  %end;

 %mend;
 %_output_errors_dataset;
  */

%mend _bad_inputs_checking;





