/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/

%macro _basetable_call_msm(codata,dbdata,idvar,byvar,out1,out2,out3);

%macro rollback_impute;

* imputation rollback code for baseline table calculations;

 data dset_nonimputed;
  set &codata;
 run;

 data _cov_list;
  set &dbdata;
  if _remove ne 1;
  if trim(left(upcase(_type))) = 'COVARIATE' then output _cov_list;
 run;

  data _cov_list;
   set _cov_list;
   _order0=_N_;
  run;
                                                         
  %_nobs_msm(_cov_list);
  %global _nobs_cov_list;
  %let _nobs_cov_list = %eval(&_nobs);

* Convert values passed from &_varsdata into macro variables;
   %do _i=1 %to &_nobs_cov_list;

    %global _covname&_i;
    %let _covname&_i = 0;
    %global _covbehavior&_i;
    %let _covbehavior&_i = 0;
    %global _covfmt&_i;
    %let _covfmt&_i=0;
    %global _covimpute&_i;
    %let _covimpute&_i=0;
   
    data _null_;
     set _cov_list;
     if _n_=&_i;
     call symput("_covname&_i",trim(left(_variable)));
     call symput("_covbehavior&_i",trim(left(_behavior)));
     call symput("_covfmt&_i",trim(left(_varfmt)));
     call symput("_covimpute&_i",trim(left(_impute)));
    run;
   
    %put _i=&_i;
    %put _covname=&&&_covname&_i;
    %put _covbehavior&=&&&_covbehavior&_i;
    %put _covfmt=&&&_covfmt&_i;
    %put _covimpute=&&&_covimpute&_i;
    %put ;
    %put ;
    %put ;
    
    %if &&&_covbehavior&_i = 3 %then %do;
     data dset_nonimputed;
      set dset_nonimputed;
      %if &&&_covfmt&_i = N %then %do;
       if _imp_&&&_covname&_i = 1 then _&&&_covname&_i = .;
      %end;
      %else %if &&&_covfmt&_i = C %then %do;
       if _imp_&&&_covname&_i = 1 then _&&&_covname&_i = '';
      %end;
     run;
    %end;

    %else %if &&&_covbehavior&_i = 4 %then %do;
     data dset_nonimputed;
      set dset_nonimputed;
      %if &&&_covfmt&_i = N %then %do;
       if _imp_&&&_covname&_i = 1 then _&&&_covname&_i = .;
      %end;
      %else %if &&&_covfmt&_i = C %then %do;
       if _imp_&&&_covname&_i = 1 then _&&&_covname&_i = '';
      %end;      
     run;
    %end;

    
   %end;

%mend rollback_impute;
%rollback_impute;

data _control_file;
 length _varname $ 40
        _vartype $ 15
        _class $ 4
        _cat $ 100
        _clabel $ 100
        ;
 set &dbdata;
 if trim(left(upcase(_type))) in ('COVARIATE','EOFTYPE');
 if trim(left(upcase(_type))) = 'EOFTYPE' then _varname = trim(left(_variable));
 else if trim(left(upcase(_type))) = 'COVARIATE' then _varname = cat('_',trim(left(_variable)));
 
 if trim(left(upcase(_type))) = 'EOFTYPE' then _vartype = 'Outcome';
 else _vartype = 'Covariate';

 if trim(left(upcase(_type))) = 'EOFTYPE' then _class = 'cat';
 else _class = lowcase(trim(left(_varclass)));

 keep _varname _vartype _class _cat _clabel;
run;


%_nobs_msm(_control_file);
%global _nobs_varlist;
%let _nobs_varlist = %eval(&_nobs);

%do _i=1 %to &_nobs_varlist;
 data _null_;
  set _control_file;
  if _n_ = &_i;
  call symput("_cov_class_&_i",trim(left(_class)));
  call symput("_cov_var_&_i",trim(left(_varname)));
 run;

 %if &&&_cov_class_&_i = cat %then %do;
  ods listing close;
  ods output onewayfreqs=_freq_var_&_i;
   proc freq data=dset_nonimputed;
    tables &&&_cov_var_&_i;
   run;
  ods listing;
  
  proc sort data=_freq_var_&_i;
   by table &&&_cov_var_&_i;
  run;

  data _freq_var_&_i;
   length _cat $ 100;
   set _freq_var_&_i;
   by table;
   retain _cat;
   if first.table then _cat = &&&_cov_var_&_i;
   else _cat = catx(" ", _cat, &&&_cov_var_&_i);
  run;
  
  data _cat_var_&_i;
   length _clabel $ 100;
   set _freq_var_&_i;
   by table;
   if last.table;
   _varname = "&&&_cov_var_&_i";
   _clabel = _cat;
   keep _cat _clabel _varname;
  run;
  
  proc print data=_cat_var_&_i;
  run;
  
  proc sort data=_cat_var_&_i;
   by _varname;
  run;
  
  proc sort data=_control_file;
   by _varname;
  run;
  
  data _control_file;
   merge _control_file(in=a) _cat_var_&_i(in=b);
   by _varname;
   if a;
  run;
 %end;

 %else %do;
  data _control_file;
   set _control_file;
   if _n_ = &_i then do;
    _cat = '';
    _clabel = '';   
   end;
  run;
 %end;
 
%end;

data _addons;
 set _control_file;
 if _n_ <= 3;
 if _n_ = 1 then do;
  _varname = '_exposure_ever_never';
  _vartype = 'Exposure';
  _class = 'cat';
  _cat = '1';
  _clabel = '1';
 end;
 else if _n_ = 2 then do;
  _varname = '_exposure_interval';
  _vartype = 'ExposureFirst';
  _class = 'cont';
  _cat = '';
  _clabel = '';
 end;
 else if _n_ = 3 then do;
  _varname = '_intmax';
  _vartype = 'Followup';
  _class = 'cont';
  _cat = '';
  _clabel = '';
 end;
run;

proc append data=_addons base=_control_file force; 
run;

data _control_file;
 set _control_file;
 if _cat = '0 1' then _cat = '1';
 if _clabel = '0 1' then _clabel = '1';
 remove = .;
 vlabel = _varname;
 clabel = tranwrd(_clabel,' ','*');
 if _class = 'cont' then do;
  roundnum = '0.1';
  stattype = '1*1*1';
  type = 'N';
 end;
 else do;
  roundnum = '';
  stattype = '';
  type = 'C';
 end;
 rename _varname = varname
 _vartype = vartype
 _cat = cat
 ; 
 drop _clabel;
run;

proc print data=_control_file;
run;

* insert output directory code here;

data _output_dir;
 set &dbdata;
 if upcase(_type) = 'OUTDATA';
 _dir = scan(_sas_data_name,1);
 call symput("_outputdir",trim(left(_dir)));
run;

%let _outputpath = %sysfunc(pathname(&_outputdir));
%put &_outputpath;

data _null_;
 set _control_file;
 if upcase(vartype) = 'OUTCOME';
 call symput("headers",clabel);
run;

data table_input_data;
 set dset_nonimputed;
run;

data mod1files;
 set _control_file;
 if upcase(vartype) ='FOLLOWUP' then delete;
 if upcase(vartype) ='EXPOSUREFIRST' then delete;
 *remove ever/never exposure from overall table;
 if upcase(vartype) ='EXPOSURE' then delete;
run;

%_msm_auto_basetable(table_input_data,mod1files,&idvar,,,
&_outputpath,
&out1);

data mod2files;
 set _control_file;
 if upcase(vartype) ne 'FOLLOWUP' then delete;
run;

%_msm_auto_basetable(table_input_data,mod2files,&idvar,&byvar,&headers,
&_outputpath,
&out2);


data mod3files;
 set _control_file;
 if upcase(vartype) ne 'EXPOSUREFIRST' then delete;
run;

*for binary exposure ;
%if &_exp_multi_level1=0 %then %do;  

data exposed_only;
 set table_input_data;
run;

 proc sort data=exposed_only;
  by &idvar descending _exposure _intnum;
 run;

 data exposed_only;
  set exposed_only;
  by &idvar;
  retain _exposure_ever_never;
  if first.&idvar then do;
   _exposure_first = 1;
   if _exposure = 1  then _exposure_ever_never = 1;
   else if _exposure = 0 then _exposure_ever_never = 0;
  end; 
  else do; 
   _exposure_ever_never = _exposure_ever_never;
   _exposure_first = 0;
  end;
 run;

 data exposed_only;
  set exposed_only;
  if _exposure_ever_never = 1;
 run;

 proc sort data=exposed_only;
  by &idvar descending _exposure_first _intnum;
 run;

 data exposed_only;
  set exposed_only;
  by &idvar;
  retain _exposure_interval;
  if first.&idvar then do;
   _exposure_interval = _intnum;
  end; 
  else do; 
   _exposure_interval = _exposure_interval;
  end;
 run;
 
 proc sort data=exposed_only;
  by &idvar _intnum;
 run;


%_msm_auto_basetable(exposed_only,mod3files,&idvar,&byvar,&headers,
&_outputpath,
&out3);
%end;


*for categorical exposure;
%if &_exp_multi_level1=1 %then %do;  

 proc sort data=table_input_data out=exposed_only;
  where _exposure not in ('','Not exposed');
  by  &idvar  _exposure  _intnum;
 run;

data exposed_only;
  set exposed_only;
  by &idvar _exposure;
  retain _exposure_interval;

  if first._exposure then do;
  _exposure_first = 1;
  _exposure_interval = _intnum;
  end;
  else do;
   _exposure_first = 0;
   _exposure_interval = _exposure_interval;
   end;
  _exposure_ever_never = 1;
 run;

%_msm_auto_basetable_multiexp(exposed_only,mod3files,&idvar,&byvar,&headers,
&_outputpath,
&out3);
%end;

%mend _basetable_call_msm;
