/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


/****************** MSM Dataset Macro - Clean up macro ***************************
 To clean up final dataset and add on time-independent baseline variables

 Create 2 different datasets:
	1. Dataset for reporting - contains no interim variables
		- contains some variables for reporting purposes
			e.g. _intmax for reporting max intervals per person
			
	2. Analysis ready dataset - for running models
		- contains only needed variables for modeling 
		- no interim or reporting variables
 Input data:
 	_msmdata = completed MSM dataset
 	_outdata = name user calls their final dataset
 	_basevars = file location and name of the input variables dataset for
 			time independent variables
				- assumes _variable name same in raw dataset
 	_ident = patient identifier
 	
**********************************************************************************/

%macro _clean_up(_msmdata,_outdata,_basevars,_ident);

data _workdata;
	set &_msmdata;
	drop _acnt _inflect ;
run;
proc sort data=_workdata;
	by &_ident;
run;

** creating dataset for reporting;
data _reporting_&_outdata;
	set _workdata;
run;

** creating dataset ready for analysis/modeling - keeping in indicator for imputation (_imp_:) per Romain 8/19/2013;
Data _workdata;
	set _reporting_&_outdata;
	
	%if &_exp_multi_level1=1 %then %do;
	drop _intmax &_studystart &_studyend  _prev_exposure _d_date _days_exposed _exp_comb: _last_int_begin 
	_last_int_percent _past_out _percent _prev_exp_begin _prev_exp_comb _prev_exp_end _interval_percent: _case
		_dt_: _exp_: _out_: level_name max_num sum_percent tie_last_end tie_level ;
	 %if &_exp_fmt1 = N  %then %do;
       if _exposure ='Not exposed ' then _exposure_n=0;
        else if _exposure =' ' then _exposure_n=.;
        else _exposure_n=input(compress(_exposure), 8.);
       drop _exposure;
       rename _exposure_n=_exposure;
     %end;
   %end;

  %else %do;
	drop _intmax &_studystart &_studyend  _d_date _days_exposed _exp_comb: _last_int_begin 
	_last_int_percent _past_out _percent _prev_exp_begin _prev_exp_comb _prev_exp_end _interval_percent: _case
		_dt_: _exp_: _out_: i level_name1  
	;
	%end;
run;

%mend;


