/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


/****** MSM MAcro 3 - covariates for Binary Exposure with periods of change ******
**** Applies predetermined logics to Exposures
	- Takes dataset from Macro 2: _exposureint2.sas
	- determines when exposure takes place within intervals
Need:
	_in_conf_case2 = dataset created in _exposureint.sas with 
		appropriate exposure data
	_ident = unique patient identifier
	_exp_start = start date of exposure of interest
	_exp_end = end data of exposure of interest
	_percent = X% of days in interval exposure is active to count as exposed (must be decimal)
			-default percent = 50%
Other variables:
	_acnt_num= keeps count of number of intervals a stretches over , kept track by _acnt
	_achange= where a new exposure is considered to happen in the interval
	_prev_exp = previous exposure result
	_ideal_dt = date of the covariate to be used in the interval, can be the comorb or previous comorbs
	
****************************************************************/

%macro _conf_case2_binary (_in_conf_case2,_comorb,_comorbdt,_outcome_dt,_studyend,_ident,_unitdays,_exp_dt,_test);

proc sort data=&_in_conf_case2 out=_conf_case2 nodupkey;
	by &_ident _acnt _intnum _intstart &_comorbdt;
run;


proc sort data=_conf_case2;
	by &_ident _intnum &_comorbdt _acnt;
run;

data _conf_case2;
	set _conf_case2;
	by &_ident _intnum &_comorbdt _acnt;
	retain _acnt_num;
	 
 
	_prev_exp=lag1(_exposure);
	_lag_int=lag1(_intnum);
	
	if first.&_ident then do;
	  %if  &_exp_multi_level1=1 %then %do;  _prev_exp='Not exposed'; %end;
   	  %else %do; _prev_exp=0; %end;
		 _lag_int=0;
	end;
	_achange=0;
	if _prev_exp ne _exposure then _achange=1;
	if first.&_ident and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) then _achange=1;
	
	if first.&_ident then do;
	
     %if  &_exp_multi_level1=1 %then %do;  
       if  missing(_exposure)=0 and _exposure ne 'Not exposed' then _acnt_num=1;
       else if _exposure = 'Not exposed'   then _acnt_num=0;
	 %end;

	 %else %do;
      if  _exposure = 1 then _acnt_num=1;
       else if _exposure = 0   then _acnt_num=0;
	 %end;
     
	end;
	else if not first.&_ident then do;
		if _prev_exp ne _exposure then _acnt_num=_acnt_num+1;
		else if _prev_exp=_exposure and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _lag_int ne _intnum then _acnt_num=_acnt_num+1;
		else _acnt_num=_acnt_num;
	end;

	keep &_ident _acnt _intnum _acnt_num _achange _prev_exp _lag_int &_comorbdt;
run;
	
proc sort data=&_in_conf_case2 out=_conf;
	by &_ident _intnum _acnt &_comorbdt;
run;
proc sort data=_conf_case2; 
	by &_ident _intnum _acnt &_comorbdt;
run;

data _conf_binary;
	merge _conf (in=a)
		_conf_case2 (in=b);
		by &_ident _intnum _acnt &_comorbdt;
	if a;
run;

* Prepare input dataset;
 proc sort data=_conf_binary;
  by &_ident _intnum _intstart &_comorbdt;
 run;

*** for first exposure;
*start of cases creation;
*** ideal_dt=date and covariate value that should be used based on latest value available;
*** prev_cdate=previous covariate value available;
*** first_cdt=first covariate value available in the interval;
*** final_dt = previous picked covariate date;
data _workdata_conf;
	set _conf_binary;
		by &_ident _intnum _intstart &_comorbdt;
	_prev_c=lag1(&_comorb);
	_prev_cdate=lag1(&_comorbdt);

	format _prev_cdate mmddyy10.;
	
%if &&&_varfmt&_i=N %then %do;
	if first.&_ident then do;
		_prev_c=.;		
		_prev_cdate=.;
	end;
%end;
%else %if &&&_varfmt&_i=C %then %do;
	if first.&_ident then do;
		_prev_c='';		
		_prev_cdate=.;
	end;
%end;
run;

** to take care of cases 13 and 14;
proc sort data=_workdata_conf (drop=_case);
	by &_ident descending _intnum  descending &_comorbdt;
run;

data _workdata_conf;
	set _workdata_conf;
	by &_ident descending _intnum descending &_comorbdt;
	retain _last_exp _last_int;

	if _intnum=_intmax then do;
		_last_exp=_d_date;
		_last_int=_last_int_begin;
	end;
	else do;
		_last_exp=_last_exp;
		_last_int=_last_int;
	end; 
run;
proc sort data=_workdata_conf;
	by &_ident _intnum &_comorbdt;
run;
data _workdata_conf;
	set _workdata_conf;
	by &_ident _intnum &_comorbdt;
	retain _prev2_cdt _prev2;
	
	if first.&_ident then do;
		_prev2_cdt=.;
		%if &&&_varfmt&_i=N %then %do;
		_prev2=.;
		%end;
		%else %if &&&_varfmt&_i=C %then %do;
		_prev2='                    ';
		%end;
	end;
	if _no_acute_change=1 then do;  
	if first._intnum then do;
		if _last_exp=. then do;
			if (_last_int-&_unitdays)<=&_comorbdt<=_last_int then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else if _last_exp ne . then do;
			if &_comorbdt<=_last_exp and (&_studyend-&_unitdays*2)<=&_comorbdt<=_last_exp then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
	end;
	else do;
		if _last_exp=. then do;
			if _prev2_cdt<&_comorbdt and (_last_int-&_unitdays)<=&_comorbdt<(_last_int) then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else if _last_exp ne . then do;
			if _prev2_cdt<&_comorbdt and &_comorbdt<=_last_exp and (&_studyend-&_unitdays*2)<=&_comorbdt<=_last_exp then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else do;
			_prev2_cdt=_prev2_cdt;
			_prev2=_prev2;
		end;
	end;
	end;
	else if _no_acute_change=0 then do; 
	if first._intnum then do;
		if _last_exp=. then do;
			if (_last_int-&_unitdays)<=&_comorbdt<_last_int then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else if _last_exp ne . then do;
			if &_comorbdt<_last_exp and (&_studyend-&_unitdays*2)<=&_comorbdt<_last_exp then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
	end;
	else do;
		if _last_exp=. then do;
			if _prev2_cdt<&_comorbdt and (_last_int-&_unitdays)<=&_comorbdt<(_last_int) then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else if _last_exp ne . then do;
			if _prev2_cdt<&_comorbdt and &_comorbdt<_last_exp and (&_studyend-&_unitdays*2)<=&_comorbdt<_last_exp then do;
				_prev2_cdt=&_comorbdt;
				_prev2=&_comorb;
			end;
		end;
		else do;
			_prev2_cdt=_prev2_cdt;
			_prev2=_prev2;
		end;
	end;
	end;
	format _last_int _last_exp _prev2_cdt mmddyy10.;
run;
** end of case 13 14 correction;

proc sort data=_workdata_conf (drop=_last_exp _last_int);
	by &_ident _intnum _intstart &_comorbdt;
run;

data _workdata_conf;
	set _workdata_conf;
		by &_ident _intnum _intstart &_comorbdt;
		retain _first_comorb _first_cdt _ideal_dt _ideal _final_dt;
	if first._intnum then do;
		if first.&_ident then do;
			_first_cdt=&_comorbdt;
			_first_comorb=&_comorb;
			_ideal_dt=&_comorbdt;
			_ideal=&_comorb;
			_final_dt=&_comorbdt;
		end;
		else do;
			_first_cdt=_prev_cdate;
			_first_comorb=_prev_c;
		end;
	end;
	else do;
		_first_cdt=_first_cdt;
		_first_comorb=_first_comorb;
		_ideal_dt=_ideal_dt;
		_ideal=_ideal;
		_final_dt=_final_dt;
	end;
	
if _acnt_num=0 then do;
*case1 - no change in exposure;
	if _intnum=0 then do;
 	  %if &_efirst1 = 1 %then %do;  
 	  	if first.&_ident and %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) and _acnt=0 then do;
			_conf=&_comorb;
			_conf_dt=&_comorbdt;
		end;
		else if %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) and _acnt=0 and 0<&_comorbdt<=_intstart then do;
			_conf=&_comorb;
			_conf_dt=&_comorbdt;				
		end;
		else if %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) and _acnt=0 and 0<_prev_cdate<=_intstart then do;
			_conf_dt=_prev_cdate;
			_conf=_prev_c;
		end; 
	  %end;
 	  %if &_efirst1 = 0 %then %do;  
 	  	if first.&_ident and %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) then do;
			_conf=&_comorb;
			_conf_dt=&_comorbdt;
		end;
		else if %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) and _acnt=0 and 0<&_comorbdt<=_intstart then do;
			_conf=&_comorb;
			_conf_dt=&_comorbdt;				
		end;
		else if %sysfunc(ifc(&_exp_multi_level1=1,_exposure='Not exposed',_exposure=0)) and _acnt=0 and 0<_prev_cdate<=_intstart then do;
			_conf_dt=_prev_cdate;
			_conf=_prev_c;
		end;
	  %end;
	  if _conf_dt ne . then _final_dt=_conf_dt;
	  else _final_dt=_final_dt;
	  _case=1;
	end;
 end;
 	if _acnt_num<=1 then do;
	 %if &_efirst1 = 1 %then %do;  
	 %put "efirst1=&_efirst1";
*case 2,4,8 - first exposure change;
	if _intnum<=_intmax and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _acnt_num=1 and _censor ne 1 then do; 
		if _no_acute_change=1 then do; 					 *if non-acute exp, no acute confounder;
		if &_comorbdt<=&_exp_dt and _prev_cdate<=&_exp_dt and _first_cdt<=&_exp_dt then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=&_exp_dt and _first_cdt<=&_exp_dt then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<=&_exp_dt then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
			
			if &_comorbdt ne . and &_comorbdt<=_ideal_dt<=&_exp_dt and &_exp_dt<=_intstart and _intnum=0 then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=&_exp_dt then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=&_exp_dt and _final_dt<_ideal_dt then do;  *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do; 				 *if acute exp;
		if &_comorbdt<&_exp_dt and _prev_cdate<&_exp_dt and _first_cdt<&_exp_dt then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<&_exp_dt and _first_cdt<&_exp_dt then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<&_exp_dt then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
			
			if _intnum=0 and &_comorbdt ne . and &_comorbdt<=_ideal_dt<=&_exp_dt and &_exp_dt<=_intstart then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _intnum=0 and &_comorbdt ne . and &_comorbdt<=&_exp_dt and &_exp_dt<=_intstart then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if &_comorbdt ne . and _prev_cdate<&_comorbdt<&_exp_dt then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<&_exp_dt and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		if _intnum=0 and _intnum<_intmax then _case=2;
		else if _intnum=_intmax and _past_out=1 then _case=8;
		else _case=4;
	end;
	*case 9;
	else if _intnum=_intmax and _past_out=1 and &_outcome_dt<=&_studyend and &_exp_dt>&_outcome_dt then do;
		if &_comorbdt<_intstart and _prev_cdate<=_intstart and _first_cdt<_intstart then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_intstart and _first_cdt<=_intstart then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_intstart then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if &_comorbdt ne . and _prev_cdate<&_comorbdt<_intstart then do;
			_conf_dt=&_comorbdt;
			_conf=&_comorb;
			if _conf_dt ne . then _final_dt=_conf_dt;;
		end;
		else if _ideal_dt<_intstart and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do = changed to look back up to last L(t) designation;
			_conf_dt=_ideal_dt;
			_conf=_ideal;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
	_case=9;
	end;
	%end;
	
	%else %if &_efirst1=0 %then %do; 	** follow case 10/11 for case 2 and 4;
		 %put "efirst1=&_efirst1";
	if _intnum<_intmax and _intnum=0 and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _acnt_num=1 and _censor ne 1 and _interval_percent>=_percent then do;
		if &_comorbdt<=&_exp_dt and _prev_cdate<=&_exp_dt and _first_cdt<=&_exp_dt then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=&_exp_dt and _first_cdt<=&_exp_dt then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<=&_exp_dt then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if _no_acute_change=1 then do; 					 *if non-acute exp;
			if &_comorbdt ne . and &_comorbdt<=_ideal_dt<=&_exp_dt and &_exp_dt<=_intstart and _intnum=0 then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=&_exp_dt then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=&_exp_dt and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do; 				 *if non-acute exp;
			if _intnum=0 and &_comorbdt ne . and &_comorbdt<=_ideal_dt<=&_exp_dt and &_exp_dt<=_intstart then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _intnum=0 and &_comorbdt ne . and &_comorbdt<=&_exp_dt and &_exp_dt<=_intstart then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<&_exp_dt then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<&_exp_dt and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=2;
	end;
	else if _intnum<_intmax and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _acnt_num=1 and _censor ne 1 and _interval_percent>=_percent then do;
		if _no_acute_change=1 then do; 					 *if non-acute exp;
			if &_comorbdt<=_d_date and _prev_cdate<=_d_date and _first_cdt<=_d_date then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
				else if _prev_cdate<=_d_date and _first_cdt<=_d_date then _ideal_dt=max(_prev_cdate,_first_cdt);
				else if _ideal_dt<=_d_date then _ideal_dt=_ideal_dt;
			if _ideal_dt=&_comorbdt then _ideal=&_comorb;
				else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
				else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
				else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=_d_date and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do; 				 *if non-acute exp;
		if &_comorbdt<_d_date and _prev_cdate<_d_date and _first_cdt<_d_date then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_d_date and _first_cdt<_d_date then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_d_date then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<_d_date and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=4;
	end;
	** for first_exp_rule=0, case 8 follows case 14;
	else if _intnum=_intmax and ((%sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _acnt_num=1 and _censor ne 1) or (_past_out=1 and &_outcome_dt<=&_studyend and _d_date<&_outcome_dt))
			and _last_int_percent>=_percent then do;
		if _no_acute_change=1 then do;
			if &_comorbdt<=_d_date and _prev_cdate<=_d_date and _first_cdt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,&_comorbdt,_prev_cdate,_first_cdt);
				else if _prev_cdate<=_d_date and _first_cdt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,_prev_cdate,_first_cdt);
				else if _ideal_dt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,_ideal_dt);
				else if _ideal_dt<=_d_date then _ideal_dt=_ideal_dt;
			if _ideal_dt=&_comorbdt then _ideal=&_comorb;
				else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
				else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
				else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
				else _ideal=_ideal;	
		
			if &_comorbdt ne . and &_comorbdt<=_ideal_dt<=_d_date and _d_date<=_intstart and _intnum=0 then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _intnum=0 and &_comorbdt ne . and _intstart<=&_comorbdt<=&_exp_dt then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if &_comorbdt ne . and _last_int_begin<&_comorbdt<=_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=_d_date and (&_studyend-&_unitdays*2)<_ideal_dt<=_d_date then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do;
			if &_comorbdt<_d_date and _prev_cdate<_d_date and _first_cdt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,&_comorbdt,_prev_cdate,_first_cdt);
				else if _prev_cdate<_d_date and _first_cdt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,_prev_cdate,_first_cdt);
				else if _ideal_dt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,_ideal_dt);
				else if _ideal_dt<_d_date then _ideal_dt=_ideal_dt;
			if _ideal_dt=&_comorbdt then _ideal=&_comorb;
				else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
				else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
				else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
				else _ideal=_ideal;	
		
			if &_comorbdt ne . and &_comorbdt<=_ideal_dt<=&_exp_dt and &_exp_dt<=_intstart and _intnum=0 then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _intnum=0 and &_comorbdt ne . and _intstart<=&_comorbdt<&_exp_dt then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if &_comorbdt ne . and _last_int_begin<&_comorbdt<_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
			end;
			else if _ideal_dt<_d_date and (&_studyend-&_unitdays*2)<_ideal_dt<_d_date then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=8;
	end;
	%end;
	
*case 7 - first change in exposure;
	else if _intnum=_intmax and %sysfunc(ifc(&_exp_multi_level1=1, missing(_exposure)=0 and _exposure ne 'Not exposed', _exposure=1)) and _acnt_num=1 and _censor=1 and (&_studyend<&_outcome_dt or &_outcome_dt=.) then do;
		if _no_acute_change=1 then do; 					 *if non-acute exp;
		if &_comorbdt<=&_studyend and _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<=&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=&_studyend then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=&_studyend and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do; 				 *if non-acute exp;
		if &_comorbdt<&_studyend and _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<&_studyend then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<&_studyend and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do- changed to look back up to last L(t) designation;;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;		
		end;
		_case=7;
	end;
*case 3 - first change in exposure;  
	else if 0<_intnum<_intmax then do;
		if &_comorbdt<_intstart and _prev_cdate<_intstart and _first_cdt<_intstart then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_intstart and _first_cdt<_intstart then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_intstart then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if  &_comorbdt ne . and _prev_cdate<&_comorbdt<_intstart then do;
			_conf_dt=&_comorbdt;
			_conf=&_comorb;
			if _conf_dt ne . then _final_dt=_conf_dt;;
		end;
		else if _ideal_dt<_intstart and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
			_conf_dt=_ideal_dt;
			_conf=_ideal;
			if _conf_dt ne . then _final_dt=_conf_dt;;
		end;
	_case=3;
	end;
*case 5;
	else if _intnum=_intmax and _censor=1 and (&_studyend<&_outcome_dt or &_outcome_dt=.) then do;
		if _no_acute_change=1 then do;
		if &_comorbdt<=&_studyend and _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<=&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<=&_studyend then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=&_studyend and _final_dt<_ideal_dt then do; 	*(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do;
		if &_comorbdt<&_studyend and _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if _prev_cdate<&_comorbdt<&_studyend and &_comorbdt ne . then do;
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<&_studyend and _final_dt<_ideal_dt then do;  *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _prev_cdate<&_studyend and _final_dt<_ideal_dt then do;  *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
	_case=5;
	end;
*case 6/9, and 13 logic when _first_exp_rule=0;
 %if &_efirst1 = 1 %then %do; 
	else if _intnum=_intmax and _past_out=1 and &_outcome_dt<=&_studyend then do;
		if &_comorbdt<_intstart and _prev_cdate<=_intstart and _first_cdt<_intstart then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_intstart and _first_cdt<=_intstart then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_intstart then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if &_comorbdt ne . and _prev_cdate<&_comorbdt<_intstart then do;
			_conf_dt=&_comorbdt;
			_conf=&_comorb;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
		else if _ideal_dt<_intstart and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
			_conf_dt=_ideal_dt;
			_conf=_ideal;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
	_case=6;
	end;
%end;
%else  %if &_efirst1 = 0 %then %do;  *uses case 13 logic, still labeled as case 6;
	else if _intnum=_intmax and _past_out=1 and _d_date=. and &_outcome_dt<=&_studyend then do;
		if &_comorbdt<=(&_studyend-&_unitdays) and _prev_cdate<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) and _first_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt,_prev2_cdt);
			else if _prev_cdate<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) and _first_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max(_prev_cdate,_first_cdt,_prev2_cdt);
			else if _ideal_dt<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max (_ideal_dt,_prev2_cdt);
			else if _ideal_dt<=(&_studyend-&_unitdays) then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
			else _ideal=_ideal;
		
		if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=(&_studyend-&_unitdays) then do;	
			_conf_dt=&_comorbdt;
			_conf=&_comorb;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
		else if _ideal_dt<=(&_studyend-&_unitdays) and (&_studyend-2*&_unitdays)<_ideal_dt<=(&_studyend-&_unitdays) then do;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
		end;
	_case=6;
   end;
 %end;
end; 
*for after 1st exposure;
    else if _acnt_num>1 then do;
 *case 11;
	if _intnum<_intmax and _d_date ne . then do;
		if &_comorbdt<_d_date and _prev_cdate<_d_date and _first_cdt<_d_date then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_d_date and _first_cdt<_d_date then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_d_date then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if _no_acute_change=1 then do; 					 *if non-acute exp;
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=_d_date and _final_dt<_ideal_dt then do;  *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do-changed to look back up to last L(t) designation;
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		else if _no_acute_change=0 then do; 				 *if non-acute exp;
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<_d_date and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed o look back up to last L(t) designation;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=11;
	end;
*case 10- after first change in exposure;
	else if _intnum<_intmax and _achange=0 and _acnt_num>1 then do;
		if &_comorbdt<_intstart and _prev_cdate<_intstart and _first_cdt<_intstart then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_intstart and _first_cdt<_intstart then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<_intstart then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
		if  &_comorbdt ne . and _prev_cdate<&_comorbdt<_intstart then do;
			_conf_dt=&_comorbdt;
			_conf=&_comorb;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
		else if _ideal_dt<_intstart and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;;
			_conf_dt=_ideal_dt;
			_conf=_ideal;
			if _conf_dt ne . then _final_dt=_conf_dt;
		end;
	_case=10;
	end;
*case 12 (previously case 13 due to mis-numbering);
	else if _intnum=_intmax and _achange in (0,1) and _censor=1 then do;	
		if _no_acute_change=1 then do; 					 *if non-acute exp;
		if &_comorbdt<=&_studyend and _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=&_studyend and _first_cdt<=&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<=&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=&_studyend then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=&_studyend and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed to look back up to last L(t) designation;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do; 				 *if non-acute exp;
		if &_comorbdt<&_studyend and _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<&_studyend and _first_cdt<&_studyend then _ideal_dt=max(_prev_cdate,_first_cdt);
			else if _ideal_dt<&_studyend then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else _ideal=_ideal;
		
			if &_comorbdt ne . and _prev_cdate<&_comorbdt<&_studyend then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<&_studyend and _final_dt<_ideal_dt then do; *(_intstart-_ideal_dt)<=(_intend-_intstart+1) then do - changed  to look back up to last L(t);
				_conf_dt=_ideal_dt;
				_conf=_ideal;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;		
		end;
		_case=12;
	end;
*case 14;
	else if _intnum=_intmax and _d_date ne . and _past_out=1 then do;
		if _no_acute_change=1 then do;
		if &_comorbdt<=_d_date and _prev_cdate<=_d_date and _first_cdt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<=_d_date and _first_cdt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,_prev_cdate,_first_cdt);
			else if _ideal_dt<=_d_date and _prev2_cdt<=_d_date then _ideal_dt=max(_prev2_cdt,_ideal_dt);
			else if _ideal_dt<=_d_date then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
			else _ideal=_ideal;	

			if &_comorbdt ne . and _last_int_begin<&_comorbdt<=_d_date then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=_d_date and (&_studyend-&_unitdays*2)<_ideal_dt<=_d_date then do;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do;
		if &_comorbdt<_d_date and _prev_cdate<_d_date and _first_cdt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,&_comorbdt,_prev_cdate,_first_cdt);
			else if _prev_cdate<_d_date and _first_cdt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,_prev_cdate,_first_cdt);
			else if _ideal_dt<_d_date and _prev2_cdt<_d_date then _ideal_dt=max(_prev2_cdt,_ideal_dt);
			else if _ideal_dt<_d_date then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
			else _ideal=_ideal;	
		
			if &_comorbdt ne . and _last_int_begin<&_comorbdt<_d_date then do;		
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<_d_date and (&_studyend-&_unitdays*2)<_ideal_dt<=_d_date then do;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=14;
	end;
*case 13;
	else if _intnum=_intmax and _d_date=. and _past_out=1 then do;	
	if &_comorbdt<=(&_studyend-&_unitdays) and _prev_cdate<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) and _first_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max(&_comorbdt,_prev_cdate,_first_cdt,_prev2_cdt);
			else if _prev_cdate<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) and _first_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max(_prev_cdate,_first_cdt,_prev2_cdt);
			else if _ideal_dt<=(&_studyend-&_unitdays) and _prev2_cdt<=(&_studyend-&_unitdays) then _ideal_dt=max(_ideal_dt,_prev2_cdt);
			else if _ideal_dt<=(&_studyend-&_unitdays) then _ideal_dt=_ideal_dt;
		if _ideal_dt=&_comorbdt then _ideal=&_comorb;
			else if _ideal_dt=_prev_cdate then _ideal=_prev_c;
			else if _ideal_dt=_first_cdt then _ideal=_first_comorb;
			else if _ideal_dt=_prev2_cdt then _ideal=_prev2;
			else _ideal=_ideal;
		if _no_acute_change=1 then do;
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=(&_studyend-&_unitdays) then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=(&_studyend-&_unitdays) and (&_studyend-2*&_unitdays)<=_ideal_dt<=(&_studyend-&_unitdays) then do;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		
		else if _no_acute_change=0 then do;
		
			if &_comorbdt ne . and _prev_cdate<=&_comorbdt<=(&_studyend-&_unitdays) then do;	
				_conf_dt=&_comorbdt;
				_conf=&_comorb;
				if _conf_dt ne . then _final_dt=_conf_dt;
			end;
			else if _ideal_dt<=(&_studyend-&_unitdays) and (&_studyend-2*&_unitdays)<_ideal_dt<=(&_studyend-&_unitdays) then do;
					_conf_dt=_ideal_dt;
					_conf=_ideal;
					if _conf_dt ne . then _final_dt=_conf_dt;
			end;
		end;
		_case=13;
	end;
  end;
*clean-up;
   if _intnum>_intmax then do;
		_conf_dt=.;
		%if &&&_varfmt&_i=N %then %do;
		_conf=.;
		%end;
		%else %if &&&_varfmt&_i=C %then %do;
		_conf='';
		%end;
	end;
 format _conf_dt &_comorbdt _ideal_dt _final_dt _first_cdt mmddyy10.;
run;

proc sort data=_workdata_conf;
	by &_ident _intstart _conf_dt;
run;

data _workdata_conf;
	set _workdata_conf;
	drop &_comorb &_comorbdt _prev_c _prev_cdate _prev2_cdt _prev2 _prev_exp _first_cdt _first_comorb _ideal _ideal_dt _achange;
run;	



data _workdata_conf;
	set _workdata_conf;
	&_comorb=_conf;
	&_comorbdt=_conf_dt;
	format &_comorbdt mmddyy10.;
	drop _conf _conf_dt _acnt_num _final_dt;
run;

proc sort data=_workdata_conf out=_workdata nodupkey; 
	by &_ident _intstart descending &_comorbdt;
run;

data _workdata;
	set _workdata;
	by &_ident _intstart descending &_comorbdt;
	if first._intstart;
run;

data _workdata;
	set _workdata;
		by 	&_ident _intstart;
		retain _complete_dt _complete;
	_comp_dt=lag1(&_comorbdt);
	_comp=lag1(&_comorb);
	if _comp_dt ne . then do;
		_complete_dt=_comp_dt;
		_complete=_comp;
	end;
	else do;
		_complete_dt=_complete_dt;
		_complete=_complete;
	end;
	format _complete_dt mmddyy10.;
run;

data _workdata;
	set _workdata;
	by &_ident _intstart;
	if first.&_ident then do;
		_complete_dt=.;
		%if &&&_varfmt&_i=N %then %do;
		_complete=.;
		%end;
		%else %if &&&_varfmt&_i=C %then %do;
		_complete='';
		%end;
	end;
	if _complete_dt=&_comorbdt and _complete=&_comorb then do;
		&_comorbdt=.;
		%if &&&_varfmt&_i=N %then %do;
		&_comorb=.;
		%end;
		%else %if &&&_varfmt&_i=C %then %do;
		&_comorb='';
		%end;
	end;
	format _complete_dt _comp_dt mmddyy10.;	
run;


data _workdata;
	set _workdata;
 	drop _complete _complete_dt _lag_int _comp ;
run;

*10/12/16 per Romain request , add value and date for covariate at row with outcome =1;

data _workdata1;
    set _workdata;
    by &_ident; 

   pre_case =lag1(_case);
   pre_d_date =lag1(_d_date);
   pre_compdt= lag1(_comp_dt);;

  if first.&_ident then  do;
   pre_case=.;
   pre_d_date=.;
   pre_compdt=.;
   end;
run;


data row_outcome;
 set _workdata1;
 where _outcome=1;

 
 if pre_case =6 then do;
    if _intmax ne 0 and missing(_comp_dt)=0 then cal_begin=_comp_dt+1;
      %if &_efirst1 = 0 %then %do;
      else if _intmax ne 0 and missing(_comp_dt)=1 
       then cal_begin = max((&_studyend - &_unitdays),pre_compdt+1);
      %end;

      %else %do;
       else if _intmax ne 0 and missing(_comp_dt)=1 then cal_begin = _intstart - &_unitdays;
      %end; 
     else if _intmax = 0 then cal_begin = &_studystart+1;
   end;
 
 if pre_case =13 then do;
   if missing(_comp_dt)=0 then cal_begin=_comp_dt+1;
    else cal_begin= max((&_studyend - &_unitdays),(pre_compdt+1));
 end;

if pre_case =8 then do;
   if missing(_comp_dt)=0 then cal_begin=_comp_dt+1;
   else do;
     if  _no_acute_change=1 then cal_begin=max(pre_d_date+1,pre_compdt+1);
     else cal_begin = max(pre_d_date,pre_compdt+1);
   end;
end;


 if pre_case =14 then do;
   if missing(_comp_dt)=0 then cal_begin=_comp_dt+1;
   if missing(_comp_dt)=1 then do;
    if  _no_acute_change=1 then cal_begin=max(pre_d_date+1,pre_compdt+1) ;
    else cal_begin =max(pre_d_date,pre_compdt+1);
   end;
end;


 format cal_begin mmddyy10.;
 run;

*merge with original covariate data set to pull to covariate records during certain period based on case;
proc sql;
 create table last_out as
 select b.*, cal_begin, a.&_studyend, _intnum
 from row_outcome a, &&&_cfile&_i b
 where a.&_ident =b.&_ident and  b.&&&_sas_date_var&_i between cal_begin and a.&_studyend
 order by &_ident, &&&_sas_date_var&_i desc ;
quit;

data last_out (keep= &_ident _intnum &_comorb &_comorbdt);
set last_out;
by &_ident;
if first.&_ident;
&_comorb =&&&_cname&_i;
&_comorbdt = &&&_sas_date_var&_i;
%if &&&_cbehavior&_i = 1 %then %do; 
if missing(&_comorbdt) =0 then &_comorb = 1;
%end; 
format &_comorbdt mmddyy10.;
run;

data _workdata;
merge _workdata last_out;
by &_ident _intnum;
drop _comp_dt _no_acute_change;
run;

proc datasets library = work nolist;
  delete _workdata_conf _workdata1 row_outcome last_out;
 quit;
 run;

 *** for testing puprposes - exporting datasets;
   %if  &unittest= conf_case2_binary %then %do;
   %let rundate = %SYSFUNC(today(),yymmddn8.); 
	data &_outdata;
		set _workdata;
	run;
	%goto exit_conf_case2;
  %end;

*** end of testing;

%_impute_lvcf (_workdata,&_ident,&_comorb,&_comorbdt);


%exit_conf_case2: %mend _conf_case2_binary;
