/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


/********* MSM Macro 4a - covariate Assignment based on Single fixed Binary Exposure *************
*********************************    _covariateint    ********************************************
 macro to create dataset where covariates merge with intervals  
  	- to be run after macro determining exposures for a single, binary exposure		
  Input:											
	_inconfoundint=input dataset, assumed to be in baseline format		
	_ident= unit identifier for each patient	
	_varsexposureint = file location and name of the input variables dataset
				- assume _variable name same in raw dataset
 Output dataset: dataset with covariates and exposures listed		
 	one row per interval, covariates listed as variables with corresponding date
 
 **assumption = covariate names and dates in baseline table is same used in csv file
 7/10/2012 - create macro for handling lab and comorbidity covariates first.	
 7/25/2013 - changed initial pick up of covariates to  (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend)
 				- will miss baseline data, but next data step should pick up all baseline data
 10/11/2013 - add in 2 more behaviors - 3 and 5
 				- behavior 3 = time-independent variables
 				- behavior 5 = time-varying with no missing 
*************************************************************************************************/

%macro _confounderint (_inconfoundint,_ident,_studystart,_studyend,_outcome_dt,_exp_dt,_varsexposureint,unittest); 

* Extract information on covariates from the basedata &varfile and source datasets;

 data _covariate;
  set &_varsexposureint;
  if _remove ne 1;
  if upcase(_type) = 'COVARIATE' then output _covariate;
 run;

 * covariate comorbidities handling;
  data _covariate;
   set _covariate;
   _order0=_N_;
  run;
                                                         
  %_nobs_msm(_covariate);
  %global _nobs_covariate;
  %let _nobs_covariate = %eval(&_nobs);

* Convert values passed from &_varsdata into macro variables;
   %do _i=1 %to &_nobs_covariate;

    %global _cname&_i;
    %let _cname&_i = 0;
    %global _cfile&_i;
    %let _cfile&_i = 0;
    %global _cbehavior&_i;
    %let _cbehavior&_i = 0;
    %global _sas_date_var&_i;
    %let  _sas_date_var&_i = 0;
    %global _varfmt&_i;
    %let _varfmt&_i=0;
    %global _impute&_i;
    %let _impute&_i=0;
    %global _no_acute_change&_i;
    %let _no_acute_change&_i = 0;
    %global _impvalue&_i;
	%let _impvalue&_i =0;
    
 
 
    data _null_;
     set _covariate;
     if _n_=&_i;
     call symput("_cname&_i",trim(left(_variable)));
     call symput("_cfile&_i",trim(left(_sas_data_name)));
     call symput("_cbehavior&_i",trim(left(_behavior)));
     call symput("_no_acute_change&_i",trim(left(_no_acute_change)));
     call symput("_sas_date_var&_i",trim(left(_sas_date_var)));
     call symput("_varfmt&_i",trim(left(_varfmt)));
     call symput("_impute&_i",trim(left(_impute)));
	 call symput("_impvalue&_i",trim(left(_value)));
    run;
   
    %put _i=&_i;
    %put _cname=&&&_cname&_i;
    %put _cfile=&&&_cfile&_i;
    %put _cbehavior=&&&_cbehavior&_i;
    %put _no_acute_change=&&&_no_acute_change&_i;
    %put _sas_date_var=&&&_sas_date_var&_i;
    %put _varfmt=&&&_varfmt&_i;
    %put _impute=&&&_impute&_i;
    %put _impvalue= &&&_impvalue&_i;
    %put ;
    %put ;
   
	*when _value is blank or behavior5 then setthe value unknown for char variable and  0 for Numerical variable;
      %if &&&_impvalue&_i=  and &&&_varfmt&_i= C %then %do; 
      %let _impvalue&_i = Unknown; 
      %end; 

      %if &&&_cbehavior&_i = 5 and &&&_varfmt&_i= C %then %do; 
      %let _impvalue&_i =None ; 
      %end; 

      %if  (&&&_impvalue&_i= or &&&_cbehavior&_i = 5) and &&&_varfmt&_i= N %then %do; 
      %let _impvalue&_i =0;
	  %end; 

     %if &&&_cbehavior&_i = 5 %then %do;
     %let _impute&_i= 1;
     %end;

    %put _impute=&&&_impute&_i;

  /*behavior1 - comorbidities, once on always on */ 
    %if &&&_cbehavior&_i = 1 and &&&_no_acute_change&_i=0 %then %do;    
     proc sql;
      create table _msm_&&&_cname&_i as
       select
        a.&_ident,
        b.&&&_sas_date_var&_i
      from &_inconfoundint as a,
           &&&_cfile&_i as b
      where a.&_ident=b.&_ident and 
            (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend);
     quit;

     proc sort data=_msm_&&&_cname&_i;
      by &_ident &&&_sas_date_var&_i;
     run;

     data _msm_&&&_cname&_i;
      set _msm_&&&_cname&_i;
      _&&&_cname&_i=1;
      _no_acute_change=&&&_no_acute_change&_i;
      _dt_&&&_cname&_i=&&&_sas_date_var&_i;
      format &&&_sas_date_var&_i _dt_&&&_cname&_i mmddyy10.;
      keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
      
     data _baseline;
	set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
	_&&&_cname&_i=&&&_cname&_i;
	_no_acute_change=&&&_no_acute_change&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
      
     data _msm_&&&_cname&_i;
	set _msm_&&&_cname&_i (in=a)
		_baseline (in=b);
		if _dt_&&&_cname&_i ne .;
     run;
		
			
      	/*re-merging interval data so we have all intervals per patient, regardless of covariate activity*/
    proc sql;
      create table _&&&_cname&_i as
       select
        a.*
       ,b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
      where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
     run; 
    %end; 
    
    
   %else %if &&&_cbehavior&_i = 1 and &&&_no_acute_change&_i=1 %then %do;    
     proc sql;
      create table _msm_&&&_cname&_i as
       select
        a.&_ident,
        b.&&&_sas_date_var&_i
      from &_inconfoundint as a,
           &&&_cfile&_i as b
      where a.&_ident=b.&_ident and 
            (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend);
     quit;


     proc sort data=_msm_&&&_cname&_i;
      by &_ident &&&_sas_date_var&_i;
     run;

     data _msm_&&&_cname&_i;
      set _msm_&&&_cname&_i;
      _&&&_cname&_i=1;
      _no_acute_change=&&&_no_acute_change&_i;
      _dt_&&&_cname&_i=&&&_sas_date_var&_i;
      format _dt_&&&_cname&_i mmddyy10.;
      keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
      
      data _baseline;							
	  *values at baseline are set to have study start dt as date of occurrence;
	set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
	_&&&_cname&_i=&&&_cname&_i;
	_no_acute_change=&&&_no_acute_change&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
      
     data _msm_&&&_cname&_i;
	set _msm_&&&_cname&_i (in=a)
		_baseline (in=b);
		if _dt_&&&_cname&_i ne .;
     run;
     
      	/*re-merging interval data so we have all intervals per patient, regardless of covariate activity*/
    proc sql;
      create table _&&&_cname&_i as
       select
        a.*
       ,b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
      where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
     run; 
   %end; 
   
  /*behavior 2 - time-varying categorical or continuous covariates, e.g. drugs with varying intensity - no missing values */
   %else %if &&&_cbehavior&_i = 2 %then %do;  
   ***checking for missing data - stops program if missing data exists;
   	 %if &&&_varfmt&_i=N %then %do;
   	 data _test_miss;
   	 	set &&&_cfile&_i;
   	 	if &&&_sas_date_var&_i ne . and &&&_cname&_i=. then output;
   	 	
   	 run;
   	%end;
   	%else %if &&&_varfmt&_i=C %then %do;
   	data _test_miss;
   	 	set &&&_cfile&_i;
   	 	if &&&_sas_date_var&_i ne . and &&&_cname&_i='' then output;
   	 	
   	 run;
   	%end;
   	%let dsid = %sysfunc (open(_test_miss));
	%let anyobs = %sysfunc (attrn(&dsid,ANY));
	%let dsid = %sysfunc (close(&dsid));
	%put anyobs=&anyobs;
	%put dsid=&dsid;
	
   	%if &anyobs>0 %then %do;
   		%put 'MACRO ERROR DETECTED: covariate behavior 2 for &&&_cfile&_i contains missing values';
     	%goto exit_confounderint;
    %end;
    
   proc sql;
      create table _msm_&&&_cname&_i as
       select
        a.&_ident,
        b.&&&_sas_date_var&_i,
       	b.&&&_cname&_i
      from &_inconfoundint as a,
           &&&_cfile&_i as b
      where a.&_ident=b.&_ident and 
          (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend);
     quit;

     proc sort data=_msm_&&&_cname&_i;
      by &_ident &&&_sas_date_var&_i;
     run;

     data _msm_&&&_cname&_i;
      set _msm_&&&_cname&_i;
      _no_acute_change=&&&_no_acute_change&_i;
      _dt_&&&_cname&_i=&&&_sas_date_var&_i;
      _&&&_cname&_i=&&&_cname&_i;
      format _dt_&&&_cname&_i mmddyy10.;
      keep &_ident _dt_&&&_cname&_i _&&&_cname&_i _no_acute_change;
     run;
    data _baseline;
		set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
		_&&&_cname&_i=&&&_cname&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
     data _msm_&&&_cname&_i;
	set _msm_&&&_cname&_i (in=a)
		_baseline (in=b);
		if _dt_&&&_cname&_i ne .;
     run;
     
  	/*re-merging interval data so we have all intervals per patient, pass down last covariate value*/
    proc sql;
      create table _&&&_cname&_i as
       select
        a.*
       ,b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
      where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
     run; 
     
     proc sort data=_&&&_cname&_i;
     	by &_ident _intstart _dt_&&&_cname&_i;
     run;
    %end; 

   
    /*behavior 3 - time-independent covariates, e.g. race or gender */
   %else %if &&&_cbehavior&_i = 3 %then %do;    
     data _msm_&&&_cname&_i;
		set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
		_&&&_cname&_i=&&&_cname&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	_no_acute_change=&&&_no_acute_change&_i;

      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
     data _msm_&&&_cname&_i;
		set _msm_&&&_cname&_i;
		if _dt_&&&_cname&_i ne .;
     run;
        
	/*re-merging interval data so we have all intervals per patient, regardless of covariate activity*/
	proc sql;
      create table _&&&_cname&_i as
       select
        a.*,
     	b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
       where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
     run; 
          
   %end;
   
/*behavior 4,5  - time-varying categorical or continuous covariates, e.g. labs */
   %else %if (&&&_cbehavior&_i = 4 or &&&_cbehavior&_i = 5) and &&&_no_acute_change&_i=0 %then %do;    
     proc sql;
      create table _msm_&&&_cname&_i as
       select
        a.&_ident,
        b.&&&_sas_date_var&_i,
       	b.&&&_cname&_i
      from &_inconfoundint as a,
           &&&_cfile&_i as b
      where a.&_ident=b.&_ident and 
          (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend);
     quit;
	
	
     proc sort data=_msm_&&&_cname&_i;
      by &_ident &&&_sas_date_var&_i;
     run;

     data _msm_&&&_cname&_i;
      set _msm_&&&_cname&_i;
      _no_acute_change=&&&_no_acute_change&_i;
      _dt_&&&_cname&_i=&&&_sas_date_var&_i;
      _&&&_cname&_i=&&&_cname&_i;
      format _dt_&&&_cname&_i mmddyy10.;
      keep &_ident _dt_&&&_cname&_i _&&&_cname&_i _no_acute_change;
     run;
    data _baseline;
		set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
		_&&&_cname&_i=&&&_cname&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
     data _msm_&&&_cname&_i;
	set _msm_&&&_cname&_i (in=a)
		_baseline (in=b);
		if _dt_&&&_cname&_i ne .;
     run;
    
    	/*re-merging interval data so we have all intervals per patient, regardless of covariate activity*/
    proc sql;
      create table _&&&_cname&_i as
       select
        a.*,
        b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
      where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
   run; 
    %end; 
    
    
    %else %if (&&&_cbehavior&_i = 4 or &&&_cbehavior&_i = 5) and &&&_no_acute_change&_i=1 %then %do;    
     proc sql;
      create table _msm_&&&_cname&_i as
       select
        a.&_ident,
        b.&&&_sas_date_var&_i,
       	b.&&&_cname&_i
      from &_inconfoundint as a,
           &&&_cfile&_i as b
      where a.&_ident=b.&_ident and 
          (a.&_studystart < b.&&&_sas_date_var&_i <= a.&_studyend);
     quit;

     proc sort data=_msm_&&&_cname&_i;
      by &_ident &&&_sas_date_var&_i;
     run;

     data _msm_&&&_cname&_i;
      set _msm_&&&_cname&_i;
      _no_acute_change=&&&_no_acute_change&_i;
      _dt_&&&_cname&_i=&&&_sas_date_var&_i;
      _&&&_cname&_i=&&&_cname&_i;
      format _dt_&&&_cname&_i mmddyy10.;
      keep &_ident _dt_&&&_cname&_i _&&&_cname&_i _no_acute_change;
     run;
     data _baseline;
		set &_inconfoundint (keep=&_ident &_studystart &&&_cname&_i);
		_&&&_cname&_i=&&&_cname&_i;
      	_dt_&&&_cname&_i=&_studystart;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
      	keep &_ident _&&&_cname&_i _dt_&&&_cname&_i _no_acute_change;
     run;
      
     data _msm_&&&_cname&_i;
	set _msm_&&&_cname&_i (in=a)
		_baseline (in=b);
		if _dt_&&&_cname&_i ne .;
     run;
     
    	/*re-merging interval data so we have all intervals per patient, regardless of covariate activity*/
    proc sql;
      create table _&&&_cname&_i as
       select
        a.*
       ,b._&&&_cname&_i, b._dt_&&&_cname&_i, b._no_acute_change
      from _workdata a,
           _msm_&&&_cname&_i b
      where a.&_ident=b.&_ident and 
            ((a._intstart <= b._dt_&&&_cname&_i <= a._intend) or (a._intnum=0 and b._dt_&&&_cname&_i<=a._intstart));
     quit;
     
     proc sort data= _&&&_cname&_i;
      by &_ident _intstart _intend; 
     run;
     proc sort data=_workdata;
     	by &_ident _intstart _intend;
     run;
     
     data  _&&&_cname&_i;
      merge _workdata(in=a)  
      	_&&&_cname&_i(in=b);
      		by &_ident _intstart;
      	if a;
      	_no_acute_change=&&&_no_acute_change&_i;
      	format _dt_&&&_cname&_i mmddyy10.;
     run; 
    %end; 
    
    
    %else %do;
     %put 'MACRO ERROR DETECTED: covariate behavior listed is currently not supported';
     %goto exit_confounderint;
    %end;
 
 title;
 title2;
 
 *** for testing puprposes - exporting datasets;
  %if  &unittest= confounderint %then %do;

	%let rundate = %SYSFUNC(today(),yymmddn8.); 
	data &_outdata;
		set _&&&_cname&_i;
	run;
	%goto exit_confounderint;
  %end;
*** end of testing;


%_conf_case2_binary (_&&&_cname&_i,_&&&_cname&_i,_dt_&&&_cname&_i,&_outcome_dt,&_studyend,&_ident,&_unitdays,&_exp_dt,0); 

 proc datasets library = work nolist;
  delete _&&&_cname&_i _msm_&&&_cname&_i;
 quit;
 run;
 %end;

 
%exit_confounderint: 
%mend _confounderint;

