/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


/*********** MSM Macro 1 - Creating Fixed Time Intervals *************
 Step 1 of MSM dataset creation 
	- takes in full time of cohort and create dataset of time intervals.
	= each interval includes start date and prior to last date
		i.e. T0->(T1-1day)
	- last inerval: Tn-1 -> Tn 
	
Variables:
	_indata = input dataset, must contain atleast ID beg date and end date
	_outdata = output dataset (permanent, will be built on in other macros)
	_studystart = begin date, or index date
	_studyend = end date, or censor date. Last date pt is in study
	_ident = identifying variable such as MRN or studyid or mrngroup
	
*********************************************************************/


%macro _crtfixint(_idcrtfixint,_studystart,_studyend,_ident,_unitdays);

proc sort data=&_idcrtfixint out=_fixint(keep=&_ident &_studystart &_studyend) nodupkey ;
	by &_ident;
run;

Data _fixint;
	set _fixint;
	_intcnt = ceil((&_studyend-&_studystart+1)/&_unitdays);
	keep &_ident &_studyend &_studystart _intcnt;
run;

data _workdata;
	set _fixint;
	retain _intstart _intend;
	do i=1 to _intcnt;
		if i=1 then do;
			_intstart=&_studystart; 
			_intend=_intstart+&_unitdays-1; 
			_intnum=i-1;
			output;
		end;
		else do;
			_intstart=_intend+1; 
			_intend=_intstart+&_unitdays-1;
			_intnum=i-1;
			output;
		end;
	end;
	format _intstart _intend mmddyy10.;
	keep &_ident _intstart _intend _intnum;
run;	

%_maxintcnt(_workdata,&_ident,_intstart);

proc datasets library = work nolist;
	        delete _fixint;
        quit;
        
%mend _crtfixint;
