/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


****** MSM MAcro 3 - Binary Exposure with periods of change ******
**** Applies predetermined logics to Exposures
	- Takes dataset from Macro 2: _exposureint2.sas
	- determines when exposure takes place within intervals
Need:
	_in_expos_case2_bin = dataset created in _exposureint.sas with 
		appropriate exposure data
	_ident = unique patient identifier
	_exp_start = start date of exposure of interest
	_exp_end = end data of exposure of interest
	_percent = X% of days in interval exposure is active to count as exposed (must be decimal)
			-default percent = 50%
****************************************************************/;
%macro _expos_case2_binary (_in_expos_case2_bin,_ename1,_ename1_end,_oname,_studystart,_studyend,_ident,_unitdays,_percent);
* Prepare input dataset;
	
 proc sort data=&_in_expos_case2_bin out=_expose_binary;
  by &_ident _intnum;
 run;
** create count of days exposed per interval for each exposure level, 
  count number of time A changes overall per person;
%do j =1 %to &n;
* track count of changes in A over time per person;
 data _expose_binary&j;
  set _expose_binary;
  by &_ident;
  retain _days_exposed ;
  

  /* code for first interval */
  
	
  if first.&_ident then do;

   /* exposed before or during first interval */
  
   if _exp_&_ename1._&&&_leveln1&j ne . and _exp_&_ename1._&&&_leveln1&j < _intstart then do;
    _exp_&_ename1._&&&_leveln1&j = _intstart;
    _exp_comb_&_ename1._&&&_leveln1&j = _intstart;
    
   end;
  

   /* cleaning of exposure end dates */  

   if _exp_&_ename1._end_&&&_leveln1&j ne . and _exp_&_ename1._end_&&&_leveln1&j > _intend then 
    _exp_&_ename1._end_&&&_leveln1&j = _intend;
   

   if _exp_&_ename1._end_&&&_leveln1&j > &_studyend then  _exp_&_ename1._end_&&&_leveln1&j = &_studyend;
  

   /* count of days exposed during interval */  
   _days_exposed = (_exp_&_ename1._end_&&&_leveln1&j - _exp_&_ename1._&&&_leveln1&j) + 1;
  
  end;
  
  /* code for all intervals after the first */
  
  else if not first.&_ident then do;

   /* cleaning exposed end date - extends past interval under study */  
      
   if _exp_&_ename1._end_&&&_leveln1&j > _intend then  _exp_&_ename1._end_&&&_leveln1&j = _intend;
   

   if _exp_&_ename1._end_&&&_leveln1&j > _out_&_oname._date and _out_&_oname._date > . then  _exp_&_ename1._end_&&&_leveln1&j = _out_&_oname._date;
 

   else if _exp_&_ename1._end_&&&_leveln1&j > &_studyend then _exp_&_ename1._end_&&&_leveln1&j = &_studyend;
    
* mark note;

   /* exposed prior to interval under study */  
  
   if . < _exp_&_ename1._&&&_leveln1&j < _intstart then do;
    _exp_&_ename1._&&&_leveln1&j= _intstart;
    _exp_comb_&_ename1._&&&_leveln1&j = _intstart;
   end;

   /* count of days exposed during interval */   
   _days_exposed = (_exp_&_ename1._end_&&&_leveln1&j-_exp_&_ename1._&&&_leveln1&j) + 1;
  
  end;
  %if &_percent=  %then  %do; 
    _percent = 0.5; 
    %end;

   %else %do;
   _percent = &_percent;
   %end;
   
   if _intnum =_intmax and ( _out_&_oname._date =. or _intmax=0)  then  _interval_percent&j = _days_exposed /(&_studyend - _intstart + 1);
   else _interval_percent&j = (_days_exposed / %eval(&_unitdays));
 
 run;

 %if &j= 1 %then %do;
/*base data set*/
  data _expose_all;
   set _expose_binary1;
    length level_name $32;
    _interval_percent= _interval_percent&j;
    
	level_name1="&&&_leveln1&j";

    _exp_&_ename1=_exp_&_ename1._&&&_leveln1&j;
    _exp_&_ename1._end=_exp_&_ename1._end_&&&_leveln1&j;
	_exp_comb_&_ename1=_exp_comb_&_ename1._&&&_leveln1&j;
	_exp_start1=_exp_&_ename1._&&&_leveln1&j;
	_exp_end1 =_exp_&_ename1._end_&&&_leveln1&j;
    level_name ="&&&_leveln1&j";
	format _exp_&_ename1 _exp_&_ename1._end _exp_comb_&_ename1 mmddyy10.;
   run;
 %end;


%else %do;
  data _expose_all;
  merge _expose_all(in=a) _expose_binary&j(keep=&_ident _intstart _interval_percent&j _exp_&_ename1._&&&_leveln1&j 
                      _exp_&_ename1._end_&&&_leveln1&j _exp_comb_&_ename1._&&&_leveln1&j );
  by &_ident _intstart; 
  if a; 
     _exp_start&j=_exp_&_ename1._&&&_leveln1&j;
    _exp_end&j =_exp_&_ename1._end_&&&_leveln1&j;
    level_name&j ="&&&_leveln1&j";

   if  _interval_percent&j > _interval_percent then do;
    _interval_percent = _interval_percent&j;
    level_name ="&&&_leveln1&j";
    _exp_&_ename1=_exp_&_ename1._&&&_leveln1&j;
    _exp_&_ename1._end=_exp_&_ename1._end_&&&_leveln1&j;
    _exp_comb_&_ename1=_exp_comb_&_ename1._&&&_leveln1&j;
  end;
  
  
  format _exp_&_ename1 _exp_&_ename1._end  _exp_comb_&_ename1 mmddyy10.;
 run;

%end;
%end; *end of J level loop;


  data _expose_all;
  set  _expose_all;
    by &_ident;
  retain _acnt;

  length tie_level $32;

   if missing(_interval_percent)=1 then level_name='';
  
  _prev_exp_end=lag1(_exp_&_ename1_end);  

 if first.&_ident then do;
  _prev_exp_end=.;  
  if missing(_exp_&_ename1)=0 then _inflect=1;
   else _inflect=0;
   _acnt=_inflect;
  end;
  
  else do; 
   if _acnt=0 and missing(_exp_&_ename1)=0 then _inflect=1;
   else if _acnt > 0 and missing(_exp_&_ename1)=0 and  ((_exp_&_ename1-_prev_exp_end)> 1 | missing(_prev_exp_end)=1) then _inflect =1;
   else _inflect=0;
   _acnt = _acnt+_inflect; 
  end;

  *flag tie cases for max exposure level;  
  max_num=0;
  sum_percent=0;
  array level(&n) _interval_percent1 -  _interval_percent&n;
  array leveln(&n) level_name1 - level_name&n;
  array exp_end(&n) _exp_end1 - _exp_end&n;

 do i= 1 to &n;
  if missing (level(i))=0  then do;
   if level(i) = _interval_percent then max_num +1;
   sum_percent + level(i);
   end;

  if max_num = 1 then do;
  tie_last_end= _exp_&_ename1._end; 
  tie_level =level_name;
  end;


   if max_num >1 and  level(i) = _interval_percent  then do;
    tie_level=catx(',',tie_level,leveln(i));
    if exp_end(i) > tie_last_end then do; 
    tie_last_end=  exp_end(i);
	level_name =leveln(i);
	end;
   end;

  *if max_num >1 and _interval_percent lt &_percent then sum_percent + level(i);
  end;

  *flag tie cases for max exposure level;  
  if max_num>1  then tie=1;
  else tie=0; 

  format _prev_exp_end tie_last_end mmddyy10.;

run;

data _expose_categorical ;
  set _expose_all;
  by &_ident;
  length _case $3 ;
  %if &_exp_fmt1 ='C' %then %do;
  _exposure $32;
 %end;
 
  retain _outcome  _past_out;
   
  _prev_exp_begin=lag1(_exp_&_ename1);
  _prev_exp_end=lag1(_exp_&_ename1_end);  
  _prev_exp_comb=lag1(_exp_comb_&_ename1);  

  if first.&_ident then do;
  _prev_exp_begin=.;
  _prev_exp_end=.;  
  _prev_exp_comb=.;  
  end;

  /* setting values for the first interval - patients with more than one interval */
  
  if first.&_ident and _intnum < _intmax then do;
   
   /* first interval, no exposure, no outcome */
   
   if _exp_&_ename1 = . and _out_&_oname._date = . then do;
    _exposure = 'Not exposed ';
    _outcome = 0;
    _censor = 0;
    _past_out = 0;
    
   %if &_efirst1 = 1 %then %do;
   _case ='2a';
   %end;

   %else %do;
   _case='2b';
   %end;
  
   /* _case = '0';*/
    
   end;

   /* first interval, exposure data present under percentage rule, no outcome */
   
   %if &_efirst1 = 0 %then %do;   
    else if _exp_&_ename1 ne . and _out_&_oname._date = . then do;
     _outcome = 0;  
     _censor = 0;
     _past_out = 0; 		

     if (_interval_percent >= _percent) then  do;
     _exposure = level_name;
     _case='1b';
	 end;

     else do;
     _exposure='Not exposed ';
     _case='2b';
	 end;

     /*_case = '0';*/
    
    end;
   %end;

   /* first interval, exposure data present under any exposure rule, no outcome */   
   %else %if &_efirst1 = 1 %then %do;
    else if _exp_&_ename1 ne . and _out_&_oname._date = . then do;
     _outcome = 0;  
     _censor = 0;
     _past_out = 0; 	
    
     if tie=1 and _intnum ne 0 then _exposure='';
     else _exposure = level_name; 
     
	  _case='1a';

     /*_case = '0';*/
    
    end; 
   %end;
    
  end;

  /* setting values for the first interval - patients with one interval only */ 

  else if first.&_ident and _intnum = _intmax then do;
   
   /* first interval, exposure data present under percentage rule, with outcome */
   
   %if &_efirst1 = 0 %then %do;   
    if _exp_&_ename1 ne . and _out_&_oname._date ne . and _out_&_oname._date <= &_studyend and _exp_&_ename1 <= _out_&_oname._date then do;
     _outcome = 0;  
     _censor = 0;
     _past_out = 1; 		

     *_last_int_begin = _intstart;   
     _last_int_percent = _interval_percent;
     *_last_int_recalc = 1;

     if (_last_int_percent >= _percent) then do;
      _exposure = level_name;
      _case ='5b';
	  end;

     else do;
     _exposure = 'Not exposed ';
     _case='6b';
	 end;
     /*_case = '0';*/
    
    end;
   %end;


   /* first interval, exposure data present under any exposure rule, with outcome */   
   %else %if &_efirst1 = 1 %then %do;
    if _exp_&_ename1 ne . and _out_&_oname._date ne . and _out_&_oname._date <= &_studyend and _exp_&_ename1 <= _out_&_oname._date then do;

    *_last_int_begin = (1 + _out_&_oname._date - &_unitdays);  
     _last_int_percent = _interval_percent;

     _outcome = 0;  
     _censor = 0;
     _past_out = 1; 		
     _exposure = level_name; 
     
	  _case ='6a';
	 
     /*_case = '0';*/
    
    end; 
   %end;

   /* first interval, no exposure, outcome occurs */
   
   else if _out_&_oname._date ne . and _out_&_oname._date <= &_studyend then do;

     _last_int_begin = (1 + _out_&_oname._date - &_unitdays);  
     _last_int_percent = _interval_percent;

    _exposure = 'Not exposed ';
    _outcome = 0;  
    _censor = 0;
    _past_out = 1;

	%if &_efirst1 = 1 %then %do;
	_case='3a';
	%end;

	%else %do;
	_case='6b';
	%end;
    /*_case = '0';*/

   end;  
    
   /* first interval, exposure data present under percentage rule, with censoring */

   %if &_efirst1 = 0 %then %do;   
   else if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1 <= &_studyend then do;
     _outcome = 0;  
     _censor = 1;
     _past_out = 0; 		

     _last_int_percent = ((1 + _exp_&_ename1_end - _exp_&_ename1) / (&_studyend - _intstart + 1));

     if (_last_int_percent >= _percent) then  do;
     _exposure = level_name;
     _case='3b';
     end;
     else do;
     _exposure = 'Not exposed ';
     _case='4b';
	 end;

     /*_case = '0';*/
    
    end;
   %end;

   /* first interval, exposure data present under any exposure rule, with censoring */

   %else %if &_efirst1 = 1 %then %do;
   else if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1 <= &_studyend then do;
     _exposure = level_name; 
     _outcome = 0;  
     _censor = 1;
     _past_out = 0; 		

     _case='5a';
     /*_case = '0';*/
    
    end; 
   %end;
    
   /* first interval, no exposure, censoring occurs */
  
   else do;
     _exposure = 'Not exposed ';
     _outcome = 0;  
     _censor = 1;
     _past_out = 0;
    
   %if &_efirst1 = 1 %then %do;
     _case='4a';
    %end;
	%else %do;
	_case='4b';
	%end;

   /*_case = '0'*/;
   end;

  end; 

  
  /* setting values for intervals after the first, assuming no prior exposure */ 
  
  else if _intnum < _intmax and _acnt = 0 then do;
   /* case 2 */
   if _exp_&_ename1 = . and _out_&_oname._date = . then do;
    _exposure = 'Not exposed ';
    _outcome = 0;
    _censor = 0;
    _past_out = _past_out;
    
	 %if &_efirst1 = 1 %then  %do; _case = '2a'; %end;
	 %else %do;  _case = '2b'; %end;

	 /*_case = '2';*/
    
   end;
  end;   

  /* setting values for the last interval, assuming no prior exposure */

  else if _intnum = _intmax and _acnt = 0 then do;
   /* case 3, interval t only */
   if _exp_&_ename1 = . and _out_&_oname._date ne . and _out_&_oname._date <= &_studyend then do;

    _last_int_begin = (1 + _out_&_oname._date - &_unitdays);  

    _exposure = 'Not exposed ';
    _outcome = 0;
    _censor = 0;
    _past_out = 1;
    
	 %if &_efirst1 = 1 %then  %do; _case = '3a'; %end;
	 %else %do;  _case = '6b'; %end;
     /* _case = '3'; */
    
   end;   
   /* case 4 */
   else if _exp_&_ename1 = . and _out_&_oname._date = . then do;
    _exposure = 'Not exposed ';
    _outcome = 0;
    _censor = 1;
    _past_out = _past_out;    
    
    %if &_efirst1 = 1 %then  %do; _case = '4a'; %end;
	 %else %do;  _case = '4b'; %end;

    /*_case = '4'*/;
    
   end;
  end;

  /* setting values for intervals after the first containing first exposure */ 

  else if _intnum < _intmax and _acnt = 1 and _inflect = 1 and _exp_&_ename1 ne . and _out_&_oname._date = . then do;
   /* case 1, percentage exposure */
   %if &_efirst1 = 0 %then %do;
    if _exp_&_ename1 ne . and _out_&_oname._date = . then do;

     if (_interval_percent >= _percent) then _exposure = level_name;
     else _exposure = 'Not exposed ';

     _outcome = 0;
     _censor = 0;
     _past_out = _past_out;
    
     if (_interval_percent >= _percent) then _case = '1b';
     else _case = '2b'; 
     /*_case = 1; */
    
    end;
   %end;

   /* case 1, any exposure */   
   %else %if &_efirst1 = 1 %then %do;
    if _exp_&_ename1 ne . and _out_&_oname._date = . then do;
     _exposure = level_name; 
     _outcome = 0;
     _censor = 0;
     _past_out = _past_out;
     
	 _case = '1a';
     /*_case = 1; */
    
    end;   
   %end;
  end;   

  /* setting values for the last interval with first exposure during interval */
  
  else if _intnum = _intmax and _acnt = 1 and _inflect = 1 then do;
   /* cases 5 and 6, percentage exposure */
   %if &_efirst1 = 0 %then %do;
    if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1 <= &_studyend then do;
     if _exp_&_ename1_end > &_studyend then _exp_&_ename1_end = &_studyend;
     _last_int_percent = ((1 + _exp_&_ename1_end - _exp_&_ename1) / (&_studyend - _intstart + 1));
     if (_last_int_percent >= _percent) then do;
      _exposure = level_name;
      _case='3b';
     /*_case = '5';*/
     end;
     else do;
      _exposure = 'Not exposed ';
	  _case='4b';
      /*_case = '6';*/
     end;
     _outcome = 0;  
     _censor = 1;
     _past_out = _past_out;    
    end;
   %end;
   
   /* cases 5 and 6, any exposure */
   %else %if &_efirst1 = 1 %then %do;
    if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1 <= &_studyend then do;
     if _exp_&_ename1_end > &_studyend then _exp_&_ename1_end = &_studyend;
     _exposure = level_name;
     _case = '5a';
     _outcome = 0;  
     _censor = 1;
     _past_out = _past_out;    
    end;   
   %end;   
   
   /* case 7 and 8, interval t only, percentage exposure */
   %if &_efirst1 = 0 %then %do;
    else if _exp_&_ename1 ne . and _out_&_oname._date ne . and _exp_&_ename1 <= _out_&_oname._date  <= &_studyend then do;
     if _exp_&_ename1_end > &_studyend then _exp_&_ename1_end = &_studyend;
     _last_int_begin = (1 + _out_&_oname._date - &_unitdays);        
     _last_int_percent = _interval_percent;
     _last_int_recalc = 1;
     if (_last_int_percent >= _percent) then do;
      _exposure = level_name;
      _case = '5b';
     end;
     else do;
       _exposure = 'Not exposed ';
      _case = '6b';
	  /* _case = '8';*/
     end;
     _outcome = 0;  
     _censor = 0;
     _past_out = 1;    
    end;
   %end;

   /* case 7 and 8, interval t only, any exposure */
   %if &_efirst1 = 1 %then %do;
    else if _exp_&_ename1 ne . and _out_&_oname._date ne . and _exp_&_ename1 <= _out_&_oname._date  <= &_studyend then do;
     if _exp_&_ename1_end > &_studyend then _exp_&_ename1_end = &_studyend;
     _last_int_begin = (1 + _out_&_oname._date - &_unitdays);        
     _last_int_percent = _interval_percent;
     _exposure = level_name; 
     _outcome = 0;  
     _censor = 0;
     _past_out = 1; 
     _case = '6a';       
    end;
   %end;


   /* case 9, Y before A */
   else if _out_&_oname._date ne . and _exp_&_ename1 ne . and _out_&_oname._date <= _exp_&_ename1 <= &_studyend then do;
     _exposure = 'Not exposed ';
    _outcome = 0;  
    _censor = 0;
    _past_out = 1;  
    
    _case = '7a';
    
   end;
  end;

  /* setting values for intermediary intervals after initial exposure */

   /* case 10 if clause, case 11 else clause*/
  else if _acnt >= 1 then do;
   if _intnum < _intmax then do;
    if _exp_&_ename1_end ne . then do;
     _outcome = 0;
     _censor = 0;
     _past_out = 0;	
     if _exp_&_ename1_end <=_intend then do;
      if (_interval_percent >= _percent) then do; 
       _exposure = level_name;
       %if &_efirst1 = 1 %then  %do; _case = '8a'; %end;
	   %else %do;  _case = '1b'; %end; 
       /*_case = '10';*/
      end;
      else do; 
        _exposure = 'Not exposed ';
       %if &_efirst1 = 1 %then  %do; _case = '9a'; %end;
	   %else %do;  _case = '2b'; %end; 
       /*_case = '11';*/
      end;
     end;
     else do;
      _exposure = level_name; 
       %if &_efirst1 = 1 %then  %do; _case = '8a'; %end;
	   %else %do;  _case = '1b'; %end; 
       /*_case = '10';*/

     end;
    end;
    else do;
     _outcome = _outcome;
     _censor = 0;
     _past_out = _past_out;
      _exposure = 'Not exposed ';
     %if &_efirst1 = 1 %then  %do; _case = '9a'; %end;
	  %else %do;  _case = '2b'; %end; 
     
     /*_case = '11';*/
    
    end;
   end;
    
    
   /* setting values for last interval after initial exposure */   
   else if _intnum=_intmax then do;

    /* case 12, no occurrence of A */   
    if _exp_&_ename1 = . and _out_&_oname._date = . and _exp_&_ename1_end <= &_studyend then do;	
      _exposure = 'Not exposed ';
     _censor = 1;
     _outcome = 0;
     _past_out = _past_out;    
     _last_int_percent = 0;

    
	 %if &_efirst1 = 1 %then  %do; _case = '11a'; %end;
	 %else %do;  _case = '4b'; %end;
    /*_case = '12';*/

    end;   
  
    /* case 12, A occurs before C */   
    else if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1_end <= &_studyend then do;	
     _censor = 1;
     _outcome = 0;
     _past_out = _past_out;
   	  		
     if _exp_&_ename1 < _intstart then _exp_&_ename1 = _intstart;
     if _exp_&_ename1_end >= &_studyend then _exp_&_ename1_end = &_studyend;

     _last_int_percent = ((1 + _exp_&_ename1_end - _exp_&_ename1) / (&_studyend - _intstart + 1));
     if (_last_int_percent >= _percent) then _exposure = level_name;
     else  _exposure = 'Not exposed ';
     
	 %if &_efirst1 = 1 %then  %do; 
	  if  (_last_int_percent >= _percent) then _case = '10a'; 
	  else _case = '11a'; 
	 %end;
	 %else %do; 
      if  (_last_int_percent >= _percent) then _case = '3b'; 
	  else _case = '4b'; 
     %end;
    /*_case = '12';*/
    
    end;

     /* case 12, A occurs through C */   
    else if _exp_&_ename1 ne . and _out_&_oname._date = . and _exp_&_ename1_end > &_studyend then do;	
     _censor = 1;
     _outcome = 0;
     _past_out = _past_out;
   	  		
     if _exp_&_ename1 < _intstart then _exp_&_ename1 = _intstart;
     if _exp_&_ename1_end >= &_studyend then _exp_&_ename1_end = &_studyend;

     _last_int_percent = ((1 + _exp_&_ename1_end - _exp_&_ename1) / (&_studyend - _intstart + 1));
     if (_last_int_percent >= _percent) then _exposure = level_name;
     else  _exposure = 'Not exposed ';

	 %if &_efirst1 = 1 %then  %do; 
	  if  (_last_int_percent >= _percent) then _case = '10a'; 
	  else _case = '11a'; 
	 %end;
	 %else %do; 
      if  (_last_int_percent >= _percent) then _case = '3b'; 
	  else _case = '4b'; 
     %end;

    /* _case = 12;*/
    
    end;


    /* cases 13 and 14, exposure in interval, interval t only */   
    else if _out_&_oname._date ne . and _out_&_oname._date <= &_studyend then do;	
    
     if 0 < _exp_&_ename1 < _intstart then _exp_&_ename1 = _intstart;
     if _exp_&_ename1_end >= _out_&_oname._date then _exp_&_ename1_end = _out_&_oname._date;

     if _exp_comb_&_ename1 = . and _prev_exp_comb = . then do;
      _last_int_begin = (1 + _out_&_oname._date - &_unitdays);
      _last_int_percent = 0;
     end;

     else if _exp_comb_&_ename1 ne . and _prev_exp_comb = . then do;
      _last_int_begin = (1 + _out_&_oname._date - &_unitdays);
      _last_int_percent = ((1 + _exp_&_ename1_end - _exp_&_ename1) / %eval(&_unitdays));
     end;
     
     else if _exp_comb_&_ename1 = . and _prev_exp_comb ne . then do;
      * if 0 < _prev_exp_begin < (_out_&_oname._date - &_unitdays) then _last_int_begin = (1 + _out_&_oname._date - &_unitdays);
      * else _last_int_begin = _prev_exp_begin;
      _last_int_begin = (1 + _out_&_oname._date - &_unitdays);
      _last_int_percent = .;
      * _last_int_percent = (((_last_int_begin - _prev_exp_end) + 1) / %eval(&_unitdays));
      _last_int_recalc = 1;
     end;

     else if _exp_comb_&_ename1 ne . and _prev_exp_comb ne . then do;
      * if 0 < _prev_exp_begin < (_out_&_oname._date - &_unitdays) then  _last_int_begin = (1 + _out_&_oname._date - &_unitdays);
      * else _last_int_begin = _prev_exp_begin;
      _last_int_begin = (1 + _out_&_oname._date - &_unitdays);      
      _last_int_percent = .;
      * _last_int_percent = (((_prev_exp_end - _last_int_begin) + (1 + _exp_&_ename1_end - _exp_&_ename1)) / %eval(&_unitdays));
      _last_int_recalc = 1;
     end;

     if (_last_int_percent >= _percent) then do; 
      _exposure = level_name;
      %if &_efirst1 = 1 %then %do; 
       _case ='12a';
     %end; 
     %else %do;
        _case='5b'; 
     %end;
      /*_case = '13';*/
     end;

    else do; 
       _exposure = 'Not exposed ';
      %if &_efirst1 = 1 %then %do; 
       _case ='13a';
     %end; 
     %else %do;
        _case='6b'; 
     %end;

       /*_case = '14';*/
     end;

     _censor = 0;
     _outcome = 0;
     _past_out = 1;

    end;   
  

   end;
  end;

 * if tie=1 and _exposure='Not exposed ' and _last_int_recalc ne 1 then tie=0;
 run;


 * Extract information on exposure and outcome from the basedata &varfile and source datasets;

 data _exposure_recalc;
  set &_varsdata;
  if _remove ne 1;
  if upcase(_type) = 'EXPOSURE' then output _exposure_recalc;
 run;
 
 * Exposure handling;
  data _exposure_recalc;
   set _exposure_recalc;
   _order0=_N_;
  run;
                                                         
    %let _estart = 0;
    %let _eend = 0;
    
    data _null_;
     set _exposure_recalc;
     if _n_=1;
     call symput("_estart",trim(left(_sas_start_var)));
     call symput("_eend",trim(left(_sas_end_var)));
    run;

   data _expose_binary_recalc;
    set _expose_categorical;
    if _last_int_recalc = 1;
   run;

   /* Date extraction for dynamic binary exposure */

        proc sql;
      create table _exp_recalc_&_ename1 as
       select 
       	a.&_ident,
       	a.&_studystart,
       	a.&_studyend,
       	b.&_estart,
       	b.&_eend
		%if &_exp_multi_level1=1 %then %do; 
        ,b.&_ename1 
       %end;
       from _expose_binary_recalc a,
       	    &_efile1 b
       where a.&_ident=b.&_ident and
       	      (
       	       (a._last_int_begin<=b.&_estart<=a.&_studyend) or
       	       (a._last_int_begin<=b.&_eend<=a.&_studyend) or
       	       (b.&_estart<a._last_int_begin and a.&_studyend<b.&_eend)
       	      )
     order by a.&_ident,
       %if &_exp_multi_level1=1 %then %do; 
        b.&_ename1, 
       %end;
       b.&_estart;
     quit;
     
	

     data _exp_recalc_&_ename1;
      set _exp_recalc_&_ename1;
      rename &_estart = _exp_recalc_start;
      rename &_eend = _exp_recalc_end;
     run;

    /* data _exp_recalc_&_ename1;
      set _exp_recalc_&_ename1;
      if _exp_recalc_start < &_studystart then _exp_recalc_start = &_studystart;
      if _exp_recalc_end > &_studyend then _exp_recalc_end = &_studyend;
     run;
   */


   * Merging exposure dates with MSM intervals, first exposure; 
      proc sql;
       create table _exp_recalc_merge as
        select
         a.&_ident
		,a._last_int_begin format=mmddyy10.
        ,a.&_studystart
        ,a.&_studyend
		,a._percent
		,a._intnum
		,a._last_int_recalc
        ,b._exp_recalc_start
        ,b._exp_recalc_end
		%if &_exp_multi_level1=1 %then %do; 
        ,b.&_ename1
        %end;
       from _expose_binary_recalc a,
            _exp_recalc_&_ename1 b
       where a.&_ident=b.&_ident and 
             (
              (a._last_int_begin <= b._exp_recalc_start <= a.&_studyend) or
              (a._last_int_begin <= b._exp_recalc_end <= a.&_studyend) or
              (b._exp_recalc_start < a._last_int_begin and a.&_studyend <b._exp_recalc_end)
             )
          ;
      quit;


     proc sort data=_exp_recalc_merge; 
      by &_ident %if &_exp_multi_level1=1 %then %do; &_ename1 %end; _exp_recalc_start;
     run;



	 * Calculating exposure and interval percentage using all data;      
    %if &_exp_multi_level1=1 %then %do;

     data _exp_recalc_merge;
      set _exp_recalc_merge;
      by  &_ident  &_ename1;
      retain _cml_exposure _exp_working_end;

      if _exp_recalc_start < _last_int_begin then _exp_recalc_start=_last_int_begin;
      if _exp_recalc_end > &_studyend then _exp_recalc_end = &_studyend;
      _exposure_length = _exp_recalc_end - _exp_recalc_start + 1;
      
      if first.&_ename1 and first.&_ename1 then do; 
       _cml_exposure = _exposure_length;
      end;
      
      else do; 
       _cml_exposure = _cml_exposure + _exposure_length;
      end;
        

       _exp_working_end = _last_int_begin + _cml_exposure - 1;
       _last_int_percent_recalc = _cml_exposure / %eval(&_unitdays);

   
     run;

     proc sort data=_exp_recalc_merge; 
      by &_ident &_ename1 _exp_working_end;
     run;

     data _exp_recalc_merge;
      set _exp_recalc_merge;
      by &_ident &_ename1 ;
      if  last.&_ename1 ;
      keep &_ident _intnum _last_int_recalc _last_int_percent_recalc &_ename1 _percent _last_int_recalc _exp_recalc_end ;
     run;
 

     proc sort data=_exp_recalc_merge; 
      by &_ident _last_int_percent_recalc;
    run;

  proc summary data=_exp_recalc_merge;
   by &_ident;
   var _last_int_percent_recalc;
  output out=_exp_max(drop=_type_ _freq_)
       max=_last_max_percent
	   sum=sum_percent_recalc;
   run;


     data _exp_recalc_merge;
      merge _exp_recalc_merge(in=a) _exp_max;
      by &_ident;

      retain last_tie_level  max_num  last_tie_end last_level_name;
      length  _exposure_recalc  last_tie_level $32;

     if first.&_ident then do;
       max_num=0;
       last_tie_end =_exp_recalc_end ;
	   last_tie_level =&_ename1;
	 end;

     if _last_int_percent_recalc= _last_max_percent then do;
	    if max_num =0 then do;
          last_level_name = &_ename1;
          last_tie_end =_exp_recalc_end ;
	      last_tie_level =&_ename1;
         end;
       max_num+1;

       if max_num>1  then do;
         last_tie_level =catx('',last_tie_level,&_ename1);

         if _exp_recalc_end > last_tie_end then do;
          last_tie_end = _exp_recalc_end;
	       last_level_name = &_ename1;
          end;
	    end;
      end;

	   if last.&_ident then do;

        if max_num >1 and _last_max_percent ge _percent  then tie=1 ;
	    else tie=0;

        if _last_max_percent >= _percent then _exposure_recalc =last_level_name ;
        else _exposure_recalc = 'Not exposed ' ;
    
	   %if &_efirst1 = 1 %then %do; 
        if _last_int_percent_recalc >= _percent then _case_recalc ='12a';
	   else _case_recalc = '13a'; 
      %end; 
      %else %do;
      if _last_max_percent >= _percent then _case_recalc ='5b';
	   else _case_recalc = '6b'; 
      %end;
     

      keep  &_ident _intnum _last_max_percent _exposure_recalc _case_recalc tie max_num 
            _last_int_recalc sum_percent_recalc last_tie_level last_level_name;
      output;
	  end;
	  run;

     proc sort data=_exp_recalc_merge; 
      by &_ident _intnum _last_int_recalc;
     run; 
 
     proc sort data=_expose_categorical;
      by &_ident _intnum _last_int_recalc;
     run;

      data _expose_updated;
      merge _expose_categorical(in=a) _exp_recalc_merge(in=b);
      by &_ident _intnum _last_int_recalc;
      if a;
      if b then do;
       _last_int_percent =  _last_max_percent;
       _exposure = _exposure_recalc;
	   _case = _case_recalc;
	   sum_percent =sum_percent_recalc;
	   tie_level= last_tie_level;
	   level_name =last_level_name;
      end;
      drop  _last_max_percent _exposure_recalc _case_recalc sum_percent_recalc last_tie_level last_level_name;
     run;
   %end;

  %else %do; *for binary exposure;
   data _exp_recalc_merge;
      set _exp_recalc_merge;
      by &_ident;
      retain _cml_exposure _exp_working_end;
      if _exp_recalc_start < _last_int_begin then _exp_recalc_start=_last_int_begin;
      if _exp_recalc_end > &_studyend then _exp_recalc_end = &_studyend;
      _exposure_length = _exp_recalc_end - _exp_recalc_start + 1;
      
      if first.&_ident then do; 
       _cml_exposure = _exposure_length;
      end;
      
      else do; 
       _cml_exposure = _cml_exposure + _exposure_length;
      end;
      
	   length  _exposure_recalc $32;
       _exp_working_end = _last_int_begin + _cml_exposure - 1;
       _last_int_percent_recalc = _cml_exposure / %eval(&_unitdays);
       if _last_int_percent_recalc >= _percent then _exposure_recalc = '1';
       else _exposure_recalc ='Not exposed ';

	 %if &_efirst1 = 1 %then %do; 
       if _last_int_percent_recalc >= _percent then _case_recalc ='12a';
	   else _case_recalc = '13a'; 
     %end; 
     %else %do;
       if _last_int_percent_recalc >= _percent then _case_recalc ='5b';
	   else _case_recalc = '6b'; 
     %end;

     run;

     proc sort data=_exp_recalc_merge; 
      by &_ident _exp_working_end;
     run;

     data _exp_recalc_merge;
      set _exp_recalc_merge;
      by &_ident;
      if last.&_ident;
      keep &_ident _intnum _last_int_recalc _exposure_recalc _last_int_percent_recalc _cml_exposure _case_recalc;
     run;

     proc sort data=_exp_recalc_merge; 
      by &_ident _intnum _last_int_recalc;
     run; 
 
     proc sort data=_expose_categorical;
      by &_ident _intnum _last_int_recalc;
     run;

     data _expose_updated;
      merge _expose_categorical(in=a) _exp_recalc_merge(in=b);
      by &_ident _intnum _last_int_recalc;
      if a;
      if b then do;
       _last_int_percent = _last_int_percent_recalc;
       _exposure = _exposure_recalc;
	   _case = _case_recalc;
      end;
      drop _cml_exposure _last_int_percent_recalc _exposure_recalc _case_recalc;
     run;
  %end;


	 *Identify and assign missing values to patients with no exposure data in composite interval;  

     data _expose_binary_recalc_list;
      set _expose_binary_recalc;
      keep &_ident _intnum;
     run;
  
     proc sort data=_expose_binary_recalc_list; 
      by &_ident _intnum;
     run;   
     
     data _expose_recalc_missing;
      merge _expose_binary_recalc_list(in=a) _exp_recalc_merge(in=b);
      by &_ident _intnum;
      if a and not b;
     run;
 
     data _expose_recalc_missing;
      set _expose_recalc_missing (keep = &_ident _intnum);
      _exposure_recalc = 'Not exposed ';
	  
	%if &_efirst1 = 1 %then %do; 
      _case_recalc ='13a';
     %end; 
     %else %do; 
       _case_recalc ='6b'; 
     %end;

      _last_int_percent_recalc = 0;
      _last_int_recalc = 1;
     run;

     proc sort data=_expose_recalc_missing; 
      by &_ident _intnum _last_int_recalc;
     run;   

     proc sort data=_expose_updated;
      by &_ident _intnum _last_int_recalc;
     run;
     
     data _expose_final;
      merge _expose_updated(in=a) _expose_recalc_missing(in=b);
      by &_ident _intnum _last_int_recalc;
      if a;
      if b then do;
       _last_int_percent = _last_int_percent_recalc;
       _exposure = _exposure_recalc;
	   _case = _case_recalc;
      end;

     length _prev_exposure $32 ;  

     _prev_exposure=lag1(_exposure);

    if first.&_ident then do;
    _prev_exposure='Not exposed '; 
    end;
      drop _last_int_recalc _last_int_percent_recalc _exposure_recalc _case_recalc;
     run;


 *pull out exposure end date in tie level, using actual end date instead of calculating date in program _exposureint
 for multiple periods of exposure in one interval 6/6/2016  ;

%if &_exp_multi_level1=1 %then %do;

data  exposure_tie;
set _expose_final;

%if &_efirst1 = 1 %then %do; 
   where tie=1 and (index(tie_level,trim(_exposure)) > 0 or (_acnt=1 and _inflect=1));
%end;

%else %do;
where tie=1 and  index(tie_level,trim(_exposure)) > 0 ;
%end;

if _intend gt &_studyend then _intend = &_studyend ;
if _past_out =1 and _intmax ne 0 then _intstart = _intend - &_unitdays +1;
run; 

   proc sql;
      create table _exp_tie_end as
       select 
       	a.&_ident,
       	a._intstart,
		a._intnum,
       	a._intend,
       	b.&_estart,
       	b.&_eend,
		b.&_ename1 
       from exposure_tie a,
       	    &_efile1 b
       where a.&_ident=b.&_ident and index(tie_level,trim(b.&_ename1))>0 and
       	      (
       	       (a._intstart<=b.&_estart<=a._intend) or
       	       (a._intstart<=b.&_eend<= a._intend) or
       	       (b.&_estart<a._intstart and b.&_eend <a._intend)
       	      )
     order by a.&_ident, a._intnum,  b.&_ename1, b.&_estart;
     quit;

data _exp_tie_end1;
set _exp_tie_end;
by &_ident _intnum &_ename1;
if last.&_ename1;
if &_eend gt _intend then &_eend = _intend;
run;

data _exp_tie_end2 (keep= &_ident _intnum exposure_rev) ;
set  _exp_tie_end1;
by  &_ident _intnum;
length exposure_rev $32.;
if first.&_ident and first._intnum then do;
tie_end =.;
exposure_rev='';
end;

retain tie_end exposure_rev;
if &_eend gt tie_end then do;
tie_end=&_eend;
exposure_rev=&_ename1;
end;
if  last._intnum then output;
run;

data _expose_final;
merge  _expose_final(in=a) _exp_tie_end2 ;
by  &_ident _intnum;
if missing(exposure_rev)=0 then _exposure = exposure_rev;
*drop exposure_rev;
run;
%end;

 * Create t+1 intervals, DEATH ONLY;
 data _add_intervals;
  set _expose_final;
  if _past_out = 1;
 run;
 
 proc sort data=_add_intervals;
  by &_ident _intnum;
 run;


 data _add_intervals;
  set _add_intervals;
  retain _intstart_new _intend_new _intnum_new;
  do _i=1 to 2;
   if _i=1 then do;
    _intstart_new=_intstart; 
    _intend_new=_intend; 
    _intnum_new=_intnum;
    output;
   end;
   else do;
    _intstart_new=_intend_new+1; 
    _intend_new=_intstart_new+&_unitdays-1;
    _intnum_new=_intnum_new+1;
   
    _exposure = '';
    _outcome = 1;
    _censor = .; 
	%if &_exp_multi_level1=1 %then %do;
    tie=0;
	%end;
    output;
   end;
  end;
  format _intstart_new _intend_new mmddyy10.;
 run;	

 data _tplusone_intervals;
  set _add_intervals (drop = _intstart _intend _intnum);
  if _i = 2;
  rename _intstart_new = _intstart
  _intend_new = _intend
  _intnum_new = _intnum;
  drop _i;
 run;

/* need to add exposure eligibility variable after checking logic with Romain */

 data _exposure_intervals; 
  set _expose_final _tplusone_intervals;
  format _prev_exp_begin _prev_exp_end _prev_exp_comb mmddyy10.;  
 run;

 proc sort data=_exposure_intervals;
  by &_ident _intnum;
 run;

 
 data _exposure_intervals;
  set _exposure_intervals;
  by &_ident;

  length _prev_exposure $32 ;  

  _prev_exposure=lag1(_exposure);

  if first.&_ident then do;
  _prev_exposure='Not exposed '; 
  end;
 run;

 *create _d_date for binary exposure;
 %if &_exp_multi_level1=0 %then %do;  
  data _expos_change _expos_no_change _expos_ending;
  set _exposure_intervals;
  if _exposure = _prev_exposure then output _expos_no_change;
  else if _exposure = 'Not exposed ' and _prev_exposure = 'Not exposed ' then output _expos_no_change;
  else if _intnum = 0 then output _expos_no_change;
  else if _intnum > _intmax then output _expos_ending;
  else output _expos_change;
 run;

 data _expos_change;
  set _expos_change;
  if _last_int_begin ne . then do; 
   _change_int_begin = _last_int_begin;
   _change_int_end = &_studyend;
  end;
  
  else do;
   _change_int_begin = _intstart;
   _change_int_end = _intend;   
  end;
  
  format _change_int_begin _change_int_end mmddyy10.; 
 run;


   * Date extraction for dynamic binary exposure; 

     proc sql;
      create table _exp_change_&_ename1 as
       select 
       	a.&_ident,
       	a.&_studystart,
       	a.&_studyend,
       	b.&_estart,
       	b.&_eend
    
       from _expos_change a,
       	    &_efile1 b
       where a.&_ident=b.&_ident  and
       	      (
       	       (a._change_int_begin<=b.&_estart<=a._change_int_end) or
       	       (a._change_int_begin<=b.&_eend<=a._change_int_end) or
       	       (b.&_estart<a._change_int_begin and a._change_int_end<b.&_eend)
       	      );
     quit;

     proc sort data=_exp_change_&_ename1;
      by &_ident &_estart;
     run;
	
     data _exp_change_&_ename1;
      set _exp_change_&_ename1;
      rename &_estart = _exp_change_start;
      rename &_eend = _exp_change_end;
     run;

     data _exp_change_&_ename1;
      set _exp_change_&_ename1;
      if _exp_change_start < &_studystart then _exp_change_start = &_studystart;
      if _exp_change_end > &_studyend then _exp_change_end = &_studyend;
     run;

   * Merging exposure dates with MSM intervals, first exposure; 
      proc sql;
       create table _exp_change_merge as
        select
         a.*
        ,b._exp_change_start
        ,b._exp_change_end
       from _expos_change a,
            _exp_change_&_ename1 b
       where a.&_ident=b.&_ident and 
             (
              (a._change_int_begin <= b._exp_change_start <= a._change_int_end) or
              (a._change_int_begin <= b._exp_change_end <= a._change_int_end) or
              (b._exp_change_start < a._change_int_begin and a._change_int_end <b._exp_change_end)
             );
      quit;
    
 proc sort data=_exp_change_merge nodup;
  by &_ident _intnum _exp_change_start;
 run;
 
 data _exp_change_merge;
  set _exp_change_merge;
  by &_ident _intnum;
  if first._intnum;
 run;


 data _exp_change_merge;
  set _exp_change_merge;
  if _exposure ne 'Not exposed ' and _prev_exposure = 'Not exposed ' and _exp_change_start <= _change_int_begin  then _d_date = _change_int_begin;
  else if _exposure ne 'Not exposed ' and _prev_exposure = 'Not exposed ' and _exp_change_start > _change_int_begin  then _d_date = _exp_change_start;
  else if _exposure = 'Not exposed ' and _prev_exposure ne 'Not exposed ' and _exp_change_start <= _change_int_begin then _d_date = _exp_change_end + 1;
  else if _exposure = 'Not exposed ' and _prev_exposure ne 'Not exposed ' and _exp_change_start > _change_int_begin then _d_date = _change_int_begin;
  format _d_date mmddyy10.;
 run;
 
 * Identify and assign missing values to patients with no exposure data in target interval;  

 data _expos_change_list;
  set _expos_change;
  keep &_ident _intstart _intend _intnum _change_int_begin _change_int_end _prev_exposure;
 run;
 
 proc sort data=_expos_change_list; 
  by &_ident _intnum;
 run;   
 
 data _expose_change_missing;
  merge _expos_change_list(in=a) _exp_change_merge(in=b);
  by &_ident _intnum;
  if a and not b;
 run;
 
 data _expose_change_missing;
  set _expose_change_missing (keep = &_ident _intstart _intend _intnum _change_int_begin _change_int_end);
  _d_date = _change_int_begin;
  _exp_change_start = .;
  _exp_change_end = .;
 run;

 * re-merge all records needing _d_date;
 
 data _exp_change_mergefinal;
  set _exp_change_merge;
  keep &_ident _intstart _intend _intnum  _d_date;
 run;
 
 data _exp_change_newdates;
  set _exp_change_mergefinal _expose_change_missing;
  keep &_ident _intstart _intend _intnum _d_date;
 run;
 
 proc sort data=_exp_change_newdates; 
  by &_ident _intnum _intstart _intend;
 run;   

 proc sort data=_exposure_intervals;
  by &_ident _intnum _intstart _intend;
 run;    

 * merge generated variables back into the output dataset;
 
 data _exposureint_output_final;
  merge _exposure_intervals(in=a) _exp_change_newdates(in=b);
  by &_ident _intnum _intstart _intend;
  if a;
 run;

 data _workdata;
  set _exposureint_output_final;
  if _intnum = 0 and _intmax = 0 and _exposure ne 'Not exposed' then _d_date = _intstart;   
  drop tie tie_last_end tie_level max_num sum_percent _prev_exposure level_name;
  
   *convert to numrical value if binary exposure;  
  %if  &_exp_multi_level1=0 %then %do;   
  if _exposure in ('Not exposed ') then _exposure_n=0;
  else if _intnum >_intmax then _exposure_n=.;
  else _exposure_n=input(compress(_exposure), 8.);
  drop _exposure;
  rename _exposure_n=_exposure;
  %end;
 format _prev_exp_begin _prev_exp_end _prev_exp_comb mmddyy10.;  
 run;
%end; *end binary exposure;


 %if &_exp_multi_level1=1 %then %do;  

  data _exposure_intervals;
  set _exposure_intervals;
  by &_ident;

  length _prev_exposure $32 ;  

  _prev_exposure=lag1(_exposure);

  if first.&_ident then _prev_exposure='Not exposed '; 
  
  *if _intnum = 0 and _intmax = 0 and _exposure ne 'Not exposed' then _d_date = _intstart;   

 * for categorical exposure, create A0_warn and tie variables;
    
  *if two exposure levels have equal frequencies then we set missing for _exposure; 
   %if &_efirst1 = 1 %then %do; 
    *if tie=1 and _acnt=1 and _inflect =1 then _exposure =level_name;
    if tie=1 and (_last_int_percent ge _percent | _interval_percent ge _percent) 
     and index(tie_level, trim(_prev_exposure)) > 0 then  _exposure =_prev_exposure;
  %end;
  
   %else %do;
   if tie=1 and (_last_int_percent ge _percent or _interval_percent ge _percent) then do;
    if index(tie_level, trim(_prev_exposure)) > 0  then _exposure =_prev_exposure;
   end;
  %end;

  /*flag max exposure percentage less then defalut exposure percent but sum of percent greater then exposure percent*/  

  if tie=1 and _exposure='Not exposed ' then tie=0;

  if _exposure = 'Not exposed ' and round(sum_percent,0.0001) ge _percent then A0_warn=1;
   else A0_warn=0 ;
  
run;

*fix exposure with  2 countinous tie (tie at t and t-1) for categorical exposure;

data _exposure_intervals;
set _exposure_intervals;
by &_ident;

_prev_exposure_rev=lag1(_exposure);

if first.&_ident then  _prev_exposure_rev='Not exposed '; 

 if tie=1 and _intnum ne 0 then do;
 if index(tie_level, trim(_prev_exposure_rev)) > 0 then  _exposure =_prev_exposure_rev;
  else _exposure = exposure_rev;
end;

_prev_exposure_new = lag1(_exposure);
if _intnum =0 then _prev_exposure_new='Not exposed '; 


*fix exposure start and end data when exposure changed at tie level;
array leveln(&n) level_name1 - level_name&n;
array exp_end(&n) _exp_end1 - _exp_end&n;
array exp_start(&n) _exp_start1 - _exp_start&n;
 do i= 1 to &n;
   if leveln(i) = _exposure then do;
     _exp_&_ename1= exp_start(i);
	 _exp_&_ename1._end=exp_end(i);
   end;
end;

drop _prev_exposure exposure_rev _prev_exposure_rev i level_name1 - level_name&n _exp_end1 - _exp_end&n _exp_start1 - _exp_start&n;
rename _prev_exposure_new = _prev_exposure;
run;


*get d_date when exposure(t) not equal exposure(t-1) and exposure(t) =any level;
data  _expo_change1;
set _exposure_intervals; 
*for last interval with outcome, reset intstart and intend  ;
if _past_out =1 then do;
 _intend = &_studyend; 
 _intstart= _last_int_begin;
end;

where  _exposure ne _prev_exposure & missing(_exposure) =0; 
run; 

proc sql;
      create table _exp_change_&_ename1 as
       select 
       	a.&_ident,
		a._intnum,
       	a._intstart,
       	a._intend,
		a._exposure,
		a._prev_exposure,
       	b.&_estart,
       	b.&_eend,
		b.&_ename1 
    
       from _expo_change1 a,
       	    &_efile1 b
       where a.&_ident=b.&_ident  and a._exposure=b.&_ename1 and 
       	      (
       	       (a._intstart<=b.&_estart<=a._intend) or
       	       (a._intstart<=b.&_eend<=a._intend) or
       	       (b.&_estart<a._intstart and a._intend<b.&_eend)
       	      )
    order by  a.&_ident, a._intnum, &_estart;;
    quit;


  data _exp_change_&_ename1;
  set _exp_change_&_ename1;
  by &_ident _intnum;
  if first._intnum;

  if &_estart <= _intstart then _d_date = _intstart;
  else if &_estart > _intstart then _d_date = &_estart;

   format _d_date mmddyy10.;
 run;


*get d_date when exposure(t) ='not exposed' and exposure(t-1) =any one level;
data  _expo_change2;
set _exposure_intervals; 
*for last interval with outcome, reset intstart and intend  ;
if _past_out =1 then do;
 _intend = &_studyend; 
 _intstart= _last_int_begin;
end;
where  _exposure ='Not exposed' and _prev_exposure ne 'Not exposed';
run; 

proc sql;
      create table _exp_change as
       select 
       	a.&_ident,
		a._intnum,
       	a._intstart,
       	a._intend,
		a._exposure,
		a._prev_exposure,
       	b.&_estart,
       	b.&_eend,
		b.&_ename1 
    
       from _expo_change2 a left join 
       	    &_efile1 b
       on a.&_ident=b.&_ident   and
       	      (
       	       (a._intstart<=b.&_estart<=a._intend) or
       	       (a._intstart<=b.&_eend<=a._intend) or
       	       (b.&_estart<a._intstart and a._intend<b.&_eend)
       	      )
    order by  a.&_ident, a._intnum, &_estart;;
    quit;

  data _exp_change1;
  set _exp_change;
  by &_ident _intnum;
  retain _d_date dflag;
  
   last_exp =lag1(&_eend);
   format _d_date last_exp mmddyy10.;

   if first._intnum then do;
    _d_date =.;
    last_exp=.;
    dflag =0;

     if &_estart > _intstart then do;
    _d_date =_intstart;
    dflag=1;
    end;
   end;
  

  if first._intnum & last._intnum & dflag=0 then do;
      if missing(&_estart) =1  then _d_date=_intstart;
       else if &_eend < _intend then _d_date = &_eend+1;
	   else _d_date =_intend;
	  dflag=1;
   end;

   if dflag=0 and missing(last_exp)= 0 and &_estart > last_exp+1 then do; 
    _d_date= last_exp +1;
	 dflag=1;
  end;

 if last._intnum then do;
   if dflag = 0 then do;
    if &_eend < _intend then _d_date = &_eend+1;
	else _d_date= _intstart; *if no unexposed day during interval, then _intstart set as _d_date;
    end; 
   keep &_ident _intnum _intstart _intend _exposure _prev_exposure _d_date ;
   output;
  end;
  format last_exp _d_date mmddyy10.;
 run;


*merge _d_date to main data set;
data _exposureint_output_final;
  merge _exposure_intervals(in=a) _exp_change_&_ename1(keep=&_ident _intnum  _d_date)
        _exp_change1(keep=&_ident _intnum  _d_date) ;
  by &_ident _intnum ;
  if a;
  if _intnum ne 0  and _exposure =lag1(_exposure) then _d_date=.; 
 run;

data _workdata;
set _exposureint_output_final;
run;
%end;
%mend;				
	


 
