/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


/*********** MSM Exposure and Outcome Extraction Macro - Macro 2 in MSM Macro Series *************
 Second internal macro of MSM dataset creation
        - reads in exposure and outcome data from source datasets
	- merges exposure and outcome data with intervals

Variables:
	_inexposureint = input dataset, assumed to be in baseline format
	_studystart = index date, sas date format
	_studyend = censor date, sas date format
	_eoftype = type of censoring (e.g. end of study, death, end of follow-up)
	_deathvalue = value of _eoftype corresponding to censoring at death
	_ident = patient identifier
	_varsexposureint = file location and name of the input variables dataset
**************************************************************************************************/
%macro _exposureint(_inexposureint,_studystart,_studyend,_eoftype,_deathvalue,_ident,_varsexposureint);
* Setup of input datasets from previous MSM macros;

* Create variable for number of intervals per MRN;
 
 data _workcount; 
   set _workdata (keep = &_ident _intnum); 
  run;
  
  proc sort data=_workcount;
   by &_ident descending _intnum;
  run;
  
  data _workcount;
   set _workcount;
   by &_ident;
   if first.&_ident;
   rename _intnum = _intmax;
  run;
  
  data _workdata;
   merge _workdata _workcount;
   by &_ident;
  run;
  
  proc datasets library = work nolist;
   delete _workcount;
  quit;


* Extract information on exposure and outcome from the basedata &varfile and source datasets;
 data _exposure _outcome;
  set &_varsexposureint;
  if _remove ne 1;
  if upcase(_type) = 'EXPOSURE' then output _exposure;
  else if upcase(_type) = 'OUTCOME' then output _outcome;
 run;
 * Exposure handling;
  data _exposure;
   set _exposure;
   _order0=_N_;
  run;
                                                         
  %_nobs_msm(_exposure);

  %global _nobs_exposure;
  %let _nobs_exposure = %eval(&_nobs);
  %put &_nobs_exposure;

 %if %eval(&_nobs_exposure) > 1 %then %do;
  %put 'MACRO ERROR DETECTED: Multiple exposures not currently supported';
  %goto exit_exposureint;
 %end;

* Extract dates of exposure events;
* Convert values passed from &_varsdata into macro variables;

%do _i=1 %to &_nobs_exposure;

*for multiple levels of exposure;
    %global _exp_var&_i;
	%global _exp_sourcedata&_i;
	%global _exp_fmt&_i;
	%global _exp_multi_level&_i;

    data _exposure;
     set _exposure;
      call symput("_exp_var&_i",lowcase(trim(left(_variable))));
      call symput("_exp_sourcedata&_i",trim(left(_sas_data_name)));
	  call symput("_exp_fmt&_i",trim(left(_varfmt)));
     if _varfmt in ('C','N') then call symput("_exp_multi_level&_i",1);
	 else call symput("_exp_multi_level&_i",0);
   run; 

   *if numrical exposure level then convert to char value in the exposure data set;
   %if  &&&_exp_fmt&_i = N %then %do;
   data  exp_source;
   set &&&_exp_sourcedata&_i ;
   exp_c= trim(input(&&&_exp_var&_i,$12.));
   drop &&&_exp_var&_i;
   rename exp_c= &&&_exp_var&_i;
  run;
  %let _exp_sourcedata&_i =exp_source;
  %put 	&&&_exp_sourcedata&_i;
  %end;

 

    %global _ename&_i;
    %let _ename&_i = 0;
    %global _efile&_i;
    %let _efile&_i = 0;
    %global _ebehavior&_i;
    %let _ebehavior&_i = 0;
    %global _edate&_i;
    %let _edate&_i = 0;
    %global _efirst&_i;
    %let _efirst&_i = 0;    
    %let _estart&_i = 0;
    %let _eend&_i = 0;
    %let _subi = %eval(&_i - 1) ;
	%global _epercent&_i;
    %let _epercent&_i = 0;

    data _null_;
     set _exposure;
     if _n_=&_i;
     call symput("_ename&_i",trim(left(_variable)));
     call symput("_efile&_i",trim(left(_sas_data_name)));
     call symput("_ebehavior&_i",trim(left(_behavior)));
     call symput("_edate&_i",trim(left(_sas_date_var)));
     call symput("_estart&_i",trim(left(_sas_start_var)));
     call symput("_eend&_i",trim(left(_sas_end_var)));
     call symput("_efirst&_i",trim(left(_first_exp_rule)));
     call symput("_epercent&_i",trim(left(_value)));
    run;
   
    %if  &&&_exp_fmt&_i = N %then %do;
     %let _efile&_i =exp_source;
     %put 	&&&_efile&_i;
     %end;


    %put _i=&_i;
    %put _ename=&&&_ename&_i;
    %put _efile=&&&_efile&_i;
    %put _ebehavior=&&&_ebehavior&_i;
    %put _edate=&&&_edate&_i;
    %put _estart=&&&_estart&_i;
    %put _eend=&&&_eend&_i;
    %put _efirst=&&&_efirst&_i;
    %put _epercent = &&&_epercent&_i;
    %put ;
    %put ;

   %global n; *total level for each exposure;
 

  %if &&&_exp_multi_level&_i =1 %then %do;
   proc sql noprint;
   select count(distinct &&&_exp_var&_i) into :_elevel&_i
   from &&&_exp_sourcedata&_i;
  
   %put &&&_elevel&_i;
  
   %let n=&&&_elevel&_i;

   select distinct &&&_exp_var&_i into :_elevelname&_i.1 - :_elevelname&_i&n 
   from &&&_exp_sourcedata&_i;
   quit;
  %end; 



  *no any level for exposure then set level=1;
   
   %if &&&_exp_multi_level&_i =0  %then %do;
   %let n =1;
   %let _elevelname&_i.1=1;
   data exp&_i._&&&_elevelname&_i.1;
   set &&&_efile&_i;
   run; 
   %end;
   
  
*===============
 Outcome handling
================;

  %_nobs_msm(_outcome);
  %global _nobs_outcome;
  %let _nobs_outcome = %eval(&_nobs);
  %put  &_nobs_outcome;

  
   %if &_nobs_outcome > 1 %then %do;
      %put 'MACRO ERROR DETECTED: Non-binary fixed outcomes not currently supported';
      %goto exit_exposureint;
    %end;

  * Extract dates of outcome events;

   * Convert values passed from &_varsdata into macro variables;
  

    %global _oname;
    %let _oname = 0;
    %global _ofile;
    %let _ofile = 0;
    %global _obehavior;
    %let _obehavior = 0;
    %global _odate;
    %let _odate = 0;

    data _null_;
     set _outcome;
     if _n_=1;
     call symput("_oname",trim(left(_variable)));
     call symput("_ofile",trim(left(_sas_data_name)));
     call symput("_obehavior",trim(left(_behavior)));
     call symput("_odate",trim(left(_sas_date_var)));
 
    run;
    
    
    %put _oname=&_oname;
    %put _ofile=&_ofile;
    %put _obehavior=&_obehavior;
    %put ;
    %put ;
    %put ;

   * Extract dates of outcome events - special case of death;
    %if &_obehavior = 0 %then %do;
     proc sql;
      create table _msm_dates_&_oname as
       select
        a.&_ident
       ,a.&_studyend
      from &_inexposureint a
      where a.&_eoftype = &_deathvalue
     order by &_ident;
     quit;

    
     data _msm_dates_&_oname;
      set _msm_dates_&_oname;
      rename &_studyend = _out_&_oname._date;
     run;
    %end;

       
     %else %if &_obehavior = 1 %then %do;    
      proc sql;
       create table _msm_dates_&_oname as
        select
         a.&_ident
        ,b.&_odate
       from &_inexposureint a,
            &_ofile b
       where a.&_ident=b.&_ident and 
             (a.&_studystart <= b.&_odate <= a.&_studyend);
      quit;
     
      proc sort data=_msm_dates_&_oname;
       by &_ident &_odate;
      run;

     data _msm_dates_&_oname;
      set _msm_dates_&_oname;
      rename &_odate = _out_&_oname._date;
     run;
     
     %end; 

	* Merging outcome event dates with MSM intervals;
    proc sql;
     create table _workdata_out as
      select
       a.*
      ,b._out_&_oname._date
     from _workdata a,
          _msm_dates_&_oname b
     where a.&_ident=b.&_ident and 
           (a._intstart <= b._out_&_oname._date <= a._intend)
     order by &_ident, _intstart, _intend; ;
    quit;
   
*==========================
 handle each exposure level
===========================;

  %do j =1 %to &n;
  %global _leveln&_i&j;
  %let _leveln&_i&j=&&&_elevelname&_i&j;

   %if &&&_exp_multi_level&_i =1 %then %do;
   *subset exposure data by exposure level;
   data exp&_i._&&&_elevelname&_i&j;
   set &&&_efile&_i;
   where &&&_ename&_i = "&&&_elevelname&_i&j";
   run; 
   %end;


  /* Date extraction for dynamic binary exposure */
  %if &&&_ebehavior&_i=2 %then %do;
   * pull out exposure records during study period;
   proc sql; 
      create table _msm_dates&_i._&j as
       select 
       	a.&_ident,
		a.&_studyend,
       	b.&&&_estart&_i,
       	b.&&&_eend&_i
       from &_inexposureint a,
       	     exp&_i._&&&_elevelname&_i&j b
       where a.&_ident=b.&_ident and
       	      (
       	       (a.&_studystart<=b.&&&_estart&_i<=a.&_studyend) or
       	       (a.&_studystart<=b.&&&_eend&_i<=a.&_studyend) or 
               (b.&&&_estart&_i < a.&_studystart and b.&&&_eend&_i > a.&_studyend) 
       	      );
     quit;
     
     proc sort data= _msm_dates&_i._&j;
      by &_ident &&&_estart&_i;
     run;
	
     data _msm_dates&_i._&j;
      set _msm_dates&_i._&j;
      rename &&&_estart&_i = _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
      rename &&&_eend&_i=_exp_&&&_ename&_i.._end_&&&_elevelname&_i&j;
     run;
   %end; 
   
  

* Merging exposure dates with MSM intervals;
    
     proc sql;
       create table _workdata_exp&_i._&j as 
      select
         a.*
		,b.&_studyend
        ,b._exp_&&&_ename&_i.._&&&_elevelname&_i&j
        ,b._exp_&&&_ename&_i.._end_&&&_elevelname&_i&j
       from _workdata a,
            _msm_dates&_i._&j b
       where a.&_ident=b.&_ident and 
             (
              (a._intstart <= b._exp_&&&_ename&_i.._&&&_elevelname&_i&j <= a._intend) or
              (a._intstart <= b._exp_&&&_ename&_i.._end_&&&_elevelname&_i&j <= a._intend) or
              (b._exp_&&&_ename&_i.._&&&_elevelname&_i&j < a._intstart and a._intend <b._exp_&&&_ename&_i.._end_&&&_elevelname&_i&j)
             )
     order  by  &_ident, _intstart, _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
      quit;
     
   data _workdata_exp&_i._&j;
   set _workdata_exp&_i._&j;
   _exp_comb_&&&_ename&_i.._&&&_elevelname&_i&j= _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
   format _exp_comb_&&&_ename&_i.._&&&_elevelname&_i&j mmddyy10.;
  run;

    data _workdata&_i._&j;
	  merge _workdata(in=a) _workdata_exp&_i._&j;
	  by &_ident _intstart;
      if a;
      run;

  
    data _single_recs _multi_recs;
      set _workdata&_i._&j;
      by &_ident _intstart;
      if first._intstart and last._intstart then output _single_recs;
      else output _multi_recs;
     run;
     
     data _multi_recs;
      set _multi_recs;
     *for last interval if exposure ends after the follow up date, then exposure end date truncated to the date of end of follow-up;
	  if _intnum=_intmax and  &_studyend lt _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j then 
         _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j =&_studyend ;

      if 0 < _intstart < _exp_&&&_ename&_i.._&&&_elevelname&_i&j then _starting_temp = _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
      else if _exp_&&&_ename&_i.._&&&_elevelname&_i&j <= _intstart then _starting_temp = _intstart;

      if 0 < _intend < _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j then _ending_temp = _intend;
      else if _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j <= _intend then _ending_temp = _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j;
      
      _exposure_length = _ending_temp - _starting_temp + 1;

      format _starting_temp _ending_temp mmddyy10.;
     run;


     data _multi_recs;
      set _multi_recs;    
      by &_ident _intstart;
	   retain _cml_exposure  _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra;
      if first._intstart then do;
       _cml_exposure = _exposure_length;
       _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra= _starting_temp;
      end;
      else do;
       _cml_exposure = _cml_exposure + _exposure_length;
       _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra = _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra;   
      end;
      
     _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra_end = _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra+ _cml_exposure - 1;
      
      format _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra_end mmddyy10.;
     run;
     
     proc sort data=_multi_recs;
      by &_ident _intstart _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
     run;
     
     data _multi_recs;
      set _multi_recs;
      by &_ident _intstart _exp_&&&_ename&_i.._&&&_elevelname&_i&j;
      if last._intstart;
      
      _exp_&&&_ename&_i.._&&&_elevelname&_i&j = _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra;
      _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j = _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra_end;
      format _exp_&&&_ename&_i.._&&&_elevelname&_i&j _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j mmddyy10.;
      
      drop _starting_temp _ending_temp _exposure_length  _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra
           _exp_&&&_ename&_i.._&&&_elevelname&_i&j.._pra_end _cml_exposure;

     run;

     data _exposure&_i._&j;
      set _multi_recs _single_recs;
	  by &_ident _intnum;
	  drop &_studyend;
     run;

   *Merging each exposure level and outcome, if exposure end date afetr outcome date, it is truncated to outcome dt;  

   data _workdata_out&_i._&j;
     merge  _workdata_out(in=a) _exposure&_i._&j(in=b);
     by  &_ident _intstart;
     if b;
    if last.&_ident and missing(_out_&_oname._date) =0 and _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j gt _out_&_oname._date then 
    _exp_&&&_ename&_i.._end_&&&_elevelname&_i&j = _out_&_oname._date;
   run;



  *if first exposure and first level, the data as base data; 

    %if  %eval(&j) = 1 %then %do;
	 data _workdata_all;
      set _workdata_out1_1;
     run;
    %end; 

    %else %do; 
     data _workdata_all;
     merge _workdata_all(in=a) _workdata_out&_i._&j;
     by &_ident _intstart; 
     if a;
     run;
   %end;

   *Delete temporary datasets;
   proc datasets library = work nolist;
    delete exp&_i._&&&_elevelname&_i.1 _msm_dates&_i._&j  _workdata_exp&_i._&j _exposure&_i._&j 
     _workdata&_i._&j _workdata_out&_i._&j; 
   run;
   quit;
  
 %end; *end loop level j; 
%end;  *end i loop ;


* Mergingexposure/outcome dates with &_studystart and &_studyend;
    proc sql;
     create table _workdata_all_final as
      select
       a.*
      ,b.&_studystart
      ,b.&_studyend
      ,b.&_eoftype
     from _workdata_all a,
          &_inexposureint b
     where a.&_ident=b.&_ident
	 order by &_ident, _intstart, _intend;
    quit;

    * output dataset for next macro;
    data _workdata;
     set _workdata_all_final;
    run;

 *Delete temporary datasets;
   proc datasets library = work nolist;
    delete  _exposure _outcome _msm_dates_&_oname 
    _workdata_out _workdata_all   _workdata_all_final;; 
   run;
   quit;
 

%exit_exposureint: 
%mend _exposureint;


