/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/



/******** Converting our MSM dataset for use with Miguel Hernan's MSM macro **************/
/** Steps needed:
	1. Create new variable Y(t+1) and C(t+1)
	2. "Roll up" last row of data, so we do not have an extra row
	3. if Y(t+1)=1, then C(t+1)=.; and if C(t+1)=1 then Y(t+1)=.
	
  Assumptions:
  	1. Dataset in question is derived from running through MSM macros
  	2. Variables in dataset follow exact naming from MSM macro
  	
  Variables:
  	_outcome_yt1=Y(t+1) represents the t+1 value for outcomes, to be used in Miguel's macro
  	_censor_ct1=C(t+1) represents the t+1 value for censors, to be used in Miguel's macro
*****************************************************************************************/


%macro _harvardformat(_msmdata,_ident);

proc sort data=&_msmdata out=_workdata; 
	by &_ident descending _intnum;
run;

data _workdata;
	set _workdata;
		by &_ident descending _intnum;
	_censor_ct1=_censor;
	_outcome_yt1=lag1(_outcome);
	if first.&_ident then _outcome_yt1=_outcome;
	if last.&_ident and _intnum=0 and _censor=1 then do;
		_censor_ct1=_censor;
		_outcome_yt1=.;	
	end;
	else if first.&_ident and _censor=1 and _intnum=_intmax then do;
		_censor_ct1=_censor;
		_outcome_yt1=.;
	end;
	if _outcome_yt1=1 then _censor_ct1=.;		
run;

proc sort data=_workdata;
	by &_ident _intnum;
run;

data _workdata;
	set _workdata;
	by &_ident _intnum;
	if last.&_ident and _intnum>_intmax then delete;
run;

* delete patients censored in the first interval;

data _workdata;
	set _workdata;
	by &_ident _intnum;
	if first.&_ident and last.&_ident and _censor = 1 then delete;
run;

* roll up censoring values for censored patients;

proc sort data=_workdata;
	by &_ident descending _intnum;
run;

data _workdata;
	set _workdata;
	by &_ident;
        _censor_yt1_rev=lag1(_censor);
        _censor_ct1 = _censor_yt1_rev;
        drop _censor_yt1_rev;
run;

proc sort data=_workdata;
	by &_ident _intnum;
run;

data _workdata;
	set _workdata;
	by &_ident _intnum;
	if _censor = 1 and _intnum=_intmax then delete;
run;

* clean up data for output;

data _workdata;
	set _workdata;
	_censor = _censor_ct1;
	_outcome = _outcome_yt1;
	if _censor = 1 then _outcome = .;
	if _outcome = 1 then _censor = .;
        drop _censor_ct1 _outcome_yt1;
run;

%mend;
