/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/



/************* MSM Macro - Imputation method 1: Last Value Carried Forward ***********************
********************************** _imput_lvcf.sas 8 *********************************************
 macro to assign values to missing values by using the last known value
 	
  Input:											
	_inlvcf = input dataset containing data with variables that need imputing
	_varname = variable containing value of the confounder,exposure,etc.
	_vardate = date for variable
	_ident= unit identifier for each patient	
	
 Output dataset: dataset with _varname imputed, or carried forward.
 
 7/10/2012 - create macro for handling lab and comorbidity _confounders first.	
 7/24/2013 - comment out _missing_imputed - not sure why included and test to see if causing
                imputation of '9' on 1 missing record in spironolactone dataset	
 9/20/2013 - remove "imputed" variable for behavior 1 - decision made that behavior 1 should 
 				never be missing since missing = 0
 			- made this imputation method impute mean/mode for missing in first row of data
11/18/2013 - reorganize again to incorporate changes in behaviors and imputation methods:
				behavior 1 = no missing. if missing considered as 0 if in first interval, up to 
							first 1 value.
							impute all values after first "1" value as 1. keep _imp=0
				behavior 2 = no missing. this is drug on/off periods. thus _imp=0
				behavior 3 = baseline values. If missing at baseline use mean/mode/missing value 
							methodes below to fill in (summarize across baseline values): _imp=1 
							if baseline is missing, else _imput = 0
				behavior 4 = allows for missing and imputation. 
                behavior 5 = allows for missing, no last value carry forword imputation. 
				
				imput=1: impute with default (0/unknown) for any missing at baseline
				imput=2: impute with mean of all baseline values in cohort as baseline value
				imput=3: impute with mode of all baseline values in cohort as baseline value
				imput=4: impute with median for all baseline values in cohort as baseline value
*************************************************************************************************/
options mergenoby=warn minoperator mindelimiter=',';
%macro _impute_lvcf (_inlvcf,_ident,_varname,_vardate);


%if &&&_cbehavior&_i = 1 %then %do;	
  %if &&&_varfmt&_i = N %then %do;

  proc sort data=&_inlvcf; 
	by &_ident _intnum;
  run;
  
  data _workdata;
	set &_inlvcf;
	by &_ident;
	retain _vars _varsdt;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname=. then do;
			_vars=0;
			_varsdt=.;
		end;

		else if &_varname=0 then do;
			_vars=&_varname;
			_varsdt=.;
		end;
		else if &_varname=1 and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname=1 and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
	end;
	else do;
		if &_varname=1 then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname ne 1 and _vars=1 then do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
		else if &_varname ne 1 and _vars ne 1 then do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
		else do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
	end;
  run;
  %end;
  
  %else %if &&&_varfmt&_i=C %then %do;
  
  proc sort data=&_inlvcf; 
	by &_ident _intnum;
  run;

  %let _varslength = 0;
  
  data _null_;
   	set &_inlvcf;
   	_varstemp = lengthc(&_varname);
     	call symput("_varstemp",trim(left(_varstemp)));
  run;  
  
  %let _varslength = %eval(&_varstemp);
 
    %put _varslength=&_varslength;
    %put _varstemp=&_varstemp;

  data _workdata;
  	length _vars $ &_varslength;
	set &_inlvcf;
	by &_ident;
	retain _vars _varsdt;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname='' then do;
			_vars='0';
			_varsdt=.;
		end;

		else if &_varname='0' and &_vardate=. then do; * nonsymmetric with numeric code;
			_vars=&_varname;
			_varsdt=.;
		end;
		else if &_varname='1' and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname='1' and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
	end;
	else do;
		if &_varname='1' then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname ne '1' and _vars='1' then do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
		else if &_varname ne '1' and _vars ne '1' then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
	end;
  run;
  %end;
%end;

%else %if &&&_cbehavior&_i = 4 or &&&_cbehavior&_i = 5 %then %do;	
 %if &&&_varfmt&_i=N %then %do;
 
  %if &&&_impute&_i=2 %then %do;	**get mean as impute method for first row missing value;
  	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;
  		if first.&_ident;
  	run;
  	
  	proc means data=baseline mean noprint;
  		var &_varname;
  		output out=_means mean=_default;
  	run;
  	data _means;
  		set _means;
  		keep _default;
  	run;

  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _means;
  	run;

  	proc datasets library=work nolist;
  		delete _means baseline;
  	quit;
  	run;
   %end;
   
   %else %if &&&_impute&_i=3 %then %do;		 **use mode as impute method for first row missing value; 
   	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;  		
  		if first.&_ident;
  	run;
	proc freq data=baseline noprint;
  		table &_varname / out=_freq_var missing;
  	run;
  	data _freq_var;
  		set _freq_var;
  		if &_varname='' then delete;
  	run;
  	proc sort data=_freq_var (rename=(&_varname=_default));
  		by descending count;
	run;
  	data _freq_var;
  		set _freq_var(obs=1);
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _freq_var;
  	run;
  	proc datasets library=work nolist;
  		delete _freq_var baseline;
  	quit;
  	run;
   %end;
      
   %else %if &&&_impute&_i=4 %then %do;		 **use median as impute method for first row missing value; 
   	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;
  		if first.&_ident;
  	run;
  	
  	proc means data=baseline median noprint;
  		var &_varname;
  		output out=_median median=_default;
  	run;
  	data _median;
  		set _median;
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _median;
  	run;
  	

  	proc datasets library=work nolist;
  		delete _means baseline;
  	quit;
  	run;
   %end;
   
   %else %if &&&_impute&_i=1 %then %do;
   	data default;
   		set &_inlvcf (obs=1);
   		%if &&&_varfmt&_i=C %then %do;
   		_default="&&&_impvalue&_i";
		%end;

       %if &&&_varfmt&_i=N %then %do;
   		_default=&&&_impvalue&_i;
		%end;
   		keep _default;
   	run;
   	
   	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set default;
  	run;
  	proc datasets library=work nolist;
  		delete default;
  	quit;
  	run;
   %end;  
   
   
  	proc sort data=_workdata; 
		by &_ident _intnum;
  	run;
  
  data _workdata;
	set _workdata;
	by &_ident;
	retain _vars _varsdt;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname=. then do;
			_vars=_default;
			_varsdt=_intstart;
			_imputed=1;
		end;
		else if &_varname ne . and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
		else if &_varname ne . and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
	end;

	else do;
  		if &_varname ne . then do;
     		_vars=&_varname;
  			_varsdt=&_vardate;
  			_imputed=0;
  		end;
  		else do;
		   %if &&&_cbehavior&_i = 5  %then %do;
		   _vars=_default;
           _varsdt =.;
           _imputed=0;
		   %end;
		   %else %do;
  			_vars=_vars;
  			_varsdt=_varsdt;
  			_imputed=1;
		   %end;
  		end;
  	end;
  run;
  %end;
  
 %else %if &&&_varfmt&_i=C %then %do;
  %if &&&_impute&_i=2 %then %do;
  	%put "error - Cannot impute based on mean. Variable is character based";
  	%goto exit_impute_lvcf;
  %end;

  %else %if &&&_impute&_i=4 %then %do;
  	%put "error - Cannot impute based on median. Variable is character based";
  	%goto exit_impute_lvcf;
  %end;

  %else %if &&&_impute&_i=3 %then %do;		**get mode (top freq) as default impute method for first row missing value;
  	proc freq data=&_inlvcf noprint;
  		table &_varname / out=_freq_var missing;
  	run;
  	data _freq_var;
	  	set _freq_var;
  		if &_varname='' then delete;
  	run;
  	proc sort data=_freq_var (rename=(&_varname=_default));
  		by descending count;
	run;
  	data _freq_var;
  		set _freq_var(obs=1);
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _freq_var;
  	run; 

 	proc datasets library=work nolist;
  		delete _freq_var;
  	quit;
  	run;
  %end;
  
  %else %if &&&_impute&_i=1  %then %do;		** use default as missing fill-in for first row missing values;  
  	data default;
   		set &_inlvcf (obs=1);
		%if &&&_varfmt&_i=C %then %do;
   		_default="&&&_impvalue&_i";
		%end;

       %if &&&_varfmt&_i=N %then %do;
   		_default=&&&_impvalue&_i;
		%end;
   		keep _default;
   	run;
   	
   	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set default;
  	run;
  	proc datasets library=work nolist;
  		delete default;
  	quit;
  	run;
   %end;  
   
  proc sort data=_workdata; 
	by &_ident _intnum;
  run;
  
  %let _varslength = 0;
  
  data _null_;
   	set _workdata;
   	_varstemp = lengthc(&_varname);
     	call symput("_varstemp",trim(left(_varstemp)));
  run;  
  
  %let _varslength = %eval(&_varstemp);
 
    %put _varslength=&_varslength;
    %put _varstemp=&_varstemp;

  %if &&&_cbehavior&_i = 5 %then %do;
  %let _varslength=20;
  %end;

  data _workdata;
  	length _vars $ &_varslength;
	set _workdata;
	by &_ident;
	retain _vars _varsdt;
	
  	if first.&_ident and _intnum=0 then do;
		if &_varname='' then do;
			_vars=_default;
			_varsdt=_intstart;
			_imputed=1;
		end;
		else if &_varname ne '' and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
		else if &_varname ne '' and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
	end;
	else do;
  		if &_varname ne '' then do;
  			_vars=&_varname;
  			_varsdt=&_vardate;
  			_imputed=0;
  		end;
  		else do;
		   %if &&&_cbehavior&_i = 5  %then %do;
		   _vars=_default;
           _varsdt =.;
           _imputed=0;
		   %end;
		   %else %do;
  			_vars=_vars;
  			_varsdt=_varsdt;
  			_imputed=1;
		   %end;
  		end;
  	end;
  run;
 %end;
%end;

** behaviore 2 - dates with no missing values. eg drug use;
 %if &&&_cbehavior&_i = 2 %then %do;
  %if &&&_varfmt&_i = N %then %do;

  proc sort data=&_inlvcf; 
	by &_ident _intnum;
  run;
  
  data _workdata;
	set &_inlvcf;
	by &_ident;
	retain _vars _varsdt;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname=. then do; 
			_vars=0;
			_varsdt=.;
		end;

		else if &_varname ne . then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname ne . and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname ne . and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
	end;
	else do;
		if &_varname ne . then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname=. then do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
	end;
	_imputed=0;
  run;
  %end;
  
  %else %if &&&_varfmt&_i=C %then %do;
  proc sort data=&_inlvcf; 
	by &_ident _intnum;
  run;
  
  %let _varslength = 0;
  
  data _null_;
   	set &_inlvcf;
   	_varstemp = lengthc(&_varname);
     	call symput("_varstemp",trim(left(_varstemp)));
  run;  
  
   %let _varslength = %eval(&_varstemp);
 
    %put _varslength=&_varslength;
    %put _varstemp=&_varstemp;
    
  data _workdata;
  	length _vars $ &_varslength;
	set &_inlvcf;
	by &_ident;
	retain _vars _varsdt;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname='' then do;
			_vars='0';
			_varsdt=.;
		end;

		else if &_varname ne '' and &_vardate=. then do;
			_vars=&_varname;
			_varsdt=.;
		end;
		else if &_varname ne '' and &_vardate>_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname ne '' and &_vardate<=_intstart then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
	end;
	else do;
		if &_varname ne '' then do;
			_vars=&_varname;
			_varsdt=&_vardate;
		end;
		else if &_varname='' then do;
			_vars=_vars;
			_varsdt=_varsdt;
		end;
	end;
	_imputed=0;
  run;  
  %end;
%end;

** behavior 3 - baseline values, eg. race or gender;
%if &&&_cbehavior&_i = 3 %then %do; **get mean as impute method for first row missing value;
 %if &&&_varfmt&_i=N %then %do;
  %if &&&_impute&_i=2 %then %do;
  	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;
  		if first.&_ident;
  	run;
  	
  	proc means data=baseline mean noprint;
  		var &_varname;
  		output out=_means mean=_default;
  	run;
  	data _means;
  		set _means;
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _means;
  	run;

  	proc datasets library=work nolist;
  		delete _means baseline;
  	quit;
  	run;
   %end;
   
   %else %if &&&_impute&_i=3 %then %do;  **use mode as impute method for first row missing value; 
   	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;
  		if first.&_ident;
  	run;
	proc freq data=baseline noprint;
  		table &_varname / out=_freq_var missing;
  	run;
  	data _freq_var;
  		set _freq_var;
  		if &_varname='' then delete;
  	run;
  	proc sort data=_freq_var (rename=(&_varname=_default));
  		by descending count;
	run;
  	data _freq_var;
  		set _freq_var(obs=1);
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _freq_var;
  	run;
  	proc datasets library=work nolist;
  		delete _freq_var baseline;
  	quit;
  	run;
   %end;
      
   %else %if &&&_impute&_i=4 %then %do;	 **use median as impute method for first row missing value; 
   	proc sort data=&_inlvcf;
  		by &_ident _intnum;
  	run;
  	
  	data baseline;
  		set &_inlvcf;
  		by &_ident;
  		if first.&_ident;
  	run;
  	
  	proc means data=baseline median noprint;
  		var &_varname;
  		output out=_median median=_default;
  	run;
  	data _median;
  		set _median;
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _median;
  	run;
  	

  	proc datasets library=work nolist;
  		delete _means baseline;
  	quit;
  	run;
   %end;
   
   %else %if &&&_impute&_i=1 %then %do;
   	data default;
   		set &_inlvcf (obs=1);
   		
   		keep _default;
   	run;
   	
   	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set default;
  	run;
  	proc datasets library=work nolist;
  		delete default;
  	quit;
  	run;
   %end;  
   
   
  	proc sort data=_workdata; 
		by &_ident _intnum;
  	run;
  
  data _workdata;
	set _workdata;
	by &_ident;
	retain _vars _varsdt _imputed;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname=. then do;
			_vars=_default;
			_varsdt=_intstart;
			_imputed=1;
		end;
		else if &_varname ne . then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
	end;
	else do;
  		if &_varname ne . then do;
	  		_vars=&_varname;
  			_varsdt=&_vardate;
  			_imputed=0;
  		end;
  		else do;
  			_vars=_vars;
  			_varsdt=_varsdt;
  			_imputed=_imputed;
  		end;
  	end;
  run;
 %end;
  
 %else %if &&&_varfmt&_i=C %then %do;
  %if &&&_impute&_i=2 %then %do;
  	%put "error - Cannot impute based on median/mean. Variable is character based";
  	%goto exit_impute_lvcf;
  %end;

  %else %if &&&_impute&_i=4  %then %do;
  	%put "error - Cannot impute based on median/mean. Variable is character based";
  	%goto exit_impute_lvcf;
  %end;

/*
  %if &&&_impute&_i in (2,4) %then %do;
  	%put "error - Cannot impute based on median/mean. Variable is character based";
  	%goto exit_inpute_lvcf;
  %end;
*/

  
  %else %if &&&_impute&_i=3 %then %do;	**get mode (top freq) as default impute method for first row missing value;
   	proc freq data=&_inlvcf noprint;
  		table &_varname / out=_freq_var missing;
  	run;
  	data _freq_var;
	  	set _freq_var;
  		if &_varname='' then delete;
  	run;
  	proc sort data=_freq_var (rename=(&_varname=_default));
  		by descending count;
	run;
  	data _freq_var;
  		set _freq_var(obs=1);
  		keep _default;
  	run;
  	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set _freq_var;
  	run; 

 	proc datasets library=work nolist;
  		delete _freq_var;
  	quit;
  	run;
  %end;
  
 %else %if &&&_impute&_i=1 %then %do;	** use default as missing fill-in for first row missing values;  
  	data default;
  	     length _default $20;
   		set &_inlvcf (obs=1);
   		%if &&&_varfmt&_i=C %then %do;
   		_default="&&&_impvalue&_i";
		%end;

        %if &&&_varfmt&_i=N %then %do;
   		_default=&&&_impvalue&_i;
		%end;
   		keep _default;
   	run;
   	
   	data _workdata;
  		set &_inlvcf;
  		if _n_=1 then set default;
  	run;
  	proc datasets library=work nolist;
  		delete default;
  	quit;
  	run;
 %end;  
   
  proc sort data=_workdata; 
	by &_ident _intnum;
  run;
 
   %let _varslength = 0;
  
  data _null_;
   	set _workdata;
   	_varstemp = lengthc(&_varname);
     	call symput("_varstemp",trim(left(_varstemp)));
  run;  
  
   %let _varslength = %eval(&_varstemp);
 
    %put _varslength=&_varslength;
    %put _varstemp=&_varstemp;
    
   data _workdata;
   	length _vars $ &_varslength;
	set _workdata;
	by &_ident;
	retain _vars _varsdt _imputed;
	
	if first.&_ident and _intnum=0 then do;
		if &_varname='' then do;
			_vars=_default;
			_varsdt=_intstart;
			_imputed=1;
		end;
		else if &_varname ne '' then do;
			_vars=&_varname;
			_varsdt=&_vardate;
			_imputed=0;
		end;
	end;
	else do;
  		if &_varname ne '' then do;
	  		_vars=&_varname;
  			_varsdt=&_vardate;
  			_imputed=0;
  		end;
  		else do;
  			_vars=_vars;
  			_varsdt=_varsdt;
  			_imputed=_imputed;
  		end;
  	end;
  run;
 %end;
%end;

  
  

data _workdata;
	set _workdata (drop=&_varname &_vardate);

  /*%if &&&_varfmt&_i=N %then %do;
	if _intnum>_intmax then do;
		_vars=.;
		_varsdt=.;
		_imputed=.;
	end;
  %end;	
  %else %if &&&_varfmt&_i=C %then %do;
	if _intnum>_intmax then do;
		_vars='';
		_varsdt=.;
		_imputed=.;
	end;
  %end;
  */	
	&_varname=_vars;
	&_vardate=_varsdt;
	_imp&_varname=_imputed;
	
	_default = .;

	format &_vardate _varsdt mmddyy10.;

	drop _vars _varsdt _imputed _default;
run;

%if &&&_cbehavior&_i in (1,2,5) %then %do;	
 data _workdata;
  set _workdata;
  drop _imp&_varname &_vardate;
 run;
%end;

*** for testing puprposes - exporting datasets;
  %if  &unittest= imput_lvcf %then %do;

	%let rundate = %SYSFUNC(today(),yymmddn8.); 
	data &_outdata;
		set _workdata;
	run;
	%goto exit_impute_lvcf;
  %end;
*** end of testing;



%exit_impute_lvcf: %mend _impute_lvcf;
