/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/

    
%macro _msm_auto_basetable(indata,varfile,idvar,byvar,byvarcat,odir,ofile);

* [BLOCK0] define utility macro to count words in &list;
%macro _wordcount_msm(list);
 %global _count;
 %let _count=0;
 %do %while(%qscan(&list,&_count+1,%str( )) ne %str());
  %let _count = %eval(&_count+1);
 %end;
%mend _wordcount_msm;

* [BLOCK0 END] define utility macro to count words in &list;

* [BLOCK1] read in and clean baseline dataset;
 data inputds;
  set &indata;
 run;

 proc sort data=inputds;
  by &idvar descending _exposure;
 run;

 data inputds;
  set inputds;
  by &idvar;
  retain _exposure_ever_never;
  if first.&idvar then do;
   if %sysfunc(ifc(&_exp_multi_level1=0, _exposure = 1, _exposure ne 'Not exposed')) then _exposure_ever_never = 1;
   else if _%sysfunc(ifc(&_exp_multi_level1=0, exposure = 0, _exposure = 'Not exposed')) then _exposure_ever_never = 0;
  end; 
  else _exposure_ever_never = _exposure_ever_never;
 run;

 proc sort data=inputds;
  by &idvar _intnum;
 run;

 data inputds;
  set inputds;
  if _intnum = 0;
 run;

* [BLOCK1 END] read in and clean baseline dataset;

* [BLOCK2] sample size calculation of population categories;
 %if %length(&byvar) >0 %then %do;

  proc sort data=inputds out=inputds;
   by &byvar;
  run;
   
 /* create dummy variable to allow for use of PROC SUMMARY */
  data inputds;
   set inputds;
   temp=1;
  run;

 /* use PROC SUMMARY to obtain population count across by variable categories */
  proc summary data=inputds;
   %if %length(&byvar)>0 %then %do; 
    class &byvar; 
   %end;
   var temp;
   output out=out_k N=Nobs;
  run;

 /* classify population using by variable categories and create variable for sample size (N) row */
  data out_k;
   length byvargrp $100 
          N        $60;
   set out_k;

   if &byvar='' then byvargrp='Overall';
   else byvargrp = &byvar;
   if nobs>=10000 then N= "(N="||trim(left(PUT(nobs,COMMA12.0)))||")";
   else N= "(N="||trim(left(nobs))||")";
  run;

  proc transpose data=out_k out=out_kt;
   var N;
   id byvargrp;
  run;

  data out_kt;
   set out_kt;
   drop _name_;
  run;

  data out_Nobs;
   length _Name_  $5
	  varname $100
	  ord0    $100
	  label0  $100
	  label1  $100
	  vartype $100;
   set out_kt;
   varname='N';
   label0='N';
   order0=0;
   vartype='N';
  run;
 %end;
 %else %do;
  ods listing close;
  ods output Attributes=out_k;
  proc contents data=inputds;
  run;
  
  ods listing;
  data out_Nobs;
   length Overall $60 
   varname $100 
   ord0 $100 
   _name_ $5
   label0 $100
   label1 $100
   vartype $100;
   set out_k;
   if label2='Observations';
   if nvalue2>=10000 then Overall= "(N="||trim(left(PUT(nvalue2,COMMA12.0)))||")";
   else Overall= "(N="||trim(left(nvalue2))||")";
   order0=0;
   varname='N';
   label1='N';
   label0='';
   vartype='N';
   keep label0 label1 Overall varname order0 ord0 vartype _name_;
  run;
 %end;
* [BLOCK2 END] sample size calculation of population categories;
  

* [BLOCK3] calculate descriptive statistics for variables specified in variables file;

 /* clean and read in variables file */
 data varlist;
  set &varfile;
  if varname='' then delete;
  if remove = 1 then delete;
 run;

 data varlist;
  set varlist;
  order0=_N_;
 run;

 %global _nobs_table;
 %_nobs_msm(varlist);
 %let _nobs_table = %eval(&_nobs);

 /* calculation of descriptive statistics - loop runs once for every specified variable */

 %do i=1 %to &_nobs_table;
  data _null_;
   set varlist;
   if _n_=&i;
   call symput("voi",varname);
   call symput("vtp",type);
   call symput("vcat",cat);
   call symput("vlabel",vlabel);
   call symput("clabel",clabel);
   call symput("roundnum",roundnum);
   call symput("stattype",stattype);
   call symput("vartype",vartype);
   call symput("order0",order0);
  run;
  
  %put i=&i;
  %put voi=&voi;
  %put vtp=&vtp;
  %put vcat=&vcat;
  %put vlabel=&vlabel;
  %put clabel=&clabel;
  %put roundnum=&roundnum;
  %put stattype=&stattype;
  %put order0=&order0;
  %put vartype=&vartype;
 
  %let voi=%cmpres(&voi);
  %let vtp=%cmpres(&vtp);

* [BLOCK3.1] descriptive statistics - numeric or numeric/categorical variables;
  %if &vtp=N or &vtp=B %then %do;
 
 /* code invoked if by variable specified for baseline table - defining number of by variable groups */
   %if %length(&byvar)>0 %then %do;
    ods listing close;
    ods output onewayfreqs=variables;
    proc freq data=inputds;
     tables &byvar;
    run;
    
    ods listing;

    data variables;
     set variables;
     by table;
     if last.table;
     numobs=_n_;
    run;

    data _null_;
     set variables;
     if _n_=1 then call symput("numbygrp",numobs);
    run;
   %end;
	
 /* code invoked if no by variable specified for baseline table - defining number of by variable groups */
   %else %do;
    %let numbygrp=0;
   %end;

 /* process stattype data for numeric variables */

   %let mn=%scan(&stattype,1,*);
   %let md=%scan(&stattype,2,*);
   %let sm=%scan(&stattype,3,*);
   
   %put mn=&mn;
   %put md=&md;
   %put sm=&sm;

 /* calculate continuous descriptive statistics using PROC SUMMARY */
   
   proc summary data=inputds;
    %if %length(&byvar) >0 %then %do; 
     class &byvar/ missing;
    %end;
    var &voi;
    output out=out_k 
           N=NNobs 
           %if &mn=1 %then %do; 
            mean=mean stddev=std 
           %end;
	   %if &md=1 %then %do; 
	    p50=median 
	    q1=iqr1 
	    q3=iqr3 
	    min=min 
	    max=max 
	   %end;
           nmiss=miss 
           PROBT=t_probt 
           %if &sm=1 %then %do; 
            sum=sum 
           %end;;
   run;

 /* assemble continuous descriptive statistics for printing in output file  */

   data out_k;
     length byvargrp $100;
    set out_k;
    %if %length(&byvar) >0 %then %do;
     if &byvar='' then byvargrp='Overall';
     else byvargrp = &byvar;
    %end;
    %else %do;
     byvargrp='Overall';
    %end;
    %if &mn=1 %then %do;
     mnstd=trim(left(put(round(mean,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     ' ('||trim(left(put(round(std,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||')';
    %end;
    %if &md=1 %then %do;
     mdiqr=trim(left(put(round(median,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     ' ('||trim(left(put(round(iqr1,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     '-'||trim(left(put(round(iqr3,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||')';
     
     range=trim(left(put(round(min,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     '-'||trim(left(put(round(max,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))));
    %end;
    %if &sm=1 %then %do;
     sumf=trim(left(put(round(sum,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))));
    %end;
    if miss>=10000 then nmiss= trim(left(PUT(miss,COMMA12.0)))||' ('||trim(left(put(round(miss*100/(nnobs+miss),0.1),12.1)))||')';
    else nmiss= trim(left(miss))||' ('||trim(left(put(round(miss*100/(nnobs+miss),0.1),12.1)))||')';
    keep byvargrp 
    %if &mn=1 %then %do; 
     mnstd 
    %end;
    %if &md=1 %then %do; 
     mdiqr range 
    %end;
    nmiss
    %if &sm=1 %then %do; 
     sumf 
    %end;;
   run;

 /* transpose descriptive statistics dataset */

   proc transpose data=out_k out=out_kt;
    var 
     %if &mn=1 %then %do; 
      mnstd 
     %end;
     %if &md=1 %then %do; 
      mdiqr range 
     %end;
     nmiss
     %if &sm=1 %then %do; 
      sumf 
     %end;;
    id byvargrp;
   run;
 
 /* combine descriptive statistics with category sample size results into output dataset */

   data out_t&i;
    length varname $100
           label0  $100
           label1  $100
           ord0    $100
           vartype $100;
    set out_kt;
    if _name_='mnstd' then ord0='1';
    else if _name_='mdiqr' then ord0='2';
    else if _name_='range' then ord0='3';
    else if _name_='nmiss' then ord0='4';
    else if _name_='sumf' then ord0='5';
    varname="&voi";
    order0=&order0;
    vartype="%trim(&vartype)";
    label0="%trim(%nrstr(&vlabel))";
    label1='';
   run;
   
   proc sort data=out_t&i;
    by ord0;
   run;
   
   proc append base=out_nobs data=out_t&i force;
   run;

* [BLOCK3.2] additional descriptive statistics - numeric/categorical variables;
 /* creating labels for categorical variable values */

   %if &vtp=B %then %do;
    %_wordcount_msm(&vcat);
    
    data inputds;
     set inputds;
     %let j = 1;
     %if &j=1 %then %do;
      if &voi ne . and &voi<%scan(&vcat,&j,' ') then &voi._cat="00: %scan(&clabel,&j,*)";
     %end;
     
     %do j=2 %to (&_count);
      else if %scan(&vcat,&j-1,' ')<=&voi<%scan(&vcat,&j,' ') then &voi._cat=
       %if &j<=10 %then %do; 
        "0%eval(&j-1): %scan(&clabel,&j,*)";
       %end;
       %else %do;
        "%eval(&j-1): %scan(&clabel,&j,*)";
       %end;
     %end;
     
     else if &voi>=%scan(&vcat,&_count,' ') then &voi._cat=
      %if &_count<=10 %then %do;
       "0&_count: %scan(&clabel,(&_count+1),*)";
      %end;
      %else %do;
       "&_count: %scan(&clabel,(&_count+1),*)";
      %end;
     else if &voi=. then &voi._cat="Unknown";
    run;

 /* calculate statistical significance tests for categorical non-binary variables using PROC FREQ */
    
    ods listing close;
    ods output 
    %if %length(&byvar)>0 %then %do; 
     CrossTabFreqs=output1
    %end;
    %else %do; 
     onewayfreqs=output1 
    %end;;
    
    proc freq data=inputds;
     tables &voi._cat 
     %if %length(&byvar)>0 %then %do;
      * &byvar/chisq
     %end;;
    run;
    
    ods listing;

    data output1;
     length byvargrp $100 
            ord0     $100;
     set output1;
     %if %length(&byvar)>0 %then %do;
      if _type_ in ('11','10');
      if _type_='10' then byvargrp='Overall';
      else byvargrp = &byvar;
     %end;
     %else %do;
      byvargrp='Overall';
     %end;
     if frequency>=10000 then freq=trim(left(PUT(frequency,COMMA12.0)));
     else freq=trim(left(frequency));
     %if %length(&byvar)>0 %then %do;
      if _type_='11' then Meas=trim(left(freq))||' ('||trim(left(put(round(colpercent,0.1),5.1)))||')';
      else Meas=trim(left(freq))||' ('||trim(left(put(round(percent,0.1),5.1)))||')';
     %end;
     %else %do;
      Meas=trim(left(freq))||' ('||trim(left(put(round(percent,0.1),5.1)))||')';
     %end;
     ord0= &voi._cat;
     keep ord0 byvargrp meas;
    run;
    
    proc sort data=output1;
     by ord0;
    run;
      
 /* transpose frequencies dataset for further processing */   
    
    proc transpose data=output1 out=ot1;
     by ord0;
     var meas;
     id byvargrp;
    run;
    
    data ot1;
     length varname $100 
            label0  $100 
            label1  $100
            vartype $100;
     set ot1;
     varname="&voi._cat";
     order0=&order0+0.5;

     vartype="%trim(&vartype)";

     label0="%trim(%nrstr(&vlabel))";
     label1=ord0;

    run;
          
    proc append base=out_nobs data=ot1 force;
    run;
   %end;
  %end;
* [BLOCK3.2 END] additional descriptive statistics - numeric/categorical variables;  
* [BLOCK3.1 END] descriptive statistics - numeric or numeric/categorical variables;


  %put type is &vtp;

* [BLOCK3.3] descriptive statistics - categorical variables;

  %if &vtp=C %then %do;
   %put &voi;
   %put &vcat;
   %_wordcount_msm(&vcat);
   %let vcat_cnt=&_count;

 /* calculate statistical significance tests for categorical non-binary variables using PROC FREQ */
  
   ods output 
   %if %length(&byvar) >0 %then %do; 
    CrossTabFreqs=output1
   %end;
   %else %do;
    onewayfreqs=output1 
   %end;;
 	
   proc freq data=inputds;
    %if %length(&byvar) >0 %then %do;
     tables &voi*&byvar/chisq;
    %end;
    %else %do;
     tables &voi;
    %end;
   run;

 /* create frequency result variables for printing to baseline table */
  
   data output1;
    length byvargrp $100 
           ord0     $100;
    set output1;
    %if %length(&byvar) >0 %then %do;
     if _type_ in ('11','10');
     if _type_='10' then byvargrp='Overall';
     else byvargrp = &byvar;
    %end;
    %else %do;
     byvargrp='Overall';
    %end;
    if frequency>=10000 then freq=trim(left(PUT(frequency,COMMA12.0)));
    else freq=trim(left(frequency));
    %if %length(&byvar) >0 %then %do;
     if _type_='11' then Meas=trim(left(freq))||' ('||trim(left(put(round(colpercent,0.1),5.1)))||')';
     else Meas=trim(left(freq))||' ('||trim(left(put(round(percent,0.1),5.1)))||')';
    %end;
    %else %do;
     if cumfrequency>=10000 then cfreq=trim(left(PUT(cumfrequency,COMMA12.0))); 
     else cfreq=trim(left(cumfrequency));
     fMeas=trim(left(freq))||' ('||trim(left(put(round(percent,0.1),5.1)))||')';
     cmeas=trim(left(cfreq))||' ('||trim(left(put(round(cumpercent,0.1),5.1)))||')';
     meas=(fmeas||'['||cmeas);

    %end;
    ord0 = &voi;
    keep ord0 byvargrp meas cmeas;
   run;
  
   %if &vcat_cnt=1 %then %do;
    %let vcat=%cmpres(&vcat);
    data output1;
     set output1;
     if trim(left(ord0)) = "&vcat";
    run;
   %end;
  
   proc sort data=output1;
    by ord0;
   run;

 /* transpose frequencies dataset for further processing */   
  
   proc transpose data=output1 out=ot1;
    by ord0;
    var meas;
    id byvargrp;
   run;
  
 /* add chi-square results and d-values to overall output dataset */  
   data ot1;
    length varname $100
           label0  $100
           label1  $100;
    set ot1;
    varname="&voi";
    order0=&order0;
    %do x=1 %to &vcat_cnt;
     %if &x > 1 %then %do; 
      else 
     %end;
     if trim(left(ord0))="%scan(&vcat,&x,' ')" then label1="%scan(&clabel,&x,*)";
    %end;
    label0="%trim(%nrstr(&vlabel))";
    tt=_N_;
   run;

    data ot1;
     length vartype $100;
     set ot1;
     vartype="%trim(&vartype)";
    run;
  
   proc append base=out_nobs data=ot1 force;
   run;
  %end;  
 %end;
* [BLOCK3.3 END] descriptive statistics - categorical variables;


* [BLOCK3.4] descriptive statistics - histograms for continuous variables;

 ods results off;
 ODS LISTING CLOSE;
 %if &sysscp = WIN %then %do;
  ods rtf file="&odir.\&ofile._histograms_%sysfunc(date(),yymmddn8.).rtf";
 %end;
 %else %do;
  ods rtf file="&odir./&ofile._histograms_%sysfunc(date(),yymmddn8.).rtf";
 %end;
 ODS OUTPUT CLEAR;
 
 /* calculation of descriptive statistics - loop runs once for every specified variable */

 %do i=1 %to &_nobs_table;
  data _null_;
   set varlist;
   if _n_=&i;
   call symput("voi",varname);
   call symput("vtp",type);
   call symput("vcat",cat);
   call symput("vlabel",vlabel);
   call symput("clabel",clabel);
   call symput("roundnum",roundnum);
   call symput("stattype",stattype);
   call symput("vartype",vartype);
   call symput("order0",order0);
  run;
  
  %put i=&i;
  %put voi=&voi;
  %put vtp=&vtp;
  %put vcat=&vcat;
  %put vlabel=&vlabel;
  %put clabel=&clabel;
  %put roundnum=&roundnum;
  %put stattype=&stattype;
  %put order0=&order0;
  %put vartype=&vartype;
 
  %let voi=%cmpres(&voi);
  %let vtp=%cmpres(&vtp);
  
  %if &vtp=N or &vtp=B %then %do;

 /* calculate continuous descriptive statistics */

   %if &varfile=mod3files %then %do; 
    title "Figure &i.a: Distribution of time to 1st non-reference exposure by &byvar ";
   %end;

   %else %if &varfile=mod2files %then %do; 
   title "Figure &i.a: Distribution of time to EOF by &byvar ";
   %end;

   %else %do; 
    title "Figure &i.a: Distribution of baseline &voi";
   %end;   
    
   footnote "MSM interval zero statistics - not identical to baseline statistics";

   proc univariate data=inputds noprint;
    var &voi;
    histogram &voi;
   run;

  %if %length(&byvar) >0 %then %do;    
   proc univariate data=inputds noprint;
    class &byvar (missing);
    var &voi;
    histogram &voi;
   run;
  %end;
   
  %end;
 %end;
 ods rtf close;
 ods results on;

* [BLOCK3.4 END] descriptive statistics - histograms for continuous variables;

* [BLOCK3 END] calculate descriptive statistics for variables specified in variables file; 
 
* [BLOCK4] set parameters for output file and print results to RTF;  

/* define output title and footnote */

 %if &varfile=mod3files %then %do; 
  title "Table 1: Distribution of time to 1st non-reference exposure";
 %end;
 %else %if &varfile=mod2files %then %do; 
  title "Table 1: Distribution of time to EOF by &byvar";
 %end;
 %else %do; 
  title "Table 1: Distribution of baseline covariates and reasons for EOF";
 %end;   

 footnote "MSM interval zero statistics - not identical to baseline statistics";

 /* create labels for printing to output file */
  
 data out_nobs;
  set out_nobs;
  ord0=trim(left(ord0));
  if _name_='mnstd' then label1='Mean (SD)';
  else if _name_='mdiqr' then label1='Median (interquartile range)';
  else if _name_='range' then label1='Range';
  else if _name_='nmiss' then label1='Missing, n (%)';
  else if _name_='sumf' then label1='Sum';

  %if %length(&byvar)=0 %then %do;
   cmeas = left(scan(overall,2,'['));  
  %end;
  
  overall_final = scan(overall,1,'[');
  drop overall;
 run;

 data out_nobs;
  set out_nobs;
  rename overall_final = overall;
 run;

 /* processing for main baseline table */
  /* count number of categories in table by variable, if specified */

 data temp;
  set out_nobs;
  drop label0 label1 varname _name_ ord0 order0 vartype overall;
 run;

 ods listing close;
 ods output variables=variables;
 proc contents data=temp;
 run;
 ods listing;
 
 %macro a;
  %global _nobs_output;
  %_nobs_msm(variables);
  %let _nobs_output = %eval(&_nobs);
  %put &_nobs_output;
  
  data _null_;
   set variables;
   %do i=1 %to &_nobs_output;
    %global var&i;
    if _n_ = &i then call symput("var&i",left(trim(variable)));
   %end;
  run;
 %mend a;

 %if %length(&byvar) >0 %then %do;
  %a;
 %end;

 /* define output template parameters */
 
 proc template;
  
  define style styles.Custom;
   parent = Styles.RTF;
   style fonts / 'docfont'=("Arial",8pt);
   style data / background=white font = ("arial",8pt);
   style cell / background=white font = ("arial",8pt);
   style header / background=white font = ("arial",8pt,bold) just=c;
  end;
  
  define table mytable;
   style={background=white
          foreground=black
          font=("arial",8pt)};
  
  column label0 label1 overall    
   %if %length(&byvar) >0 %then %do;
    %do i=1 %to &_nobs_output; 
     &&&var&i 
    %end;

   %end;
   %if %length(&byvar)=0 %then %do;
    cmeas
   %end;
   ;
   
  define column label0;
   blank_dups=on;
  end;
  	
  define column overall;
   just=c;
   header="Overall";
   just=l;
   cellstyle label1='Mean (SD)' as {just=c},
             label1='Median (interquartile range)' as {just=c},
             label1='Range' as {just=c},
             label1='Missing, n (%)' as {just=c},
             label1='Sumf' as {just=c},
             label0='N' as {just=c};
  end;

  %if %length(&byvar)=0 %then %do;
   define column cmeas;
    just=c;
    header="Cumulative";
    just=l;
    cellstyle label1='Mean (SD)' as {just=c},
              label1='Median (interquartile range)' as {just=c},
              label1='Range' as {just=c},
              label1='Missing, n (%)' as {just=c},
              label1='Sumf' as {just=c},
              label0='N' as {just=c};
   end;
  %end;

 /* format columns for categories in table by variable, if specified */
  
  %if %length(&byvar) >0 %then %do;
   %do i=1 %to &_nobs_output;
    define column &&&var&i;
     just=c;
     header="%scan(&byvarcat,&i,*)";
     just=l;
     cellstyle label1='Mean (SD)' as {just=c},
     label1='Median (interquartile range)' as {just=c},
     label1='Range' as {just=c},
     label1='Missing, n (%)' as {just=c},
     label1='Sumf' as {just=c},
     label0='N' as {just=c};
    end;
   %end;
  %end;
 end;
 run;

 /* apply formatting for missing values */
 
 data out_nobs;
  set out_nobs;
  by vartype notsorted;
  if last.vartype then do;
   output;
   label0='';
   label1='';
   overall='';
   %if %length(&byvar) >0 %then %do;
    %do i=1 %to &_nobs_output;
     &&&var&i='';
    %end;
   %end;
   output;
  end;
  else output;
  run;
  
  data out_nobs;
   set out_nobs;
   if ((trim(left(label0)) = '' and trim(left(label1)) ne '') or (trim(left(label0)) ne '' and trim(left(label1)) = '')) and overall='' then do;
    overall= '0 '||'(0.0)';
    %if %length(&byvar)>0 %then %do;
     %do i=1 %to &_nobs_output;
      &&&var&i='0 '||'(0.0)';
     %end;
    %end;
   end;
  run;

  data out_nobs;
   set out_nobs;
   if label0 = '' then cmeas = '';
  run;
 
 /* print results to RTF file */

 ods results off; 
 options orientation=landscape ls=150;
 %if &sysscp = WIN %then %do;
  ods rtf file="&odir.\&ofile._table_%sysfunc(date(),yymmddn8.).rtf" sasdate style=Custom ;
 %end; 
 %else %do;
  ods rtf file="&odir./&ofile._table_%sysfunc(date(),yymmddn8.).rtf" sasdate style=Custom ;
 %end;
 data _null_;
  set out_nobs;
  file print ods=(template='mytable');
  put _ods_;
 run;
 ods rtf close;
 ods results on;  

%mend _msm_auto_basetable;
