/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


%macro _msm_auto_basetable_multiexp(indata,varfile,idvar, byvar, byvarcat, odir, ofile);
 data inputds;
  set &INDATA;;
 run;

 proc sort data=inputds;
  by &idvar _exposure;
 run;

 data inputds;
  set inputds;
  by id _exposure;
  if first._exposure;
  temp=1;
 run;


proc summary data=inputds;
   class &byvar _exposure; 
   var temp;
   output out=out_k N=Nobs;
  run;

  data out_total;
   length byvargrp $100 
          N        $60;
   set out_k;
   where _exposure='';
   if &byvar='' then byvargrp='Overall';
   else byvargrp = &byvar;
   
   if nobs>=10000 then N= "(N="||trim(left(PUT(nobs,COMMA12.0)))||")";
   else N= "(N="||trim(left(nobs))||")";
  run;
             
   proc transpose data=out_total out=out_totalt;
   var N;
   id byvargrp ;
  run;

data out_Nobs; 
length 
       ord0    $1
      _exposure $20;
   set out_totalt;
   _exposure='N';
 run;
 
/* clean and read in variables file */
 data varlist;
  set &varfile;
  if varname='' then delete;
 run;

  data _null_;
   set varlist;
   call symput("voi",varname);
   call symput("roundnum",roundnum);
  run;
  

  %put voi=&voi;
  %put roundnum=&roundnum;
 
   proc summary data=inputds;
    class &byvar _exposure/ missing;
    var &voi;
    output out=out_k1 
           N=NNobs  
         
            mean=mean stddev=std 
      
	    p50=median 
	    q1=iqr1 
	    q3=iqr3 
	    min=min 
	    max=max 
	   
           nmiss=miss 
           PROBT=t_probt 
           
            sum=sum ;
        
   run;
   data out_k2;
     length byvargrp $100;
    set out_k1;
	where _exposure ne '' ;
     if &byvar='' then byvargrp='Overall';
     else byvargrp = &byvar;
    
   
     mnstd=trim(left(put(round(mean,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     ' ('||trim(left(put(round(std,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||')';
   
 
     mdiqr=trim(left(put(round(median,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     ' ('||trim(left(put(round(iqr1,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     '-'||trim(left(put(round(iqr3,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||')';
     
     range=trim(left(put(round(min,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))))||
     '-'||trim(left(put(round(max,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))));
  
   
     sumf=trim(left(put(round(sum,%trim(%left(&roundnum))),12.%trim(%left(%length(%substr(&roundnum,3)))))));
  
    if miss>=10000 then nmiss= trim(left(PUT(miss,COMMA12.0)))||' ('||trim(left(put(round(miss*100/(nnobs+miss),0.1),12.1)))||')';
    else nmiss= trim(left(miss))||' ('||trim(left(put(round(miss*100/(nnobs+miss),0.1),12.1)))||')';
    keep _exposure byvargrp  mnstd mdiqr range nmiss sumf ;
   run;

    /* transpose descriptive statistics dataset */
 proc sort data= out_k2;
 by _exposure;
 run;

   proc transpose data=out_k2 out=out_k2t;
    var  mnstd mdiqr range nmiss sumf; 
    by _exposure;
	id byvargrp;
   run;

    data out_exps;
    set out_k2t;
    if _name_='mnstd' then ord0='1';
    else if _name_='mdiqr' then ord0='2';
    else if _name_='range' then ord0='3';
    else if _name_='nmiss' then ord0='4';
    else if _name_='sumf' then ord0='5';
   run;

   proc sort data=out_exps;
    by _exposure ord0;
   run;
   
   proc append base=out_nobs data=out_exps force;
   run;
proc format;
value $label
'mnstd'="Mean (SD)"
'mdiqr'="Median (interquartile range)"
'range'='Range'
'nmiss'="Missing,n(%)"
'sumf'='Sum';
run;

ods listing close;
ods rtf file="&odir/&ofile._table_%sysfunc(date(),yymmddn8.).rtf"; 
options orientation=landscape ls=150;
title " Table: Distribution of time to First Interval by exposure and &byvar"; 
proc report data=out_nobs nowd;
column _exposure _name_ Overall AMI adminEnd death disenroll;
define _exposure/group;
define _name_ /'label' format = $label.; 
run;
ods rtf close;

*histograms for first inverval with exposus ; 

ods rtf file="&odir/&ofile._histograms_%sysfunc(date(),yymmddn8.).rtf"; 
title "Figure 1.a: Distribution of time to First Interval by exposure";
proc univariate data=inputds noprint;
    class  _exposure(missing);
    var &voi;
    histogram &voi;
run;
title;

title "Figure 2.a: Distribution of time to First Interval by exposure and &byvar";
proc univariate data=inputds noprint;
    class  &byvar _exposure(missing);
    var &voi;
    histogram &voi;
run;
title;
ods rtf close;
ods listing;

%mend;
