/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/



/*********** MSM Container Macro - Invokes MSM Dataset Construction Process *************
 Control program of MSM dataset creation
        - applies checks to macro arguments to ensure compatibility
	- invokes MSM series macros in order

Variables:
	_varsdata = file location and name of the input variables dataset

****************************************************************************************/
%macro _msmstructure(_varsdata, unittest);


%put 'MSMStructure data structuring macro - version 1.0.4';

* temp - fill in exposure behavior = 2 to make default for missing behavior in dashboard;
	data &_varsdata;
		set &_varsdata;
		if upcase(_type) = 'EXPOSURE' then _behavior = 2;
		if _remove = . then _remove = 0;
	run;

/* temp - output log to external file if running on PC SAS;
 %if &sysscp = WIN %then %do;
  data _log_dir;
   set &_varsdata;
   if upcase(_type) = 'OUTDATA';
   _dir = scan(_sas_data_name,1);
   call symput("_logdir",trim(left(_dir)));
  run;

  %let _logpath = %sysfunc(pathname(&_logdir));
  %put &_logpath;


  proc printto log="&_logpath\msmlog.log" print="&_logpath\msmlst.lst" new;
  run;
 %end;

** end of temp ;
*/
* read in model parameters from _varsdata file;

 data _cohort _outdata _studystart _studyend _eoftype _studyid _unitdays _harvardformat _baseline;
  set &_varsdata;
  if _remove ne 1;
  if upcase(_type) = 'COHORT' then output _cohort;
  else if upcase(_type) = 'OUTDATA' then output _outdata;
  else if upcase(_type) = 'STUDYSTART' then output _studystart;
  else if upcase(_type) = 'STUDYEND' then output _studyend;
  else if upcase(_type) = 'EOFTYPE' then output _eoftype;
  else if upcase(_type) = 'STUDYID' then output _studyid;
  else if upcase(_type) = 'UNITDAYS' then output _unitdays;
  else if upcase(_type) = 'HARVARDFORMAT' then output _harvardformat;
  else if upcase(_type) = 'COVARIATE' then output _baseline;
 run;

 %global _cohort;
 %let _cohort = 0;
 %global _outdata;
 %let _outdata = 0;
 %global _studystart;
 %let _studystart = 0;
 %global _studyend;
 %let _studyend = 0;
 %global _eoftype;
 %let _eoftype = 0;
 %global _studyid;
 %let _studyid = 0;
 %global _unitdays;
 %let _unitdays = 0;
 %global _harvardformat;
 %let _harvardformat = 0;
 %global _baseline;
 %let _baseline = 0;

 data _null_;
  set _cohort;
  if _n_=1;
  call symput("_cohort",trim(left(_sas_data_name)));
 run;

 data _null_;
  set _outdata;
  if _n_=1;
  call symput("_outdata",trim(left(_sas_data_name)));
 run;

 data _null_;
  set _studystart;
  if _n_=1;
  call symput("_studystart",trim(left(_sas_date_var)));
 run;

 data _null_;
  set _studyend;
  if _n_=1;
  call symput("_studyend",trim(left(_sas_date_var)));
 run;

 data _null_;
  set _eoftype;
  if _n_=1;
  call symput("_eoftype",trim(left(_variable)));
  call symput("_deathvalue",cat("'",trim(left(_value)),"'"));
 run;

 data _null_;
  set _studyid;
  if _n_=1;
  call symput("_ident",trim(left(_variable)));
 run;

 data _null_;
  set _unitdays;
  if _n_=1;
  call symput("_unitdays",trim(left(_value)));
 run;

 data _null_;
  set _harvardformat;
  if _n_=1;
  call symput("_harvardformat",trim(left(_value)));
 run;

 data _null_;
  set _baseline;
  if _n_=1;
  call symput("_covariate",trim(left(_value)));
 run;

* create work dataset with study start and end data, one record per person;

 proc sort data=&_cohort out=_work_base_onerecord nodupkey;
  by &_ident;
 run;


*error checking in user input files and parameters;

%_bad_inputs_checking(&_varsdata);

 proc sort data=_error_checks out=_error_detected;
  by descending _error_flag;
 run;


 data _error_detected;
  set _error_detected;
  if _n_ = 1;
  if _error_flag = 1 then do;
   call symput("_error_halt",1);
  end;
  else do;
   call symput("_error_halt",0);
  end;
 run;

 %if &_error_halt = 1 %then %do;
  %put 'MACRO ERROR DETECTED: PROGRAM HAS DETECTED ERRORS IN INPUT DATASETS OR PARAMETERS';
  %put 'REFER TO OUTPUT FILE FOR LIST OF ERROR CHECKS';
  %goto exit; 
 %end;


* preparing intervals and exposure/outcome dates;
 
 %_crtfixint(_work_base_onerecord,&_studystart,&_studyend,&_ident,&_unitdays);

 %_exposureint(&_cohort,&_studystart,&_studyend,&_eoftype,&_deathvalue,&_ident,&_varsdata);

 %if &unittest= exposureint %then %do;
 proc sort data=_workdata;
  by &_ident _intnum;
 run;

  data &_outdata;
  set _workdata;
 run;
 %goto exit;
%end;


*apply logic for assigning exposure/outcome values in intervals;
 
 %if &_nobs_outcome = 1 %then %do;
  %if &_nobs_exposure = 1 and &_ebehavior1 = 2 %then %do;
   %_expos_case2_binary(_workdata,&_ename1,&_ename1._end, &_oname,&_studystart,&_studyend,&_ident,&_unitdays,&_epercent1);
  %end;
  %else %do;
   %put 'MACRO ERROR DETECTED: Only models with one binary exposure are currently supported';
   %goto exit;
  %end;
 %end;
 
 %else %do;
  %put 'MACRO ERROR DETECTED: Only one outcome may be specified';
  %goto exit;
 %end;

%if &unittest= expos_case2_binary %then %do;
 proc sort data=_workdata;
  by &_ident _intnum;
 run;

 data &_outdata;
  set _workdata;
  format _last_int_begin mmddyy10.;
 run;
 %goto exit;
%end;

%if  &unittest= confounderint or  &unittest=conf_case2_binary or &unittest=imput_lvcf %then %do;
 %_confounderint(&_cohort,&_ident,&_studystart,&_studyend,_out_&_oname._date,_exp_&_ename1,&_varsdata,&unittest);  
 %goto exit;
%end;

%else %do;
 * apply logic for assigning confounder values in intervals;
 %_confounderint(&_cohort,&_ident,&_studystart,&_studyend,_out_&_oname._date,_exp_&_ename1,&_varsdata);  


%if  &unittest= msmstructure %then %do;
%goto exit_harvfmt;
%end;

* calculate and output automatic summary statistics;
%_basetable_call_msm(_workdata,&_varsdata,&_ident,&_eoftype,
overall,maximum_interval,exposed_interval);


* roll up last records for use in MSM analysis macro;

%exit_harvfmt:

%if &_harvardformat = 1 %then %do;
  %_harvardformat(_workdata,&_ident);
%end;
 
 %_clean_up(_workdata,_workdata,,&_ident); 
 
* reassign work dataset to output dataset;

 data &_outdata;
  set _workdata;
 run;

* delete work datasets;
 proc datasets library = work nolist;
  delete _workdata _work_base_onerecord;
 quit;
 run;

* undo output of log to external file if on PC SAS;
 %if &sysscp = WIN %then %do;
  proc printto;
  run;
 
 %end;
%end;
%exit: %mend _msmstructure;
