/***********************************************************************************************

Authors: Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer
Maintainer: Romain Neugebauer (romain.s.neugebauer@kp.org)

Copyright 2016 Thomas K Leong, Grace H Tabada, Jingrong Yang, Zheng Zhu, Romain Neugebauer

This file is part of %_MSMstructure.

%_MSMstructure is free software: you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation, either version 3
of the License, or (at your option) any later version.

%_MSMstructure is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for  more details.
 
You should have received a copy of the GNU General Public License along with %_MSMstructure.  
If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************************/


%macro _mtanyobs(data=_last_);
     %* This macro returns 1 if there are any observations in a data set,
        0 if the data set is empty, or . if the data set does not exist or
        cannot be opened.
        
        - The macro first opens the data set. An error message is displayed
          and processing stops if the dataset cannot be opened.
        - It next checks the values of the data set attributes
          ANOBS (does SAS know how many observations there are?) and
          WHSTMT (is a where statement in effect?).
        - If SAS knows the number of observations and there is no
          where clause, the data set attribute ANY is used to determine
          whether there are any observations.
        - If SAS does not know the number of observations (a view or transport
          data set) or if a where clause is in effect, the macro tries to read
          the first observation. If a record is found, the macro returns 1,
          otherwise it returns 0.
         
        This macro requires the data set information functions,
        which are available in SAS version 6.09 and greater. ;

     %* By Jack Hamilton, First Health, January 2001. ;
     %* ADAPTED FOR USE BY THOMAS LEONG, 2/11/2014;

 %global hasobs;
 %local dsid anobs whstmt rc;
 %let DSID = %sysfunc(open(&DATA., IS));

 %if &DSID = 0 %then %do;
  %put %sysfunc(sysmsg());
  %let hasobs = .;
  %goto mexit;
 %end;
 %else %do;
  %let anobs = %sysfunc(attrn(&DSID, ANOBS));
  %let whstmt = %sysfunc(attrn(&DSID, WHSTMT));
 %end;
 
 %if &anobs=1 & &whstmt = 0 %then %do;
  %let hasobs = %sysfunc(attrn(&DSID, ANY));
  %if &hasobs = -1 %then %let hasobs = 0;
 %end;

 %else %do;
  %if %sysfunc(getoption(msglevel)) = I %then %put INFO: First observation in "&DATA." must be fetched.;
  %let hasobs = 0;
  %if %sysfunc(fetch(&DSID)) = 0 %then %let hasobs = 1;
 %end;

 %let rc = %sysfunc(close(&DSID));
 
 %MEXIT:
%mend _mtanyobs;
